@@ -1,30 +1,33 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
-import { div, input } from "react-dom-factories";
-import PropTypes from "prop-types";
+import {
+  div,
+  input,
+  label,
+} from "devtools/client/shared/vendor/react-dom-factories";
+import PropTypes from "devtools/client/shared/vendor/react-prop-types";
 
 export default function ExceptionOption({
   className,
   isChecked = false,
-  label,
+  label: inputLabel,
   onChange,
 }) {
-  return div(
+  return label(
     {
       className,
-      onClick: onChange,
     },
     input({
       type: "checkbox",
       checked: isChecked,
-      onChange: e => e.stopPropagation() && onChange(),
+      onChange,
     }),
     div(
       {
         className: "breakpoint-exceptions-label",
       },
-      label
+      inputLabel
     )
   );
 }
