@@ -136,9 +136,9 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
   },
 
   /**
-   * This function makes sure that the "devtoolstheme" attribute is set on the browser
-   * window to make it possible to change colors on elements in the browser (like the
-   * splitter between the toolbox and web content).
+   * This function makes sure that the "devtoolstheme" attribute is set on the
+   * browser window to make it possible to change colors on elements in the
+   * browser (like the splitter between the toolbox and web content).
    */
   updateDevtoolsThemeAttribute(win) {
     // Set an attribute on root element of each window to make it possible
@@ -147,13 +147,7 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
     if (devtoolsTheme != "dark") {
       devtoolsTheme = "light";
     }
-
-    // Style the splitter between the toolbox and page content.  This used to
-    // set the attribute on the browser's root node but that regressed tpaint:
-    // bug 1331449.
-    win.document
-      .getElementById("appcontent")
-      .setAttribute("devtoolstheme", devtoolsTheme);
+    win.document.documentElement.setAttribute("devtoolstheme", devtoolsTheme);
   },
 
   observe(subject, topic, prefName) {
@@ -258,7 +252,7 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
    *         - `id` used to identify any other key shortcuts like about:debugging
    * @param {Number} startTime
    *        Optional, indicates the time at which the key event fired. This is a
-   *        `Cu.now()` timing.
+   *        `ChromeUtils.now()` timing.
    */
   async onKeyShortcut(window, key, startTime) {
     // Avoid to open devtools when the about:devtools-toolbox page is showing
@@ -304,11 +298,7 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
         if (!toolbox) {
           break;
         }
-        const dbg = await toolbox.getPanel("jsdebugger");
-        if (!dbg) {
-          break;
-        }
-        dbg.toggleJavascriptTracing();
+        await toolbox.commands.tracerCommand.toggle();
         break;
     }
   },
@@ -354,8 +344,11 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
     if (gDevToolsBrowser._trackedBrowserWindows.has(win)) {
       return;
     }
+    if (!win.document.getElementById("menuWebDeveloperPopup")) {
+      // Menus etc. set up here are browser specific.
+      return;
+    }
     gDevToolsBrowser._trackedBrowserWindows.add(win);
-
     BrowserMenus.addMenus(win.document);
 
     this.updateCommandAvailability(win);
