@@ -5,42 +5,65 @@
 "use strict";
 
 const {
+  ADD_REQUEST,
   ADD_TIMING_MARKER,
   CLEAR_TIMING_MARKERS,
   CLEAR_REQUESTS,
-} = require("../constants");
+} = require("resource://devtools/client/netmonitor/src/constants.js");
 
 function TimingMarkers() {
   return {
     firstDocumentDOMContentLoadedTimestamp: -1,
     firstDocumentLoadTimestamp: -1,
+    firstDocumentRequestStartTimestamp: +Infinity,
   };
 }
 
-function addTimingMarker(state, action) {
-  state = { ...state };
-
-  if (action.marker.name === "document::DOMContentLoaded" &&
-      state.firstDocumentDOMContentLoadedTimestamp === -1) {
-    state.firstDocumentDOMContentLoadedTimestamp = action.marker.unixTime / 1000;
-    return state;
-  }
-
-  if (action.marker.name === "document::Load" &&
-      state.firstDocumentLoadTimestamp === -1) {
-    state.firstDocumentLoadTimestamp = action.marker.unixTime / 1000;
-    return state;
+function addRequest(state, action) {
+  const { startedMs } = action.data;
+  if (startedMs < state.firstDocumentRequestStartTimestamp) {
+    return {
+      ...state,
+      firstDocumentRequestStartTimestamp: startedMs,
+    };
   }
 
   return state;
 }
 
-function clearTimingMarkers(state) {
+function addTimingMarker(state, action) {
+  if (
+    action.marker.name === "dom-interactive" &&
+    state.firstDocumentDOMContentLoadedTimestamp === -1
+  ) {
+    return {
+      ...state,
+      firstDocumentDOMContentLoadedTimestamp: action.marker.time,
+    };
+  }
+
+  if (
+    action.marker.name === "dom-complete" &&
+    state.firstDocumentLoadTimestamp === -1
+  ) {
+    return {
+      ...state,
+      firstDocumentLoadTimestamp: action.marker.time,
+    };
+  }
+
+  return state;
+}
+
+function clearTimingMarkers() {
   return new TimingMarkers();
 }
 
 function timingMarkers(state = new TimingMarkers(), action) {
   switch (action.type) {
+    case ADD_REQUEST:
+      return addRequest(state, action);
+
     case ADD_TIMING_MARKER:
       return addTimingMarker(state, action);
 
@@ -55,5 +78,5 @@ function timingMarkers(state = new TimingMarkers(), action) {
 
 module.exports = {
   TimingMarkers,
-  timingMarkers
+  timingMarkers,
 };
