@@ -125,7 +125,13 @@ const OPEN_REQUEST_IN_TAB_URL = EXAMPLE_URL + "html_open-request-in-tab.html";
 const CSP_URL = EXAMPLE_URL + "html_csp-test-page.html";
 const CSP_RESEND_URL = EXAMPLE_URL + "html_csp-resend-test-page.html";
 const IMAGE_CACHE_URL = HTTPS_EXAMPLE_URL + "html_image-cache.html";
+const STYLESHEET_CACHE_URL = HTTPS_EXAMPLE_URL + "html_stylesheet-cache.html";
+const SCRIPT_CACHE_URL = HTTPS_EXAMPLE_URL + "html_script-cache.html";
+const MODULE_SCRIPT_CACHE_URL =
+  HTTPS_EXAMPLE_URL + "html_module-script-cache.html";
 const SLOW_REQUESTS_URL = EXAMPLE_URL + "html_slow-requests-test-page.html";
+const HTTPS_SLOW_REQUESTS_URL =
+  HTTPS_EXAMPLE_URL + "html_slow-requests-test-page.html";
 
 const SIMPLE_SJS = EXAMPLE_URL + "sjs_simple-test-server.sjs";
 const HTTPS_SIMPLE_SJS = HTTPS_EXAMPLE_URL + "sjs_simple-test-server.sjs";
@@ -146,6 +152,7 @@ const CORS_SJS_PATH =
 const HSTS_SJS = EXAMPLE_URL + "sjs_hsts-test-server.sjs";
 const METHOD_SJS = EXAMPLE_URL + "sjs_method-test-server.sjs";
 const HTTPS_SLOW_SJS = HTTPS_EXAMPLE_URL + "sjs_slow-test-server.sjs";
+const DELAY_SJS = HTTPS_EXAMPLE_URL + "sjs_delay-test-server.sjs";
 const SET_COOKIE_SAME_SITE_SJS = EXAMPLE_URL + "sjs_set-cookie-same-site.sjs";
 const SEARCH_SJS = EXAMPLE_URL + "sjs_search-test-server.sjs";
 const HTTPS_SEARCH_SJS = HTTPS_EXAMPLE_URL + "sjs_search-test-server.sjs";
@@ -180,6 +187,9 @@ Services.prefs.setBoolPref("devtools.debugger.log", false);
 const gDefaultFilters = Services.prefs.getCharPref(
   "devtools.netmonitor.filters"
 );
+const gDefaultRequestFilter = Services.prefs.getCharPref(
+  "devtools.netmonitor.requestfilter"
+);
 
 // Reveal many columns for test
 Services.prefs.setCharPref(
@@ -192,7 +202,8 @@ Services.prefs.setCharPref(
 
 Services.prefs.setCharPref(
   "devtools.netmonitor.columnsData",
-  '[{"name":"status","minWidth":30,"width":5},' +
+  '[{"name":"override","minWidth":20,"width":2},' +
+    '{"name":"status","minWidth":30,"width":5},' +
     '{"name":"method","minWidth":30,"width":5},' +
     '{"name":"domain","minWidth":30,"width":10},' +
     '{"name":"file","minWidth":30,"width":25},' +
@@ -206,12 +217,6 @@ Services.prefs.setCharPref(
 
 registerCleanupFunction(() => {
   info("finish() was called, cleaning up...");
-
-  Services.prefs.setBoolPref("devtools.debugger.log", gEnableLogging);
-  Services.prefs.setCharPref("devtools.netmonitor.filters", gDefaultFilters);
-  Services.prefs.clearUserPref("devtools.cache.disabled");
-  Services.prefs.clearUserPref("devtools.netmonitor.columnsData");
-  Services.prefs.clearUserPref("devtools.netmonitor.visibleColumns");
   Services.cookies.removeAll();
 });
 
@@ -232,6 +237,23 @@ async function disableCacheAndReload(toolbox, waitForLoad) {
   }
 }
 
+async function enableCacheAndReload(toolbox, waitForLoad) {
+  // Disable the cache for any toolbox that it is opened from this point on.
+  Services.prefs.setBoolPref("devtools.cache.disabled", false);
+
+  await toolbox.commands.targetConfigurationCommand.updateConfiguration({
+    cacheDisabled: false,
+  });
+
+  // If the page which is reloaded is not found, this will likely cause
+  // reloadTopLevelTarget to not return so let not wait for it.
+  if (waitForLoad) {
+    await toolbox.commands.targetCommand.reloadTopLevelTarget();
+  } else {
+    toolbox.commands.targetCommand.reloadTopLevelTarget();
+  }
+}
+
 /**
  * Wait for 2 markers during document load.
  */
@@ -278,7 +300,7 @@ const updatedTypes = [
   "NetMonitor:NetworkEventUpdated:EventTimings",
 ];
 
-// Start collecting all networkEventUpdate event when panel is opened.
+// Start collecting all networkEventUpdate events when the panel is opened.
 // removeTab() should be called once all corresponded RECEIVED_* events finished.
 function startNetworkEventUpdateObserver(panelWin) {
   updatingTypes.forEach(type =>
@@ -321,6 +343,7 @@ function initNetMonitor(
     expectedEventTimings,
     waitForLoad = true,
     enableCache = false,
+    openInPrivateWindow = false,
   }
 ) {
   info("Initializing a network monitor pane.");
@@ -333,15 +356,22 @@ function initNetMonitor(
   }
 
   return (async function () {
-    await SpecialPowers.pushPrefEnv({
-      set: [
-        // Capture all stacks so that the timing of devtools opening
-        // doesn't affect the stack trace results.
-        ["javascript.options.asyncstack_capture_debuggee_only", false],
-      ],
-    });
+    let tab = null;
+    let privateWindow = null;
+
+    if (openInPrivateWindow) {
+      privateWindow = await BrowserTestUtils.openNewBrowserWindow({
+        private: true,
+      });
+      ok(
+        PrivateBrowsingUtils.isContentWindowPrivate(privateWindow),
+        "window is private"
+      );
+      tab = BrowserTestUtils.addTab(privateWindow.gBrowser, url);
+    } else {
+      tab = await addTab(url, { waitForLoad });
+    }
 
-    const tab = await addTab(url, { waitForLoad });
     info("Net tab added successfully: " + url);
 
     const toolbox = await gDevTools.showToolboxForTab(tab, {
@@ -369,9 +399,25 @@ function initNetMonitor(
       await disableCacheAndReload(toolbox, waitForLoad);
       await Promise.all(allComplete);
       await clearNetworkEvents(monitor);
+    } else if (Services.prefs.getBoolPref("devtools.cache.disabled")) {
+      info("Enabling cache and reloading page.");
+
+      const allComplete = [];
+      allComplete.push(
+        waitForNetworkEvents(monitor, requestCount, {
+          expectedEventTimings,
+        })
+      );
+
+      if (waitForLoad) {
+        allComplete.push(waitForTimelineMarkers(monitor));
+      }
+      await enableCacheAndReload(toolbox, waitForLoad);
+      await Promise.all(allComplete);
+      await clearNetworkEvents(monitor);
     }
 
-    return { tab, monitor, toolbox };
+    return { tab, monitor, toolbox, privateWindow };
   })();
 }
 
@@ -405,10 +451,10 @@ async function clearNetworkEvents(monitor) {
   await waitForAllNetworkUpdateEvents();
 
   info("Clearing the network requests in the UI");
-  store.dispatch(Actions.clearRequests());
+  store.dispatch(Actions.clearRequests({ isExplicitClear: true }));
 }
 
-function teardown(monitor) {
+function teardown(monitor, privateWindow) {
   info("Destroying the specified network monitor.");
 
   return (async function () {
@@ -419,6 +465,12 @@ function teardown(monitor) {
 
     await monitor.toolbox.destroy();
     await removeTab(tab);
+
+    if (privateWindow) {
+      const closed = BrowserTestUtils.windowClosed(privateWindow);
+      privateWindow.BrowserCommands.tryToCloseWindow();
+      await closed;
+    }
   })();
 }
 
@@ -437,20 +489,26 @@ function waitForNetworkEvents(monitor, getRequests, options = {}) {
   return new Promise(resolve => {
     const panel = monitor.panelWin;
     let networkEvent = 0;
-    let nonBlockedNetworkEvent = 0;
     let payloadReady = 0;
     let eventTimings = 0;
 
+    // Use a set to monitor blocked events, because a network resource might
+    // only receive its blockedReason in onPayloadReady.
+    let nonBlockedNetworkEvents = new Set();
+
     function onNetworkEvent(resource) {
       networkEvent++;
       if (!resource.blockedReason) {
-        nonBlockedNetworkEvent++;
+        nonBlockedNetworkEvents.add(resource.actor);
       }
       maybeResolve(TEST_EVENTS.NETWORK_EVENT, resource.actor);
     }
 
     function onPayloadReady(resource) {
       payloadReady++;
+      if (resource.blockedReason) {
+        nonBlockedNetworkEvents.delete(resource.actor);
+      }
       maybeResolve(EVENTS.PAYLOAD_READY, resource.actor);
     }
 
@@ -462,7 +520,7 @@ function waitForNetworkEvents(monitor, getRequests, options = {}) {
     function onClearNetworkResources() {
       // Reset all counters.
       networkEvent = 0;
-      nonBlockedNetworkEvent = 0;
+      nonBlockedNetworkEvents = new Set();
       payloadReady = 0;
       eventTimings = 0;
     }
@@ -474,7 +532,7 @@ function waitForNetworkEvents(monitor, getRequests, options = {}) {
       // * hidden in background,
       // * for any blocked request,
       let expectedEventTimings =
-        document.visibilityState == "hidden" ? 0 : nonBlockedNetworkEvent;
+        document.visibilityState == "hidden" ? 0 : nonBlockedNetworkEvents.size;
       let expectedPayloadReady = getRequests;
       // Typically ignore this option if it is undefined or null
       if (typeof options?.expectedEventTimings == "number") {
@@ -526,7 +584,7 @@ function waitForNetworkEvents(monitor, getRequests, options = {}) {
   });
 }
 
-function verifyRequestItemTarget(
+async function verifyRequestItemTarget(
   document,
   requestList,
   requestItem,
@@ -581,8 +639,15 @@ function verifyRequestItemTarget(
   } = requestItem;
   const formattedIPPort = getFormattedIPAndPort(remoteAddress, remotePort);
   const remoteIP = remoteAddress ? `${formattedIPPort}` : "unknown";
-  const duration = getFormattedTime(totalTime);
-  const latency = getFormattedTime(eventTimings.timings.wait);
+  // TODO Bug 1959359: timing columns duration and latency use a custom formatting for now for undefined/NaN values
+  const duration =
+    totalTime === undefined || isNaN(totalTime)
+      ? ""
+      : getFormattedTime(totalTime);
+  const latency =
+    eventTimings.timings.wait === undefined || isNaN(eventTimings.timings.wait)
+      ? ""
+      : getFormattedTime(eventTimings.timings.wait);
   const protocol = getFormattedProtocol(requestItem);
 
   if (fuzzyUrl) {
@@ -704,25 +769,38 @@ function verifyRequestItemTarget(
   );
 
   if (status !== undefined) {
+    info("Wait for the request status to be updated");
+    await waitFor(
+      () =>
+        target.querySelector(".requests-list-status-code").textContent == status
+    );
+
     const value = target
       .querySelector(".requests-list-status-code")
       .getAttribute("data-status-code");
-    const codeValue = target.querySelector(
-      ".requests-list-status-code"
-    ).textContent;
-    const tooltip = target
-      .querySelector(".requests-list-status-code")
-      .getAttribute("title");
-    info("Displayed status: " + value);
-    info("Displayed code: " + codeValue);
-    info("Tooltip status: " + tooltip);
     is(
       `${value}`,
       displayedStatus ? `${displayedStatus}` : `${status}`,
-      "The displayed status is correct."
+      `The displayed status "${value}" is correct.`
+    );
+
+    const codeValue = target.querySelector(
+      ".requests-list-status-code"
+    ).textContent;
+    is(
+      `${codeValue}`,
+      `${status}`,
+      `The displayed status code "${codeValue}" is correct.`
+    );
+
+    const tooltip = target
+      .querySelector(".requests-list-status-code")
+      .getAttribute("title");
+    is(
+      tooltip,
+      status + " " + statusText,
+      `The tooltip status "${tooltip}" is correct.`
     );
-    is(`${codeValue}`, `${status}`, "The displayed status code is correct.");
-    is(tooltip, status + " " + statusText, "The tooltip status is correct.");
   }
   if (cause !== undefined) {
     const value = Array.from(
@@ -753,28 +831,45 @@ function verifyRequestItemTarget(
     is(tooltip, fullMimeType, "The tooltip type is correct.");
   }
   if (transferred !== undefined) {
-    const value = target.querySelector(
-      ".requests-list-transferred"
-    ).textContent;
+    let transferedValue;
+    info("Wait for the transfered value to get updated");
+    const trnsOk = await waitFor(() => {
+      transferedValue = target.querySelector(
+        ".requests-list-transferred"
+      ).textContent;
+      return transferedValue == transferred;
+    });
+    ok(
+      trnsOk,
+      `The displayed transferred size "${transferedValue}" is correct.`
+    );
+
     const tooltip = target
       .querySelector(".requests-list-transferred")
       .getAttribute("title");
-    info("Displayed transferred size: " + value);
-    info("Tooltip transferred size: " + tooltip);
-    is(value, transferred, "The displayed transferred size is correct.");
-    is(tooltip, transferred, "The tooltip transferred size is correct.");
+    is(
+      tooltip,
+      transferred,
+      `The tooltip transferred size "${tooltip}" is correct.`
+    );
   }
   if (size !== undefined) {
-    const value = target.querySelector(".requests-list-size").textContent;
+    let sizeValue;
+    info("Wait for the size to get updated");
+    const sizeOk = await waitFor(() => {
+      sizeValue = target.querySelector(".requests-list-size").textContent;
+      return sizeValue == size;
+    });
+    ok(sizeOk, `The displayed size "${sizeValue}" is correct.`);
+
     const tooltip = target
       .querySelector(".requests-list-size")
       .getAttribute("title");
-    info("Displayed size: " + value);
-    info("Tooltip size: " + tooltip);
-    is(value, size, "The displayed size is correct.");
-    is(tooltip, size, "The tooltip size is correct.");
+    is(tooltip, size, `The tooltip size "${tooltip}" is correct.`);
   }
   if (time !== undefined) {
+    info("Wait for timings total to get updated");
+    await waitFor(() => target.querySelector(".requests-list-timings-total"));
     const value = target.querySelector(
       ".requests-list-timings-total"
     ).textContent;
@@ -783,8 +878,16 @@ function verifyRequestItemTarget(
       .getAttribute("title");
     info("Displayed time: " + value);
     info("Tooltip time: " + tooltip);
-    ok(~~value.match(/[0-9]+/) >= 0, "The displayed time is correct.");
-    ok(~~tooltip.match(/[0-9]+/) >= 0, "The tooltip time is correct.");
+    Assert.greaterOrEqual(
+      ~~value.match(/[0-9]+/),
+      0,
+      "The displayed time is correct."
+    );
+    Assert.greaterOrEqual(
+      ~~tooltip.match(/[0-9]+/),
+      0,
+      "The tooltip time is correct."
+    );
   }
 
   if (visibleIndex !== -1) {
@@ -870,7 +973,7 @@ function testFilterButtonsCustom(monitor, isChecked) {
  *
  */
 function promiseXHR(data) {
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     const xhr = new content.XMLHttpRequest();
 
     const method = data.method || "GET";
@@ -883,7 +986,7 @@ function promiseXHR(data) {
 
     xhr.addEventListener(
       "loadend",
-      function (event) {
+      function () {
         resolve({ status: xhr.status, response: xhr.response });
       },
       { once: true }
@@ -917,7 +1020,7 @@ function promiseXHR(data) {
  *
  */
 function promiseWS(data) {
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     let url = data.url;
 
     if (data.nocache) {
@@ -928,7 +1031,7 @@ function promiseWS(data) {
     const socket = new content.WebSocket(url);
 
     /* Since we only use HTTP server to mock websocket, so just ignore the error */
-    socket.onclose = e => {
+    socket.onclose = () => {
       socket.close();
       resolve({
         status: 101,
@@ -936,7 +1039,7 @@ function promiseWS(data) {
       });
     };
 
-    socket.onerror = e => {
+    socket.onerror = () => {
       socket.close();
       resolve({
         status: 101,
@@ -1066,7 +1169,9 @@ async function selectIndexAndWaitForSourceEditor(monitor, index) {
 /**
  * Helper function for executing XHRs on a test page.
  *
- * @param {Number} count Number of requests to be executed.
+ * @param {Object} monitor
+ * @param {Object} tab - The current browser tab
+ * @param {Number} count - Number of requests to be executed.
  */
 async function performRequests(monitor, tab, count) {
   const wait = waitForNetworkEvents(monitor, count);
@@ -1081,7 +1186,7 @@ async function performRequests(monitor, tab, count) {
  */
 function getCodeMirrorValue(monitor) {
   const { document } = monitor.panelWin;
-  return document.querySelector(".CodeMirror").CodeMirror.getValue();
+  return document.querySelector(".CodeMirror")?.CodeMirror.getValue();
 }
 
 /**
@@ -1110,27 +1215,28 @@ function getSettingsMenuItem(monitor, itemKey) {
 /**
  * Wait for lazy fields to be loaded in a request.
  *
- * @param Object Store redux store containing request list.
- * @param array fields array of strings which contain field names to be checked
- * on the request.
+ * @param {Object} Store - redux store containing request list.
+ * @param {Array} fields - array of strings which contain field names to be checked
+ * @param {Number} id - The id of the request whose data we need to wait for
+ * @param {Number} index - The position of the request in the sorted request list.
  */
-function waitForRequestData(store, fields, id) {
+function waitForRequestData(store, fields, id, index = 0) {
   return waitUntil(() => {
     let item;
     if (id) {
       item = getRequestById(store.getState(), id);
     } else {
-      item = getSortedRequests(store.getState())[0];
+      item = getSortedRequests(store.getState())[index];
     }
     if (!item) {
       return false;
     }
     for (const field of fields) {
-      if (!item[field]) {
+      if (item[field] == undefined) {
         return false;
       }
     }
-    return true;
+    return item;
   });
 }
 
@@ -1148,7 +1254,11 @@ function checkTelemetryEvent(expectedEvent, query) {
   is(events.length, 1, "There was only 1 event logged");
 
   const [event] = events;
-  ok(event.session_id > 0, "There is a valid session_id in the logged event");
+  Assert.greater(
+    Number(event.session_id),
+    0,
+    "There is a valid session_id in the logged event"
+  );
 
   const f = e => JSON.stringify(e, null, 2);
   is(
@@ -1187,7 +1297,7 @@ function queryTelemetryEvents(query) {
  *     When set to true, requests are allowed to be in a different order in the
  *     netmonitor than in the expected requests array. Defaults to false.
  */
-function validateRequests(requests, monitor, options = {}) {
+async function validateRequests(requests, monitor, options = {}) {
   const { allowDifferentOrder } = options;
   const { document, store, windowRequire } = monitor.panelWin;
 
@@ -1196,7 +1306,7 @@ function validateRequests(requests, monitor, options = {}) {
   );
   const sortedRequests = getSortedRequests(store.getState());
 
-  requests.forEach((spec, i) => {
+  for (const [i, spec] of requests.entries()) {
     const { method, url, causeType, causeUri, stack } = spec;
 
     let requestItem;
@@ -1206,7 +1316,7 @@ function validateRequests(requests, monitor, options = {}) {
       requestItem = sortedRequests[i];
     }
 
-    verifyRequestItemTarget(
+    await verifyRequestItemTarget(
       document,
       getDisplayedRequests(store.getState()),
       requestItem,
@@ -1220,102 +1330,78 @@ function validateRequests(requests, monitor, options = {}) {
 
     if (stack) {
       ok(stacktrace, `Request #${i} has a stacktrace`);
-      ok(
-        stackLen > 0,
+      Assert.greater(
+        stackLen,
+        0,
         `Request #${i} (${causeType}) has a stacktrace with ${stackLen} items`
       );
 
       // if "stack" is array, check the details about the top stack frames
       if (Array.isArray(stack)) {
         stack.forEach((frame, j) => {
-          // If the `fn` is "*", it means the request is triggered from chrome
-          // resources, e.g. `resource:///modules/XX.jsm`, so we skip checking
-          // the function name for now (bug 1280266).
-          if (frame.file.startsWith("resource:///")) {
-            todo(false, "Requests from chrome resource should not be included");
-          } else {
-            let value = stacktrace[j].functionName;
-            if (Object.is(value, null)) {
-              value = undefined;
-            }
-            is(
-              value,
-              frame.fn,
-              `Request #${i} has the correct function on JS stack frame #${j}`
-            );
-            is(
-              stacktrace[j].filename.split("/").pop(),
-              frame.file.split("/").pop(),
-              `Request #${i} has the correct file on JS stack frame #${j}`
-            );
-            is(
-              stacktrace[j].lineNumber,
-              frame.line,
-              `Request #${i} has the correct line number on JS stack frame #${j}`
-            );
-            value = stacktrace[j].asyncCause;
-            if (Object.is(value, null)) {
-              value = undefined;
-            }
-            is(
-              value,
-              frame.asyncCause,
-              `Request #${i} has the correct async cause on JS stack frame #${j}`
-            );
+          let value = stacktrace[j].functionName;
+          if (Object.is(value, null)) {
+            value = undefined;
           }
+          is(
+            value,
+            frame.fn,
+            `Request #${i} has the correct function on JS stack frame #${j}`
+          );
+          is(
+            stacktrace[j].filename.split("/").pop(),
+            frame.file.split("/").pop(),
+            `Request #${i} has the correct file on JS stack frame #${j}`
+          );
+          is(
+            stacktrace[j].lineNumber,
+            frame.line,
+            `Request #${i} has the correct line number on JS stack frame #${j}`
+          );
+          value = stacktrace[j].asyncCause;
+          if (Object.is(value, null)) {
+            value = undefined;
+          }
+          is(
+            value,
+            frame.asyncCause,
+            `Request #${i} has the correct async cause on JS stack frame #${j}`
+          );
         });
       }
     } else {
       is(stackLen, 0, `Request #${i} (${causeType}) has an empty stacktrace`);
     }
-  });
+  }
 }
 
 /**
- * Retrieve the context menu element corresponding to the provided id, for the provided
- * netmonitor instance.
- * @param {Object} monitor
- *        The network monnitor object
- * @param {String} id
- *        The id of the context menu item
+ * @see getNetmonitorContextMenuItem in shared-head.js
  */
 function getContextMenuItem(monitor, id) {
-  const Menu = require("resource://devtools/client/framework/menu.js");
-  return Menu.getMenuElementById(id, monitor.panelWin.document);
+  return getNetmonitorContextMenuItem(monitor, id);
 }
 
-async function maybeOpenAncestorMenu(menuItem) {
-  const parentPopup = menuItem.parentNode;
-  if (parentPopup.state == "shown") {
+/**
+ * Hides the provided netmonitor context menu
+ *
+ * @param {XULPopupElement} popup
+ *        The popup to hide.
+ */
+async function hideContextMenu(popup) {
+  if (popup.state !== "open") {
     return;
   }
-  const shown = BrowserTestUtils.waitForEvent(parentPopup, "popupshown");
-  if (parentPopup.state == "showing") {
-    await shown;
-    return;
-  }
-  const parentMenu = parentPopup.parentNode;
-  await maybeOpenAncestorMenu(parentMenu);
-  parentMenu.openMenu(true);
-  await shown;
+  const onPopupHidden = BrowserTestUtils.waitForEvent(popup, "popuphidden");
+  popup.hidePopup();
+  await onPopupHidden;
 }
 
-/*
- * Selects and clicks the context menu item, it should
- * also wait for the popup to close.
- * @param {Object} monitor
- *        The network monnitor object
- * @param {String} id
- *        The id of the context menu item
+/**
+ * @see selectNetmonitorContextMenuItem in shared-head.js
  */
 async function selectContextMenuItem(monitor, id) {
-  const contextMenuItem = getContextMenuItem(monitor, id);
-
-  const popup = contextMenuItem.parentNode;
-  await maybeOpenAncestorMenu(contextMenuItem);
-  const hidden = BrowserTestUtils.waitForEvent(popup, "popuphidden");
-  popup.activateItem(contextMenuItem);
-  await hidden;
+  return selectNetmonitorContextMenuItem(monitor, id);
 }
 
 /**
@@ -1382,7 +1468,7 @@ function clickElement(element, monitor) {
  *        Target browser to observe the favicon load.
  */
 function registerFaviconNotifier(browser) {
-  const listener = async (name, data) => {
+  const listener = async name => {
     if (name == "SetIcon" || name == "SetFailedIcon") {
       await SpecialPowers.spawn(browser, [], async () => {
         content.document
@@ -1516,3 +1602,261 @@ async function toggleUrlPreview(shouldExpand, monitor) {
   );
   return wait;
 }
+
+/**
+ * Wait for the eager evaluated result from the split console
+ * @param {Object} hud
+ * @param {String} text - expected evaluation result
+ */
+async function waitForEagerEvaluationResult(hud, text) {
+  await waitUntil(() => {
+    const elem = hud.ui.outputNode.querySelector(".eager-evaluation-result");
+    if (elem) {
+      if (text instanceof RegExp) {
+        return text.test(elem.innerText);
+      }
+      return elem.innerText == text;
+    }
+    return false;
+  });
+  ok(true, `Got eager evaluation result ${text}`);
+}
+
+/**
+ * Assert the contents of the filter urls autocomplete box
+ *
+ * @param {Array} expected
+ * @param {Object} document
+ */
+function testAutocompleteContents(expected, document) {
+  expected.forEach(function (item, i) {
+    is(
+      document.querySelector(
+        `.devtools-autocomplete-listbox .autocomplete-item:nth-child(${i + 1})`
+      ).textContent,
+      item,
+      `${expected[i]} found`
+    );
+  });
+}
+
+/**
+ * Check if a valid numerical size is displayed in the request column for the
+ * provided request.
+ *
+ * @param {Element} request
+ *     A request element from the netmonitor requests list.
+ * @return {boolean}
+ *     True if the size column contains a valid size, false otherwise.
+ *
+ */
+function hasValidSize(request) {
+  const VALID_SIZE_RE = /^\d+(\.\d+)? \w+/;
+  return VALID_SIZE_RE.test(
+    request.querySelector(".requests-list-size").innerText
+  );
+}
+
+function getThrottleProfileItem(monitor, profileId) {
+  const toolboxDoc = monitor.toolbox.doc;
+
+  const popup = toolboxDoc.querySelector("#network-throttling-menu");
+  const menuItems = [...popup.querySelectorAll(".menuitem > .command")];
+  return menuItems.find(menuItem => menuItem.id == profileId);
+}
+
+async function selectThrottle(monitor, profileId) {
+  const panelDoc = monitor.panelWin.document;
+  const toolboxDoc = monitor.toolbox.doc;
+
+  info("Opening the throttling menu");
+
+  const onShown = BrowserTestUtils.waitForPopupEvent(toolboxDoc, "shown");
+  panelDoc.getElementById("network-throttling").click();
+
+  info("Waiting for the throttling menu to be displayed");
+  await onShown;
+
+  const profileItem = getThrottleProfileItem(monitor, profileId);
+  ok(profileItem, "Found a profile throttling menu item for id " + profileId);
+
+  info(`Selecting the '${profileId}' profile`);
+  profileItem.click();
+
+  info(`Waiting for the '${profileId}' profile to be applied`);
+  await monitor.panelWin.api.once(TEST_EVENTS.THROTTLING_CHANGED);
+}
+
+/**
+ * Resize a netmonitor column.
+ *
+ * @param {Element} columnHeader
+ * @param {number} newPercent
+ * @param {number} parentWidth
+ * @param {string} dir
+ */
+function resizeColumn(columnHeader, newPercent, parentWidth, dir = "ltr") {
+  const newWidthInPixels = (newPercent * parentWidth) / 100;
+  const win = columnHeader.ownerDocument.defaultView;
+  const currentWidth = columnHeader.getBoundingClientRect().width;
+  const mouseDown = dir === "rtl" ? 0 : currentWidth;
+  const mouseMove =
+    dir === "rtl" ? currentWidth - newWidthInPixels : newWidthInPixels;
+
+  EventUtils.synthesizeMouse(
+    columnHeader,
+    mouseDown,
+    1,
+    { type: "mousedown" },
+    win
+  );
+  EventUtils.synthesizeMouse(
+    columnHeader,
+    mouseMove,
+    1,
+    { type: "mousemove" },
+    win
+  );
+  EventUtils.synthesizeMouse(
+    columnHeader,
+    mouseMove,
+    1,
+    { type: "mouseup" },
+    win
+  );
+}
+
+/**
+ * Resize the waterfall netmonitor column.
+ * Uses slightly different logic than for the other columns.
+ *
+ * @param {Element} columnHeader
+ * @param {number} newPercent
+ * @param {number} parentWidth
+ * @param {string} dir
+ */
+function resizeWaterfallColumn(
+  columnHeader,
+  newPercent,
+  parentWidth,
+  dir = "ltr"
+) {
+  const newWidthInPixels = (newPercent * parentWidth) / 100;
+  const win = columnHeader.ownerDocument.defaultView;
+  const mouseDown =
+    dir === "rtl"
+      ? columnHeader.getBoundingClientRect().right
+      : columnHeader.getBoundingClientRect().left;
+  const mouseMove =
+    dir === "rtl"
+      ? mouseDown +
+        (newWidthInPixels - columnHeader.getBoundingClientRect().width)
+      : mouseDown +
+        (columnHeader.getBoundingClientRect().width - newWidthInPixels);
+
+  EventUtils.synthesizeMouse(
+    columnHeader.parentElement,
+    mouseDown,
+    1,
+    { type: "mousedown" },
+    win
+  );
+  EventUtils.synthesizeMouse(
+    columnHeader.parentElement,
+    mouseMove,
+    1,
+    { type: "mousemove" },
+    win
+  );
+  EventUtils.synthesizeMouse(
+    columnHeader.parentElement,
+    mouseMove,
+    1,
+    { type: "mouseup" },
+    win
+  );
+}
+
+function getCurrentVisibleColumns(monitor) {
+  const { store, windowRequire } = monitor.panelWin;
+  const { getColumns, getVisibleColumns, hasOverride } = windowRequire(
+    "devtools/client/netmonitor/src/selectors/index"
+  );
+  const hasOverrideState = hasOverride(monitor.toolbox.store.getState());
+  const visibleColumns = getVisibleColumns(
+    getColumns(store.getState(), hasOverrideState)
+  );
+
+  // getVisibleColumns returns an array of arrays [name, isVisible=true], flatten
+  // to return name.
+  return visibleColumns.map(([name]) => name);
+}
+
+function findRequestByInitiator(document, initiator) {
+  for (const request of document.querySelectorAll(".request-list-item")) {
+    if (
+      request.querySelector(".requests-list-initiator").getAttribute("title") ==
+      initiator
+    ) {
+      return request;
+    }
+  }
+  return null;
+}
+
+/**
+ * Click on the "save response as" context menu item for the provided request
+ * element in the provided netmonitor panel.
+ *
+ * Resolves when the context menu is closed.
+ *
+ * @param {object} monitor
+ *     The netmonitor instance
+ * @param {HTMLElement} request
+ *     The request item in the netmonitor table
+ */
+async function triggerSaveResponseAs(monitor, request) {
+  EventUtils.sendMouseEvent({ type: "mousedown" }, request);
+  EventUtils.sendMouseEvent({ type: "contextmenu" }, request);
+
+  info("Open the save dialog");
+  await selectContextMenuItem(monitor, "request-list-context-save-response-as");
+}
+
+/**
+ * Wait until the provided path has a non-zero size on the file system.
+ *
+ * @param {string} path
+ *     The path to wait for.
+ */
+async function waitForFileSavedToDisk(path) {
+  info("Wait for the downloaded file to be fully saved to disk: " + path);
+  await TestUtils.waitForCondition(async () => {
+    if (!(await IOUtils.exists(path))) {
+      return false;
+    }
+    const { size } = await IOUtils.stat(path);
+    return size > 0;
+  });
+}
+
+/**
+ * Create a temporary directory to save files for a test.
+ * Register a cleanup function to delete the directory after the test.
+ *
+ * @returns {nsIFile}
+ *     The created temporary directory.
+ */
+function createTemporarySaveDirectory() {
+  const saveDir = Services.dirsvc.get("TmpD", Ci.nsIFile);
+  saveDir.append("testsavedir");
+
+  if (!saveDir.exists()) {
+    saveDir.create(Ci.nsIFile.DIRECTORY_TYPE, 0o755);
+  }
+
+  registerCleanupFunction(function () {
+    saveDir.remove(true);
+  });
+  return saveDir;
+}
