@@ -72,33 +72,7 @@ SimpleTest.waitForExplicitFinish();
 // should be enough.
 requestLongerTimeout(2);
 
-// The appearance of this notification causes intermittent behavior in some tests that
-// send mouse events, since it causes the content to shift when it appears.
-Services.prefs.setBoolPref(
-  "devtools.responsive.reloadNotification.enabled",
-  false
-);
-// Don't show the setting onboarding tooltip in the test suites.
-Services.prefs.setBoolPref("devtools.responsive.show-setting-tooltip", false);
-
 registerCleanupFunction(async () => {
-  Services.prefs.clearUserPref(
-    "devtools.responsive.reloadNotification.enabled"
-  );
-  Services.prefs.clearUserPref("devtools.responsive.html.displayedDeviceList");
-  Services.prefs.clearUserPref(
-    "devtools.responsive.reloadConditions.touchSimulation"
-  );
-  Services.prefs.clearUserPref(
-    "devtools.responsive.reloadConditions.userAgent"
-  );
-  Services.prefs.clearUserPref("devtools.responsive.show-setting-tooltip");
-  Services.prefs.clearUserPref("devtools.responsive.showUserAgentInput");
-  Services.prefs.clearUserPref("devtools.responsive.touchSimulation.enabled");
-  Services.prefs.clearUserPref("devtools.responsive.userAgent");
-  Services.prefs.clearUserPref("devtools.responsive.viewport.height");
-  Services.prefs.clearUserPref("devtools.responsive.viewport.pixelRatio");
-  Services.prefs.clearUserPref("devtools.responsive.viewport.width");
   await asyncStorage.removeItem("devtools.responsive.deviceState");
   await removeLocalDevices();
 
@@ -179,7 +153,12 @@ function addRDMTaskWithPreAndPost(url, preTask, task, postTask, options) {
     }
 
     if (!onlyPrefAndTask) {
+      // Close the toolbox first, as closing RDM might trigger a reload if
+      // touch simulation was enabled, which will trigger RDP requests.
+      await closeToolboxIfOpen();
+
       await closeRDM(tab);
+
       if (postTask) {
         await postTask({
           message,
@@ -218,7 +197,11 @@ function addRDMTask(rdmURL, rdmTask, options) {
 
 async function spawnViewportTask(ui, args, task) {
   // Await a reflow after the task.
-  const result = await ContentTask.spawn(ui.getViewportBrowser(), args, task);
+  const result = await SpecialPowers.spawn(
+    ui.getViewportBrowser(),
+    [args],
+    task
+  );
   await promiseContentReflow(ui);
   return result;
 }
@@ -377,17 +360,10 @@ async function testViewportResize(
   expectedHandleMove,
   { hasDevice } = {}
 ) {
-  let deviceRemoved;
-  let waitForDevToolsReload;
-  if (hasDevice) {
-    // If a device was defined, a reload will be triggered by the resize,
-    // wait for devtools to reload completely.
-    waitForDevToolsReload = await watchForDevToolsReload(
-      ui.getViewportBrowser()
-    );
-    // and wait for the device-associaton-removed event.
-    deviceRemoved = once(ui, "device-association-removed");
-  }
+  // If a device was defined, wait for the device-associaton-removed event.
+  const deviceRemoved = hasDevice
+    ? once(ui, "device-association-removed")
+    : null;
 
   const resized = ui.once("viewport-resize-dragend");
   const startRect = dragElementBy(selector, ...moveBy, ui);
@@ -405,12 +381,7 @@ async function testViewportResize(
     `The y move of ${selector} is as expected`
   );
 
-  if (hasDevice) {
-    const { reloadTriggered } = await deviceRemoved;
-    if (reloadTriggered) {
-      await waitForDevToolsReload();
-    }
-  }
+  await deviceRemoved;
 }
 
 async function openDeviceModal(ui) {
@@ -475,10 +446,22 @@ async function testMenuItems(toolWindow, button, testFn) {
     win.document.addEventListener(
       "popupshown",
       async () => {
-        if (button.id === "device-selector") {
-          const popup = toolWindow.document.querySelector(
-            "#device-selector-menu"
-          );
+        // Handle MenuButton popups (device selector and network throttling).
+        if (
+          button.id === "device-selector" ||
+          button.id == "user-agent-selector" ||
+          button.id === "network-throttling"
+        ) {
+          let popupId;
+          if (button.id === "device-selector") {
+            popupId = "#device-selector-menu";
+          } else if (button.id === "user-agent-selector") {
+            popupId = "#user-agent-selector-menu";
+          } else {
+            popupId = "#network-throttling-menu";
+          }
+
+          const popup = toolWindow.document.querySelector(popupId);
           const menuItems = [...popup.querySelectorAll(".menuitem > .command")];
 
           testFn(menuItems);
@@ -526,7 +509,7 @@ const selectDevicePixelRatio = (ui, value) =>
 const selectNetworkThrottling = (ui, value) =>
   Promise.all([
     once(ui, "network-throttling-changed"),
-    selectMenuItem(ui, "#network-throttling-menu", value),
+    selectMenuItem(ui, "#network-throttling", value),
   ]);
 
 function getSessionHistory(browser) {
@@ -641,7 +624,7 @@ async function testDevicePixelRatio(ui, expected) {
   is(dppx, expected, `devicePixelRatio should be set to ${expected}`);
 }
 
-async function testTouchEventsOverride(ui, expected) {
+function testTouchEventsOverride(ui, expected) {
   const { document } = ui.toolWindow;
   const touchButton = document.getElementById("touch-simulation-button");
 
@@ -724,7 +707,11 @@ function testViewportDimensions(ui, w, h) {
   );
 }
 
-async function changeUserAgentInput(ui, value) {
+async function changeUserAgentInput(
+  ui,
+  value,
+  keyPressedAfterChange = "VK_RETURN"
+) {
   const { Simulate } = ui.toolWindow.require(
     "resource://devtools/client/shared/vendor/react-dom-test-utils.js"
   );
@@ -733,17 +720,27 @@ async function changeUserAgentInput(ui, value) {
 
   const userAgentInput = document.getElementById("user-agent-input");
   userAgentInput.value = value;
+  userAgentInput.focus();
   Simulate.change(userAgentInput);
 
-  const userAgentChanged = waitUntilState(
-    store,
-    state => state.ui.userAgent === value
-  );
-  const changed = once(ui, "user-agent-changed");
+  function pressKey() {
+    EventUtils.synthesizeKey(keyPressedAfterChange, {}, ui.toolWindow);
+  }
 
-  const waitForDevToolsReload = await watchForDevToolsReload(browser);
-  Simulate.keyUp(userAgentInput, { keyCode: KeyCodes.DOM_VK_RETURN });
-  await Promise.all([changed, waitForDevToolsReload(), userAgentChanged]);
+  if (keyPressedAfterChange === "VK_ESCAPE") {
+    pressKey();
+  } else {
+    const userAgentChanged = waitUntilState(
+      store,
+      state => state.ui.userAgent === value
+    );
+    const changed = once(ui, "user-agent-changed");
+    const waitForDevToolsReload = await watchForDevToolsReload(browser);
+
+    pressKey();
+
+    await Promise.all([changed, waitForDevToolsReload(), userAgentChanged]);
+  }
 }
 
 /**
