@@ -4,9 +4,24 @@
 /* eslint-env browser */
 "use strict";
 
-const { Component, createFactory } = require("devtools/client/shared/vendor/react");
-const PropTypes = require("devtools/client/shared/vendor/react-prop-types");
-const dom = require("devtools/client/shared/vendor/react-dom-factories");
+const {
+  Component,
+  createFactory,
+} = require("resource://devtools/client/shared/vendor/react.mjs");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
+const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
+const { scrollIntoView } = ChromeUtils.importESModule(
+  "resource://devtools/client/shared/scroll.mjs"
+);
+const {
+  preventDefaultAndStopPropagation,
+} = require("resource://devtools/client/shared/events.js");
+
+const lazy = {};
+ChromeUtils.defineESModuleGetters(lazy, {
+  wrapMoveFocus: "resource://devtools/client/shared/focus.mjs",
+  getFocusableElements: "resource://devtools/client/shared/focus.mjs",
+});
 
 const AUTO_EXPAND_DEPTH = 0;
 const NUMBER_OF_OFFSCREEN_ITEMS = 1;
@@ -195,6 +210,19 @@ class Tree extends Component {
       // Handle when a new item is focused.
       onFocus: PropTypes.func,
 
+      // The currently active (keyboard) item, if any such item exists.
+      active: PropTypes.any,
+
+      // Handle when item is activated with a keyboard (using Space or Enter)
+      onActivate: PropTypes.func,
+
+      // The currently shown item, if any such item exists.
+      shown: PropTypes.any,
+
+      // Indicates if pressing ArrowRight key should only expand expandable node
+      // or if the selection should also move to the next node.
+      preventNavigationOnArrowRight: PropTypes.bool,
+
       // The depth to which we should automatically expand new items.
       autoExpandDepth: PropTypes.number,
 
@@ -225,6 +253,7 @@ class Tree extends Component {
   static get defaultProps() {
     return {
       autoExpandDepth: AUTO_EXPAND_DEPTH,
+      preventNavigationOnArrowRight: true,
     };
   }
 
@@ -235,6 +264,7 @@ class Tree extends Component {
       scroll: 0,
       height: window.innerHeight,
       seen: new Set(),
+      mouseDown: false,
     };
 
     this._onExpand = oncePerAnimationFrame(this._onExpand).bind(this);
@@ -242,31 +272,69 @@ class Tree extends Component {
     this._onScroll = oncePerAnimationFrame(this._onScroll).bind(this);
     this._focusPrevNode = oncePerAnimationFrame(this._focusPrevNode).bind(this);
     this._focusNextNode = oncePerAnimationFrame(this._focusNextNode).bind(this);
-    this._focusParentNode = oncePerAnimationFrame(this._focusParentNode).bind(this);
+    this._focusParentNode = oncePerAnimationFrame(this._focusParentNode).bind(
+      this
+    );
+    this._focusFirstNode = oncePerAnimationFrame(this._focusFirstNode).bind(
+      this
+    );
+    this._focusLastNode = oncePerAnimationFrame(this._focusLastNode).bind(this);
 
     this._autoExpand = this._autoExpand.bind(this);
     this._preventArrowKeyScrolling = this._preventArrowKeyScrolling.bind(this);
     this._updateHeight = this._updateHeight.bind(this);
+    this._onResize = this._onResize.bind(this);
     this._dfs = this._dfs.bind(this);
     this._dfsFromRoots = this._dfsFromRoots.bind(this);
     this._focus = this._focus.bind(this);
-    this._onBlur = this._onBlur.bind(this);
+    this._activate = this._activate.bind(this);
     this._onKeyDown = this._onKeyDown.bind(this);
   }
 
   componentDidMount() {
-    window.addEventListener("resize", this._updateHeight);
+    window.addEventListener("resize", this._onResize);
+    this._autoExpand();
+    this._updateHeight();
+    this._scrollItemIntoView();
+  }
+
+  // FIXME: https://bugzilla.mozilla.org/show_bug.cgi?id=1774507
+  UNSAFE_componentWillReceiveProps(nextProps) {
+    if (nextProps.autoExpandDepth != this.props.autoExpandDepth) {
+      this.setState({ seen: new Set() });
+    }
     this._autoExpand();
     this._updateHeight();
   }
 
-  componentWillReceiveProps(nextProps) {
-    this._autoExpand();
-    this._updateHeight();
+  shouldComponentUpdate(nextProps, nextState) {
+    const { scroll, height, seen, mouseDown } = this.state;
+
+    return (
+      scroll !== nextState.scroll ||
+      height !== nextState.height ||
+      seen !== nextState.seen ||
+      mouseDown === nextState.mouseDown
+    );
+  }
+
+  componentDidUpdate(prevProps) {
+    if (prevProps.shown != this.props.shown) {
+      this._scrollItemIntoView();
+    }
   }
 
   componentWillUnmount() {
-    window.removeEventListener("resize", this._updateHeight);
+    window.removeEventListener("resize", this._onResize);
+  }
+
+  _scrollItemIntoView() {
+    const { shown } = this.props;
+    if (!shown) {
+      return;
+    }
+
+    this._scrollIntoView(shown);
   }
 
   _autoExpand() {
@@ -278,8 +346,10 @@ class Tree extends Component {
     // not use the usual DFS infrastructure because we don't want to ignore
     // collapsed nodes.
     const autoExpand = (item, currentDepth) => {
-      if (currentDepth >= this.props.autoExpandDepth ||
-          this.state.seen.has(item)) {
+      if (
+        currentDepth >= this.props.autoExpandDepth ||
+        this.state.seen.has(item)
+      ) {
         return;
       }
 
@@ -306,16 +376,8 @@ class Tree extends Component {
       case "ArrowDown":
       case "ArrowLeft":
       case "ArrowRight":
-        e.preventDefault();
-        e.stopPropagation();
-        if (e.nativeEvent) {
-          if (e.nativeEvent.preventDefault) {
-            e.nativeEvent.preventDefault();
-          }
-          if (e.nativeEvent.stopPropagation) {
-            e.nativeEvent.stopPropagation();
-          }
-        }
+        preventDefaultAndStopPropagation(e);
+        break;
     }
   }
 
@@ -323,9 +385,7 @@ class Tree extends Component {
    * Updates the state's height based on clientHeight.
    */
   _updateHeight() {
-    this.setState({
-      height: this.refs.tree.clientHeight
-    });
+    this.setState({ height: this.refs.tree.clientHeight });
   }
 
   /**
@@ -399,6 +459,60 @@ class Tree extends Component {
     }
   }
 
+  /**
+   * Scroll item into view. Depending on whether the item is already rendered,
+   * we might have to calculate the position of the item based on its index and
+   * the item height.
+   *
+   * @param {Object} item
+   *        The item to be scrolled into view.
+   * @param {Number|undefined} index
+   *        The index of the item in a full DFS traversal (ignoring collapsed
+   *        nodes) or undefined.
+   * @param {Object} options
+   *        Optional information regarding item's requested alignement when
+   *        scrolling.
+   */
+  _scrollIntoView(item, index, options = {}) {
+    const treeElement = this.refs.tree;
+    if (!treeElement) {
+      return;
+    }
+
+    const element = document.getElementById(this.props.getKey(item));
+    if (element) {
+      scrollIntoView(element, { ...options, container: treeElement });
+      return;
+    }
+
+    if (index == null) {
+      // If index is not provided, determine item index from traversal.
+      const traversal = this._dfsFromRoots();
+      index = traversal.findIndex(({ item: i }) => i === item);
+    }
+
+    if (index == null || index < 0) {
+      return;
+    }
+
+    const { itemHeight } = this.props;
+    const { clientHeight, scrollTop } = treeElement;
+    const elementTop = index * itemHeight;
+    let scrollTo;
+    if (scrollTop >= elementTop + itemHeight) {
+      scrollTo = elementTop;
+    } else if (scrollTop + clientHeight <= elementTop) {
+      scrollTo = elementTop;
+    }
+
+    if (scrollTo != undefined) {
+      treeElement.scrollTo({
+        left: 0,
+        top: scrollTo,
+      });
+    }
+  }
+
   /**
    * Sets the passed in item to be the focused item.
    *
@@ -409,21 +523,15 @@ class Tree extends Component {
    * @param {Object|undefined} item
    *        The item to be focused, or undefined to focus no item.
    */
-  _focus(index, item) {
-    if (item !== undefined) {
-      const itemStartPosition = index * this.props.itemHeight;
-      const itemEndPosition = (index + 1) * this.props.itemHeight;
+  _focus(index, item, options = {}) {
+    if (item !== undefined && !options.preventAutoScroll) {
+      this._scrollIntoView(item, index, options);
+    }
 
-      // Note that if the height of the viewport (this.state.height) is less
-      // than `this.props.itemHeight`, we could accidentally try and scroll both
-      // up and down in a futile attempt to make both the item's start and end
-      // positions visible. Instead, give priority to the start of the item by
-      // checking its position first, and then using an "else if", rather than
-      // a separate "if", for the end position.
-      if (this.state.scroll > itemStartPosition) {
-        this.refs.tree.scrollTo(0, itemStartPosition);
-      } else if ((this.state.scroll + this.state.height) < itemEndPosition) {
-        this.refs.tree.scrollTo(0, itemEndPosition - this.state.height);
+    if (this.props.active != null) {
+      this._activate(null);
+      if (this.refs.tree !== this.activeElement) {
+        this.refs.tree.focus();
       }
     }
 
@@ -432,23 +540,34 @@ class Tree extends Component {
     }
   }
 
+  _activate(item) {
+    if (this.props.onActivate) {
+      this.props.onActivate(item);
+    }
+  }
+
   /**
-   * Sets the state to have no focused item.
+   * Update state height and tree's scrollTop if necessary.
    */
-  _onBlur() {
-    this._focus(0, undefined);
+  _onResize() {
+    // When tree size changes without direct user action, scroll top cat get re-set to 0
+    // (for example, when tree height changes via CSS rule change). We need to ensure that
+    // the tree's scrollTop is in sync with the scroll state.
+    if (this.state.scroll !== this.refs.tree.scrollTop) {
+      this.refs.tree.scrollTo({ left: 0, top: this.state.scroll });
+    }
+
+    this._updateHeight();
   }
 
   /**
    * Fired on a scroll within the tree's container, updates
    * the stored position of the view port to handle virtual view rendering.
-   *
-   * @param {Event} e
    */
-  _onScroll(e) {
+  _onScroll() {
     this.setState({
       scroll: Math.max(this.refs.tree.scrollTop, 0),
-      height: this.refs.tree.clientHeight
+      height: this.refs.tree.clientHeight,
     });
   }
 
@@ -457,6 +576,7 @@ class Tree extends Component {
    *
    * @param {Event} e
    */
+  // eslint-disable-next-line complexity
   _onKeyDown(e) {
     if (this.props.focused == null) {
       return;
@@ -479,8 +599,10 @@ class Tree extends Component {
         break;
 
       case "ArrowLeft":
-        if (this.props.isExpanded(this.props.focused)
-            && this.props.getChildren(this.props.focused).length) {
+        if (
+          this.props.isExpanded(this.props.focused) &&
+          this.props.getChildren(this.props.focused).length
+        ) {
           this._onCollapse(this.props.focused);
         } else {
           this._focusParentNode();
@@ -488,15 +610,64 @@ class Tree extends Component {
         break;
 
       case "ArrowRight":
-        if (!this.props.isExpanded(this.props.focused)) {
+        if (
+          this.props.getChildren(this.props.focused).length &&
+          !this.props.isExpanded(this.props.focused)
+        ) {
           this._onExpand(this.props.focused);
-        } else {
+        } else if (!this.props.preventNavigationOnArrowRight) {
           this._focusNextNode();
         }
         break;
+
+      case "Home":
+        this._focusFirstNode();
+        break;
+
+      case "End":
+        this._focusLastNode();
+        break;
+
+      case "Enter":
+      case " ":
+        // On space or enter make focused tree node active. This means keyboard focus
+        // handling is passed on to the tree node itself.
+        if (this.refs.tree === this.activeElement) {
+          preventDefaultAndStopPropagation(e);
+          if (this.props.active !== this.props.focused) {
+            this._activate(this.props.focused);
+          }
+        }
+        break;
+
+      case "Escape":
+        preventDefaultAndStopPropagation(e);
+        if (this.props.active != null) {
+          this._activate(null);
+        }
+
+        if (this.refs.tree !== this.activeElement) {
+          this.refs.tree.focus();
+        }
+        break;
     }
   }
 
+  get activeElement() {
+    return this.refs.tree.ownerDocument.activeElement;
+  }
+
+  _focusFirstNode() {
+    const traversal = this._dfsFromRoots();
+    this._focus(0, traversal[0].item, { alignTo: "top" });
+  }
+
+  _focusLastNode() {
+    const traversal = this._dfsFromRoots();
+    const lastIndex = traversal.length - 1;
+    this._focus(lastIndex, traversal[lastIndex].item, { alignTo: "bottom" });
+  }
+
   /**
    * Sets the previous node relative to the currently focused item, to focused.
    */
@@ -523,7 +694,7 @@ class Tree extends Component {
       return;
     }
 
-    this._focus(prevIndex, prev);
+    this._focus(prevIndex, prev, { alignTo: "top" });
   }
 
   /**
@@ -547,7 +718,7 @@ class Tree extends Component {
     }
 
     if (i + 1 < traversal.length) {
-      this._focus(i + 1, traversal[i + 1].item);
+      this._focus(i + 1, traversal[i + 1].item, { alignTo: "bottom" });
     }
   }
 
@@ -570,7 +741,7 @@ class Tree extends Component {
       }
     }
 
-    this._focus(parentIndex, parent);
+    this._focus(parentIndex, parent, { alignTo: "top" });
   }
 
   render() {
@@ -582,10 +753,14 @@ class Tree extends Component {
     // the top and bottom of the page are filled with the `NUMBER_OF_OFFSCREEN_ITEMS`
     // previous and next items respectively, which helps the user to see fewer empty
     // gaps when scrolling quickly.
-    const { itemHeight, focused } = this.props;
+    const { itemHeight, active, focused } = this.props;
     const { scroll, height } = this.state;
-    const begin = Math.max(((scroll / itemHeight) | 0) - NUMBER_OF_OFFSCREEN_ITEMS, 0);
-    const end = Math.ceil((scroll + height) / itemHeight) + NUMBER_OF_OFFSCREEN_ITEMS;
+    const begin = Math.max(
+      ((scroll / itemHeight) | 0) - NUMBER_OF_OFFSCREEN_ITEMS,
+      0
+    );
+    const end =
+      Math.ceil((scroll + height) / itemHeight) + NUMBER_OF_OFFSCREEN_ITEMS;
     const toRender = traversal.slice(begin, end);
     const topSpacerHeight = begin * itemHeight;
     const bottomSpacerHeight = Math.max(traversal.length - end, 0) * itemHeight;
@@ -597,9 +772,9 @@ class Tree extends Component {
         style: {
           padding: 0,
           margin: 0,
-          height: topSpacerHeight + "px"
-        }
-      })
+          height: topSpacerHeight + "px",
+        },
+      }),
     ];
 
     for (let i = 0; i < toRender.length; i++) {
@@ -608,33 +783,44 @@ class Tree extends Component {
       const last = index == traversal.length - 1;
       const { item, depth } = toRender[i];
       const key = this.props.getKey(item);
-      nodes.push(TreeNode({
-        key,
-        index,
-        first,
-        last,
-        item,
-        depth,
-        id: key,
-        renderItem: this.props.renderItem,
-        focused: focused === item,
-        expanded: this.props.isExpanded(item),
-        hasChildren: !!this.props.getChildren(item).length,
-        onExpand: this._onExpand,
-        onCollapse: this._onCollapse,
-        onClick: () => this._focus(begin + i, item),
-      }));
+      nodes.push(
+        TreeNode({
+          // We make a key unique depending on whether the tree node is in active or
+          // inactive state to make sure that it is actually replaced and the tabbable
+          // state is reset.
+          key: `${key}-${active === item ? "active" : "inactive"}`,
+          index,
+          first,
+          last,
+          item,
+          depth,
+          id: key,
+          renderItem: this.props.renderItem,
+          focused: focused === item,
+          active: active === item,
+          expanded: this.props.isExpanded(item),
+          hasChildren: !!this.props.getChildren(item).length,
+          onExpand: this._onExpand,
+          onCollapse: this._onCollapse,
+          // Since the user just clicked the node, there's no need to check if
+          // it should be scrolled into view.
+          onClick: () =>
+            this._focus(begin + i, item, { preventAutoScroll: true }),
+        })
+      );
     }
 
-    nodes.push(dom.div({
-      key: "bottom-spacer",
-      role: "presentation",
-      style: {
-        padding: 0,
-        margin: 0,
-        height: bottomSpacerHeight + "px"
-      }
-    }));
+    nodes.push(
+      dom.div({
+        key: "bottom-spacer",
+        role: "presentation",
+        style: {
+          padding: 0,
+          margin: 0,
+          height: bottomSpacerHeight + "px",
+        },
+      })
+    );
 
     return dom.div(
       {
@@ -646,18 +832,24 @@ class Tree extends Component {
         onKeyPress: this._preventArrowKeyScrolling,
         onKeyUp: this._preventArrowKeyScrolling,
         onScroll: this._onScroll,
-        onFocus: ({nativeEvent}) => {
-          if (focused || !nativeEvent || !this.refs.tree) {
+        onMouseDown: () => this.setState({ mouseDown: true }),
+        onMouseUp: () => this.setState({ mouseDown: false }),
+        onFocus: () => {
+          if (focused || this.state.mouseDown) {
             return;
           }
 
-          let { explicitOriginalTarget } = nativeEvent;
-          // Only set default focus to the first tree node if the focus came
-          // from outside the tree (e.g. by tabbing to the tree from other
-          // external elements).
-          if (explicitOriginalTarget !== this.refs.tree &&
-              !this.refs.tree.contains(explicitOriginalTarget)) {
-            this._focus(begin, toRender[0].item);
+          // Only set default focus to the first tree node if focused node is
+          // not yet set and the focus event is not the result of a mouse
+          // interarction.
+          this._focus(begin, toRender[0].item);
+        },
+        onBlur: e => {
+          if (active != null) {
+            const { relatedTarget } = e;
+            if (!this.refs.tree.contains(relatedTarget)) {
+              this._activate(null);
+            }
           }
         },
         onClick: () => {
@@ -669,8 +861,8 @@ class Tree extends Component {
         "aria-activedescendant": focused && this.props.getKey(focused),
         style: {
           padding: 0,
-          margin: 0
-        }
+          margin: 0,
+        },
       },
       nodes
     );
@@ -692,18 +884,23 @@ class ArrowExpanderClass extends Component {
     };
   }
 
-  shouldComponentUpdate(nextProps, nextState) {
-    return this.props.item !== nextProps.item
-      || this.props.visible !== nextProps.visible
-      || this.props.expanded !== nextProps.expanded;
+  shouldComponentUpdate(nextProps) {
+    return (
+      this.props.item !== nextProps.item ||
+      this.props.visible !== nextProps.visible ||
+      this.props.expanded !== nextProps.expanded
+    );
   }
 
   render() {
     const attrs = {
       className: "arrow theme-twisty",
+      // To collapse/expand the tree rows use left/right arrow keys.
+      tabIndex: "-1",
+      "aria-hidden": true,
       onClick: this.props.expanded
         ? () => this.props.onCollapse(this.props.item)
-        : e => this.props.onExpand(this.props.item, e.altKey)
+        : e => this.props.onExpand(this.props.item, e.altKey),
     };
 
     if (this.props.expanded) {
@@ -712,7 +909,7 @@ class ArrowExpanderClass extends Component {
 
     if (!this.props.visible) {
       attrs.style = {
-        visibility: "hidden"
+        visibility: "hidden",
       };
     }
 
@@ -725,6 +922,7 @@ class TreeNodeClass extends Component {
     return {
       id: PropTypes.any.isRequired,
       focused: PropTypes.bool.isRequired,
+      active: PropTypes.bool.isRequired,
       item: PropTypes.any.isRequired,
       expanded: PropTypes.bool.isRequired,
       hasChildren: PropTypes.bool.isRequired,
@@ -739,6 +937,52 @@ class TreeNodeClass extends Component {
     };
   }
 
+  constructor(props) {
+    super(props);
+
+    this._onKeyDown = this._onKeyDown.bind(this);
+  }
+
+  componentDidMount() {
+    // Make sure that none of the focusable elements inside the tree node container are
+    // tabbable if the tree node is not active. If the tree node is active and focus is
+    // outside its container, focus on the first focusable element inside.
+    const elms = lazy.getFocusableElements(this.refs.treenode);
+    if (elms.length === 0) {
+      return;
+    }
+
+    if (!this.props.active) {
+      elms.forEach(elm => elm.setAttribute("tabindex", "-1"));
+      return;
+    }
+
+    if (!elms.includes(this.refs.treenode.ownerDocument.activeElement)) {
+      elms[0].focus();
+    }
+  }
+
+  _onKeyDown(e) {
+    const { target, key, shiftKey } = e;
+
+    if (key !== "Tab") {
+      return;
+    }
+
+    const focusMoved = !!lazy.wrapMoveFocus(
+      lazy.getFocusableElements(this.refs.treenode),
+      target,
+      shiftKey
+    );
+    if (focusMoved) {
+      // Focus was moved to the begining/end of the list, so we need to prevent the
+      // default focus change that would happen here.
+      e.preventDefault();
+    }
+
+    e.stopPropagation();
+  }
+
   render() {
     const arrow = ArrowExpander({
       item: this.props.item,
@@ -748,15 +992,12 @@ class TreeNodeClass extends Component {
       onCollapse: this.props.onCollapse,
     });
 
-    let classList = [ "tree-node", "div" ];
-    if (this.props.index % 2) {
-      classList.push("tree-node-odd");
+    const classList = ["tree-node", "div"];
+    if (this.props.active) {
+      classList.push("tree-node-active");
     }
-    if (this.props.first) {
-      classList.push("tree-node-first");
-    }
-    if (this.props.last) {
-      classList.push("tree-node-last");
+    if (this.props.focused) {
+      classList.push("focused");
     }
 
     let ariaExpanded;
@@ -772,22 +1013,27 @@ class TreeNodeClass extends Component {
         id: this.props.id,
         className: classList.join(" "),
         role: "treeitem",
-        "aria-level": this.props.depth,
+        ref: "treenode",
+        "aria-level": this.props.depth + 1,
         onClick: this.props.onClick,
+        onKeyDownCapture: this.props.active ? this._onKeyDown : undefined,
         "aria-expanded": ariaExpanded,
         "data-expanded": this.props.expanded ? "" : undefined,
         "data-depth": this.props.depth,
         style: {
           padding: 0,
-          margin: 0
-        }
+          // This helps the CSS compute a margin based on the depth.
+          "--tree-node-depth": this.props.depth,
+        },
       },
 
-      this.props.renderItem(this.props.item,
-                            this.props.depth,
-                            this.props.focused,
-                            arrow,
-                            this.props.expanded),
+      this.props.renderItem(
+        this.props.item,
+        this.props.depth,
+        this.props.focused,
+        arrow,
+        this.props.expanded
+      )
     );
   }
 }
