@@ -10,6 +10,27 @@ const {
   isSupportedByConsoleTable,
 } = require("resource://devtools/shared/webconsole/messages.js");
 
+loader.lazyRequireGetter(
+  this,
+  "getAdHocFrontOrPrimitiveGrip",
+  "resource://devtools/client/fronts/object.js",
+  true
+);
+
+loader.lazyRequireGetter(
+  this,
+  "TRACER_FIELDS_INDEXES",
+  "resource://devtools/server/actors/tracer.js",
+  true
+);
+
+loader.lazyRequireGetter(
+  this,
+  "TRACER_LOG_METHODS",
+  "resource://devtools/shared/specs/tracer.js",
+  true
+);
+
 // URL Regex, common idioms:
 //
 // Lead-in (URL):
@@ -82,7 +103,11 @@ function prepareMessage(resource, idGenerator, persistLogs) {
     resource = transformResource(resource, persistLogs);
   }
 
-  resource.id = idGenerator.getNextId(resource);
+  // The Tracer resource transformer may process some resource
+  // which aren't translated into any item in the console (Tracer frames)
+  if (resource) {
+    resource.id = idGenerator.getNextId(resource);
+  }
   return resource;
 }
 
@@ -96,7 +121,11 @@ function prepareMessage(resource, idGenerator, persistLogs) {
 function transformResource(resource, persistLogs) {
   switch (resource.resourceType || resource.type) {
     case ResourceCommand.TYPES.CONSOLE_MESSAGE: {
-      return transformConsoleAPICallResource(resource, persistLogs);
+      return transformConsoleAPICallResource(
+        resource,
+        persistLogs,
+        resource.targetFront
+      );
     }
 
     case ResourceCommand.TYPES.PLATFORM_MESSAGE: {
@@ -115,6 +144,14 @@ function transformResource(resource, persistLogs) {
       return transformNetworkEventResource(resource);
     }
 
+    case ResourceCommand.TYPES.JSTRACER_STATE: {
+      return transformTracerStateResource(resource);
+    }
+
+    case ResourceCommand.TYPES.JSTRACER_TRACE: {
+      return transformTraceResource(resource);
+    }
+
     case "will-navigate": {
       return transformNavigationMessagePacket(resource);
     }
@@ -127,14 +164,14 @@ function transformResource(resource, persistLogs) {
 }
 
 // eslint-disable-next-line complexity
-function transformConsoleAPICallResource(consoleMessageResource, persistLogs) {
-  const { message, targetFront } = consoleMessageResource;
-
-  let parameters = message.arguments;
-  let type = message.level;
+function transformConsoleAPICallResource(
+  consoleMessageResource,
+  persistLogs,
+  targetFront
+) {
+  let { arguments: parameters, level: type, timer } = consoleMessageResource;
   let level = getLevelFromType(type);
   let messageText = null;
-  const { timer } = message;
 
   // Special per-type conversion.
   switch (type) {
@@ -148,7 +185,7 @@ function transformConsoleAPICallResource(consoleMessageResource, persistLogs) {
     case "countReset":
       // Chrome RDP doesn't have a special type for count.
       type = MESSAGE_TYPE.LOG;
-      const { counter } = message;
+      const { counter } = consoleMessageResource;
 
       if (!counter) {
         // We don't show anything if we don't have counter data.
@@ -235,12 +272,12 @@ function transformConsoleAPICallResource(consoleMessageResource, persistLogs) {
       break;
   }
 
-  const frame = message.filename
+  const frame = consoleMessageResource.filename
     ? {
-        source: message.filename,
-        sourceId: message.sourceId,
-        line: message.lineNumber,
-        column: message.columnNumber,
+        source: consoleMessageResource.filename,
+        sourceId: consoleMessageResource.sourceId,
+        line: consoleMessageResource.lineNumber,
+        column: consoleMessageResource.columnNumber,
       }
     : null;
 
@@ -255,13 +292,15 @@ function transformConsoleAPICallResource(consoleMessageResource, persistLogs) {
     level,
     parameters,
     messageText,
-    stacktrace: message.stacktrace ? message.stacktrace : null,
+    stacktrace: consoleMessageResource.stacktrace
+      ? consoleMessageResource.stacktrace
+      : null,
     frame,
-    timeStamp: message.timeStamp,
-    userProvidedStyles: message.styles,
-    prefix: message.prefix,
-    private: message.private,
-    chromeContext: message.chromeContext,
+    timeStamp: consoleMessageResource.timeStamp,
+    userProvidedStyles: consoleMessageResource.styles,
+    prefix: consoleMessageResource.prefix,
+    private: consoleMessageResource.private,
+    chromeContext: consoleMessageResource.chromeContext,
   });
 }
 
@@ -271,7 +310,9 @@ function transformNavigationMessagePacket(packet) {
     source: MESSAGE_SOURCE.CONSOLE_FRONTEND,
     type: MESSAGE_TYPE.NAVIGATION_MARKER,
     level: MESSAGE_LEVEL.LOG,
-    messageText: l10n.getFormatStr("webconsole.navigated", [url]),
+    messageText: url
+      ? l10n.getFormatStr("webconsole.navigated", [url])
+      : l10n.getStr("webconsole.reloaded"),
     timeStamp: packet.timeStamp,
     allowRepeating: false,
   });
@@ -346,6 +387,173 @@ function transformNetworkEventResource(networkEventResource) {
   return new NetworkEventMessage(networkEventResource);
 }
 
+function transformTraceResource(traceResource) {
+  const { targetFront } = traceResource;
+  const type = traceResource[TRACER_FIELDS_INDEXES.TYPE];
+  const collectedFrames = targetFront.getJsTracerCollectedFramesArray();
+  switch (type) {
+    case "frame":
+      collectedFrames.push(traceResource);
+      return null;
+    case "enter": {
+      const [, prefix, frameIndex, timeStamp, depth, args] = traceResource;
+      const frame = collectedFrames[frameIndex];
+      return new ConsoleMessage({
+        targetFront,
+        source: MESSAGE_SOURCE.JSTRACER,
+        frame: {
+          source: frame[TRACER_FIELDS_INDEXES.FRAME_URL],
+          sourceId: frame[TRACER_FIELDS_INDEXES.FRAME_SOURCEID],
+          line: frame[TRACER_FIELDS_INDEXES.FRAME_LINE],
+          // tracer's column is 0-based while frame uses 1-based numbers
+          column: frame[TRACER_FIELDS_INDEXES.FRAME_COLUMN] + 1,
+        },
+        depth,
+        implementation: frame[TRACER_FIELDS_INDEXES.FRAME_IMPLEMENTATION],
+        displayName: frame[TRACER_FIELDS_INDEXES.FRAME_NAME],
+        parameters: args
+          ? args.map(p =>
+              p ? getAdHocFrontOrPrimitiveGrip(p, targetFront) : p
+            )
+          : null,
+        messageText: null,
+        timeStamp,
+        prefix,
+        // Allow the identical frames to be coalesced into a unique message
+        // with a repeatition counter so that we keep the output short in case of loops.
+        allowRepeating: true,
+      });
+    }
+    case "exit": {
+      const [
+        ,
+        prefix,
+        frameIndex,
+        timeStamp,
+        depth,
+        relatedTraceId,
+        returnedValue,
+        why,
+      ] = traceResource;
+      const frame = collectedFrames[frameIndex];
+      return new ConsoleMessage({
+        targetFront,
+        source: MESSAGE_SOURCE.JSTRACER,
+        frame: {
+          source: frame[TRACER_FIELDS_INDEXES.FRAME_URL],
+          sourceId: frame[TRACER_FIELDS_INDEXES.FRAME_SOURCEID],
+          line: frame[TRACER_FIELDS_INDEXES.FRAME_LINE],
+          column: frame[TRACER_FIELDS_INDEXES.FRAME_COLUMN],
+        },
+        depth,
+        implementation: frame[TRACER_FIELDS_INDEXES.FRAME_IMPLEMENTATION],
+        displayName: frame[TRACER_FIELDS_INDEXES.FRAME_NAME],
+        parameters: null,
+        returnedValue:
+          returnedValue != undefined
+            ? getAdHocFrontOrPrimitiveGrip(returnedValue, targetFront)
+            : null,
+        relatedTraceId,
+        why,
+        messageText: null,
+        timeStamp,
+        prefix,
+        // Allow the identical frames to be coallesced into a unique message
+        // with a repeatition counter so that we keep the output short in case of loops.
+        allowRepeating: true,
+      });
+    }
+    case "dom-mutation": {
+      const [
+        ,
+        prefix,
+        frameIndex,
+        timeStamp,
+        depth,
+        mutationType,
+        mutationElement,
+      ] = traceResource;
+      const frame = collectedFrames[frameIndex];
+      return new ConsoleMessage({
+        targetFront,
+        source: MESSAGE_SOURCE.JSTRACER,
+        frame: {
+          source: frame[TRACER_FIELDS_INDEXES.FRAME_URL],
+          sourceId: frame[TRACER_FIELDS_INDEXES.FRAME_SOURCEID],
+          line: frame[TRACER_FIELDS_INDEXES.FRAME_LINE],
+          column: frame[TRACER_FIELDS_INDEXES.FRAME_COLUMN],
+        },
+        depth,
+        implementation: frame[TRACER_FIELDS_INDEXES.FRAME_IMPLEMENTATION],
+        displayName: frame[TRACER_FIELDS_INDEXES.FRAME_NAME],
+        parameters: null,
+        messageText: null,
+        timeStamp,
+        prefix,
+        mutationType,
+        mutationElement: mutationElement
+          ? getAdHocFrontOrPrimitiveGrip(mutationElement, targetFront)
+          : null,
+        // Allow the identical frames to be coallesced into a unique message
+        // with a repeatition counter so that we keep the output short in case of loops.
+        allowRepeating: true,
+      });
+    }
+    case "event": {
+      const [, prefix, , timeStamp, , eventName] = traceResource;
+      return new ConsoleMessage({
+        targetFront,
+        source: MESSAGE_SOURCE.JSTRACER,
+        depth: 0,
+        prefix,
+        timeStamp,
+        eventName,
+        allowRepeating: false,
+      });
+    }
+  }
+  return null;
+}
+
+function transformTracerStateResource(stateResource) {
+  const { targetFront, enabled, logMethod, timeStamp, reason } = stateResource;
+  let message;
+  if (enabled) {
+    if (logMethod == TRACER_LOG_METHODS.STDOUT) {
+      message = l10n.getStr("webconsole.message.commands.startTracingToStdout");
+    } else if (logMethod == "console") {
+      message = l10n.getStr(
+        "webconsole.message.commands.startTracingToWebConsole"
+      );
+    } else if (logMethod == TRACER_LOG_METHODS.DEBUGGER_SIDEBAR) {
+      message = l10n.getStr(
+        "webconsole.message.commands.startTracingToDebuggerSidebar"
+      );
+    } else if (logMethod == TRACER_LOG_METHODS.PROFILER) {
+      message = l10n.getStr(
+        "webconsole.message.commands.startTracingToProfiler"
+      );
+    } else {
+      throw new Error(`Unsupported tracer log method ${logMethod}`);
+    }
+  } else if (reason) {
+    message = l10n.getFormatStr(
+      "webconsole.message.commands.stopTracingWithReason",
+      [reason]
+    );
+  } else {
+    message = l10n.getStr("webconsole.message.commands.stopTracing");
+  }
+  return new ConsoleMessage({
+    targetFront,
+    source: MESSAGE_SOURCE.CONSOLE_API,
+    type: MESSAGE_TYPE.JSTRACER,
+    level: MESSAGE_LEVEL.LOG,
+    messageText: message,
+    timeStamp,
+  });
+}
+
 function transformEvaluationResultPacket(packet) {
   let {
     exceptionMessage,
@@ -447,7 +655,9 @@ function areMessagesSimilar(message1, message2) {
     message1.isPromiseRejection !== message2.isPromiseRejection ||
     message1.userProvidedStyles?.length !==
       message2.userProvidedStyles?.length ||
-    `${message1.userProvidedStyles}` !== `${message2.userProvidedStyles}`
+    `${message1.userProvidedStyles}` !== `${message2.userProvidedStyles}` ||
+    message1.mutationType !== message2.mutationType ||
+    message1.mutationElement != message2.mutationElement
   ) {
     return false;
   }
@@ -464,6 +674,7 @@ function areMessagesSimilar(message1, message2) {
  * @param {Message} message2
  * @returns {Boolean}
  */
+// eslint-disable-next-line complexity
 function areMessagesParametersSimilar(message1, message2) {
   const message1ParamsLength = message1.parameters?.length;
   if (message1ParamsLength !== message2.parameters?.length) {
@@ -507,6 +718,10 @@ function areMessagesParametersSimilar(message1, message2) {
       if (message1Parameter.text !== message2Parameter.text) {
         return false;
       }
+      // Some objects don't have a text property but a name one (e.g. Symbol)
+      if (message1Parameter.name !== message2Parameter.name) {
+        return false;
+      }
     } else if (message1Parameter !== message2Parameter) {
       return false;
     }
@@ -633,29 +848,26 @@ function createSimpleTableMessage(columns, items, timeStamp) {
  * Given the a regular warning message, compute the label of the warning group the message
  * could be in.
  * For example, if the message text is:
- * The resource at “http://evil.com” was blocked because content blocking is enabled
+ * The resource at “http://evil.com” was blocked because Enhanced Tracking Protection is enabled
  *
  * it may be turned into
  *
- * The resource at “<URL>” was blocked because content blocking is enabled
+ * The resource at “<URL>” was blocked because Enhanced Tracking Protection is enabled
  *
  * @param {ConsoleMessage} firstMessage
  * @returns {String} The computed label
  */
 function getWarningGroupLabel(firstMessage) {
   if (
-    isContentBlockingMessage(firstMessage) ||
+    isEnhancedTrackingProtectionMessage(firstMessage) ||
     isStorageIsolationMessage(firstMessage) ||
     isTrackingProtectionMessage(firstMessage)
   ) {
     return replaceURL(firstMessage.messageText, "<URL>");
   }
 
-  if (isCookieSameSiteMessage(firstMessage)) {
-    if (Services.prefs.getBoolPref("network.cookie.sameSite.laxByDefault")) {
-      return l10n.getStr("webconsole.group.cookieSameSiteLaxByDefaultEnabled2");
-    }
-    return l10n.getStr("webconsole.group.cookieSameSiteLaxByDefaultDisabled2");
+  if (isCookieMessage(firstMessage)) {
+    return l10n.getStr("webconsole.group.cookie");
   }
 
   if (isCSPMessage(firstMessage)) {
@@ -726,13 +938,13 @@ function getWarningGroupType(message) {
 
   if (
     message.level !== MESSAGE_LEVEL.WARN &&
-    // CookieSameSite messages are not warnings but infos
+    // Cookie messages are both warnings and infos
     message.level !== MESSAGE_LEVEL.INFO
   ) {
     return null;
   }
 
-  if (isContentBlockingMessage(message)) {
+  if (isEnhancedTrackingProtectionMessage(message)) {
     return MESSAGE_TYPE.CONTENT_BLOCKING_GROUP;
   }
 
@@ -744,8 +956,8 @@ function getWarningGroupType(message) {
     return MESSAGE_TYPE.TRACKING_PROTECTION_GROUP;
   }
 
-  if (isCookieSameSiteMessage(message)) {
-    return MESSAGE_TYPE.COOKIE_SAMESITE_GROUP;
+  if (isCookieMessage(message)) {
+    return MESSAGE_TYPE.COOKIE_GROUP;
   }
 
   if (isCSPMessage(message)) {
@@ -781,18 +993,18 @@ function isWarningGroup(message) {
     message.type === MESSAGE_TYPE.CONTENT_BLOCKING_GROUP ||
     message.type === MESSAGE_TYPE.STORAGE_ISOLATION_GROUP ||
     message.type === MESSAGE_TYPE.TRACKING_PROTECTION_GROUP ||
-    message.type === MESSAGE_TYPE.COOKIE_SAMESITE_GROUP ||
+    message.type === MESSAGE_TYPE.COOKIE_GROUP ||
     message.type === MESSAGE_TYPE.CORS_GROUP ||
     message.type === MESSAGE_TYPE.CSP_GROUP
   );
 }
 
 /**
- * Returns true if the message is a content blocking message.
+ * Returns true if the message is an Enhanced Tracking Protection message.
  * @param {ConsoleMessage} message
  * @returns {Boolean}
  */
-function isContentBlockingMessage(message) {
+function isEnhancedTrackingProtectionMessage(message) {
   const { category } = message;
   return (
     category == "cookieBlockedPermission" ||
@@ -827,9 +1039,14 @@ function isTrackingProtectionMessage(message) {
  * @param {ConsoleMessage} message
  * @returns {Boolean}
  */
-function isCookieSameSiteMessage(message) {
+function isCookieMessage(message) {
   const { category } = message;
-  return category == "cookieSameSite";
+  return [
+    "cookiesCHIPS",
+    "cookiesOversize",
+    "cookieSameSite",
+    "cookieInvalidAttribute",
+  ].includes(category);
 }
 
 /**
@@ -907,7 +1124,7 @@ module.exports = {
   getNaturalOrder,
   getParentWarningGroupMessageId,
   getWarningGroupType,
-  isContentBlockingMessage,
+  isEnhancedTrackingProtectionMessage,
   isGroupType,
   isMessageNetworkError,
   isPacketPrivate,
