@@ -1,25 +1,34 @@
- /* This Source Code Form is subject to the terms of the Mozilla Public
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 "use strict";
 
-const { AutoRefreshHighlighter } = require("./auto-refresh");
+const {
+  AutoRefreshHighlighter,
+} = require("resource://devtools/server/actors/highlighters/auto-refresh.js");
 const {
   CanvasFrameAnonymousContentHelper,
-  createNode,
-  createSVGNode,
   getBindingElementAndPseudo,
   hasPseudoClassLock,
   isNodeValid,
   moveInfobar,
-} = require("./utils/markup");
+} = require("resource://devtools/server/actors/highlighters/utils/markup.js");
+const {
+  PSEUDO_CLASSES,
+} = require("resource://devtools/shared/css/constants.js");
 const {
-  setIgnoreLayoutChanges,
   getCurrentZoom,
- } = require("devtools/shared/layout/utils");
-const inspector = require("devtools/server/actors/inspector");
-const nodeConstants = require("devtools/shared/dom-node-constants");
+  setIgnoreLayoutChanges,
+} = require("resource://devtools/shared/layout/utils.js");
+const {
+  getNodeDisplayName,
+  getNodeGridFlexType,
+} = require("resource://devtools/server/actors/inspector/utils.js");
+const nodeConstants = require("resource://devtools/shared/dom-node-constants.js");
+loader.lazyGetter(this, "HighlightersBundle", () => {
+  return new Localization(["devtools/shared/highlighters.ftl"], true);
+});
 
 // Note that the order of items in this array is important because it is used
 // for drawing the BoxModelHighlighter's path elements correctly.
@@ -27,8 +36,6 @@ const BOX_MODEL_REGIONS = ["margin", "border", "padding", "content"];
 const BOX_MODEL_SIDES = ["top", "right", "bottom", "left"];
 // Width of boxmodelhighlighter guides
 const GUIDE_STROKE_WIDTH = 1;
-// FIXME: add ":visited" and ":link" after bug 713106 is fixed
-const PSEUDO_CLASSES = [":hover", ":active", ":focus"];
 
 /**
  * The BoxModelHighlighter draws the box model regions on top of a node.
@@ -43,26 +50,24 @@ const PSEUDO_CLASSES = [":hover", ":active", ":focus"];
  * h.hide();
  * h.destroy();
  *
- * Available options:
- * - region {String}
- *   "content", "padding", "border" or "margin"
- *   This specifies the region that the guides should outline.
- *   Defaults to "content"
- * - hideGuides {Boolean}
- *   Defaults to false
- * - hideInfoBar {Boolean}
- *   Defaults to false
- * - showOnly {String}
- *   "content", "padding", "border" or "margin"
- *   If set, only this region will be highlighted. Use with onlyRegionArea to
- *   only highlight the area of the region.
- * - onlyRegionArea {Boolean}
- *   This can be set to true to make each region's box only highlight the area
- *   of the corresponding region rather than the area of nested regions too.
- *   This is useful when used with showOnly.
+ * @param {String} options.region
+ *        Specifies the region that the guides should outline:
+ *          "content" (default), "padding", "border" or "margin".
+ * @param {Boolean} options.hideGuides
+ *        Defaults to false
+ * @param {Boolean} options.hideInfoBar
+ *        Defaults to false
+ * @param {String} options.showOnly
+ *        If set, only this region will be highlighted. Use with onlyRegionArea
+ *        to only highlight the area of the region:
+ *        "content", "padding", "border" or "margin"
+ * @param {Boolean} options.onlyRegionArea
+ *        This can be set to true to make each region's box only highlight the
+ *        area of the corresponding region rather than the area of nested
+ *        regions too. This is useful when used with showOnly.
  *
  * Structure:
- * <div class="highlighter-container">
+ * <div class="highlighter-container" aria-hidden="true">
  *   <div class="box-model-root">
  *     <svg class="box-model-elements" hidden="true">
  *       <g class="box-model-regions">
@@ -84,6 +89,8 @@ const PSEUDO_CLASSES = [":hover", ":active", ":focus"];
  *           <span class="box-model-infobar-id">Node id</span>
  *           <span class="box-model-infobar-classes">.someClass</span>
  *           <span class="box-model-infobar-pseudo-classes">:hover</span>
+ *           <span class="box-model-infobar-grid-type">Grid Type</span>
+ *           <span class="box-model-infobar-flex-type">Flex Type</span>
  *         </div>
  *       </div>
  *       <div class="box-model-infobar-arrow box-model-infobar-arrow-bottom"/>
@@ -95,168 +102,192 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
   constructor(highlighterEnv) {
     super(highlighterEnv);
 
-    this.ID_CLASS_PREFIX = "box-model-";
-
-    this.markup = new CanvasFrameAnonymousContentHelper(this.highlighterEnv,
-      this._buildMarkup.bind(this));
-
-    /**
-     * Optionally customize each region's fill color by adding an entry to the
-     * regionFill property: `highlighter.regionFill.margin = "red";
-     */
-    this.regionFill = {};
+    this.markup = new CanvasFrameAnonymousContentHelper(
+      this.highlighterEnv,
+      this._buildMarkup.bind(this),
+      {
+        contentRootHostClassName: "devtools-highlighter-box-model",
+      }
+    );
+    this.isReady = this.markup.initialize();
 
     this.onPageHide = this.onPageHide.bind(this);
     this.onWillNavigate = this.onWillNavigate.bind(this);
 
     this.highlighterEnv.on("will-navigate", this.onWillNavigate);
 
-    let { pageListenerTarget } = highlighterEnv;
+    const { pageListenerTarget } = highlighterEnv;
     pageListenerTarget.addEventListener("pagehide", this.onPageHide);
   }
 
-  _buildMarkup() {
-    let doc = this.win.document;
+  /**
+   * Static getter that indicates that BoxModelHighlighter supports
+   * highlighting in XUL windows.
+   */
+  static get XULSupported() {
+    return true;
+  }
 
-    let highlighterContainer = doc.createElement("div");
-    highlighterContainer.setAttribute("role", "presentation");
+  get supportsSimpleHighlighters() {
+    return true;
+  }
+
+  _buildMarkup() {
+    const highlighterContainer =
+      this.markup.anonymousContentDocument.createElement("div");
     highlighterContainer.className = "highlighter-container box-model";
 
+    this.highlighterContainer = highlighterContainer;
+    // We need a better solution for how to handle the highlighter from the
+    // accessibility standpoint. For now, in order to avoid displaying it in the
+    // accessibility tree lets hide it altogether. See bug 1598667 for more
+    // context.
+    highlighterContainer.setAttribute("aria-hidden", "true");
+
     // Build the root wrapper, used to adapt to the page zoom.
-    let rootWrapper = createNode(this.win, {
+    this.rootEl = this.markup.createNode({
       parent: highlighterContainer,
       attributes: {
-        "id": "root",
-        "class": "root",
-        "role": "presentation"
+        id: "box-model-root",
+        class:
+          "box-model-root" +
+          (this.highlighterEnv.useSimpleHighlightersForReducedMotion
+            ? " use-simple-highlighters"
+            : ""),
+        role: "presentation",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     // Building the SVG element with its polygons and lines
 
-    let svg = createSVGNode(this.win, {
+    const svg = this.markup.createSVGNode({
       nodeType: "svg",
-      parent: rootWrapper,
+      parent: this.rootEl,
       attributes: {
-        "id": "elements",
-        "width": "100%",
-        "height": "100%",
-        "hidden": "true",
-        "role": "presentation"
+        id: "box-model-elements",
+        width: "100%",
+        height: "100%",
+        hidden: "true",
+        role: "presentation",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
-    let regions = createSVGNode(this.win, {
+    const regions = this.markup.createSVGNode({
       nodeType: "g",
       parent: svg,
       attributes: {
-        "class": "regions",
-        "role": "presentation"
+        class: "box-model-regions",
+        role: "presentation",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
-    for (let region of BOX_MODEL_REGIONS) {
-      createSVGNode(this.win, {
+    for (const region of BOX_MODEL_REGIONS) {
+      this.markup.createSVGNode({
         nodeType: "path",
         parent: regions,
         attributes: {
-          "class": region,
-          "id": region,
-          "role": "presentation"
+          class: "box-model-" + region,
+          id: "box-model-" + region,
+          role: "presentation",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
     }
 
-    for (let side of BOX_MODEL_SIDES) {
-      createSVGNode(this.win, {
+    for (const side of BOX_MODEL_SIDES) {
+      this.markup.createSVGNode({
         nodeType: "line",
         parent: svg,
         attributes: {
-          "class": "guide-" + side,
-          "id": "guide-" + side,
+          class: "box-model-guide-" + side,
+          id: "box-model-guide-" + side,
           "stroke-width": GUIDE_STROKE_WIDTH,
-          "role": "presentation"
+          role: "presentation",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
     }
 
     // Building the nodeinfo bar markup
 
-    let infobarContainer = createNode(this.win, {
-      parent: rootWrapper,
+    const infobarContainer = this.markup.createNode({
+      parent: this.rootEl,
       attributes: {
-        "class": "infobar-container",
-        "id": "infobar-container",
-        "position": "top",
-        "hidden": "true"
+        class: "box-model-infobar-container",
+        id: "box-model-infobar-container",
+        position: "top",
+        hidden: "true",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
-    let infobar = createNode(this.win, {
+    const infobar = this.markup.createNode({
       parent: infobarContainer,
       attributes: {
-        "class": "infobar"
+        class: "box-model-infobar",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
-    let texthbox = createNode(this.win, {
+    const texthbox = this.markup.createNode({
       parent: infobar,
       attributes: {
-        "class": "infobar-text"
+        class: "box-model-infobar-text",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "span",
       parent: texthbox,
       attributes: {
-        "class": "infobar-tagname",
-        "id": "infobar-tagname"
+        class: "box-model-infobar-tagname",
+        id: "box-model-infobar-tagname",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "span",
       parent: texthbox,
       attributes: {
-        "class": "infobar-id",
-        "id": "infobar-id"
+        class: "box-model-infobar-id",
+        id: "box-model-infobar-id",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "span",
       parent: texthbox,
       attributes: {
-        "class": "infobar-classes",
-        "id": "infobar-classes"
+        class: "box-model-infobar-classes",
+        id: "box-model-infobar-classes",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "span",
       parent: texthbox,
       attributes: {
-        "class": "infobar-pseudo-classes",
-        "id": "infobar-pseudo-classes"
+        class: "box-model-infobar-pseudo-classes",
+        id: "box-model-infobar-pseudo-classes",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "span",
       parent: texthbox,
       attributes: {
-        "class": "infobar-dimensions",
-        "id": "infobar-dimensions"
+        class: "box-model-infobar-dimensions",
+        id: "box-model-infobar-dimensions",
+      },
+    });
+
+    this.markup.createNode({
+      nodeType: "span",
+      parent: texthbox,
+      attributes: {
+        class: "box-model-infobar-grid-type",
+        id: "box-model-infobar-grid-type",
+      },
+    });
+
+    this.markup.createNode({
+      nodeType: "span",
+      parent: texthbox,
+      attributes: {
+        class: "box-model-infobar-flex-type",
+        id: "box-model-infobar-flex-type",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     return highlighterContainer;
@@ -268,17 +299,19 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
   destroy() {
     this.highlighterEnv.off("will-navigate", this.onWillNavigate);
 
-    let { pageListenerTarget } = this.highlighterEnv;
+    const { pageListenerTarget } = this.highlighterEnv;
     if (pageListenerTarget) {
       pageListenerTarget.removeEventListener("pagehide", this.onPageHide);
     }
 
     this.markup.destroy();
+    this.rootEl = null;
+
     AutoRefreshHighlighter.prototype.destroy.call(this);
   }
 
   getElement(id) {
-    return this.markup.getElement(this.ID_CLASS_PREFIX + id);
+    return this.markup.getElement(id);
   }
 
   /**
@@ -288,20 +321,21 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * @return {Boolean}
    */
   _isNodeValid(node) {
-    return node && (isNodeValid(node) || isNodeValid(node, nodeConstants.TEXT_NODE));
+    return (
+      node && (isNodeValid(node) || isNodeValid(node, nodeConstants.TEXT_NODE))
+    );
   }
 
   /**
    * Show the highlighter on a given node
    */
   _show() {
-    if (BOX_MODEL_REGIONS.indexOf(this.options.region) == -1) {
+    if (!BOX_MODEL_REGIONS.includes(this.options.region)) {
       this.options.region = "content";
     }
 
-    let shown = this._update();
+    const shown = this._update();
     this._trackMutations();
-    this.emit("ready");
     return shown;
   }
 
@@ -311,9 +345,9 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    */
   _trackMutations() {
     if (isNodeValid(this.currentNode)) {
-      let win = this.currentNode.ownerGlobal;
+      const win = this.currentNode.ownerGlobal;
       this.currentNodeObserver = new win.MutationObserver(this.update);
-      this.currentNodeObserver.observe(this.currentNode, {attributes: true});
+      this.currentNodeObserver.observe(this.currentNode, { attributes: true });
     }
   }
 
@@ -330,27 +364,35 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * Should be called whenever node size or attributes change
    */
   _update() {
+    const node = this.currentNode;
     let shown = false;
     setIgnoreLayoutChanges(true);
 
     if (this._updateBoxModel()) {
       // Show the infobar only if configured to do so and the node is an element or a text
       // node.
-      if (!this.options.hideInfoBar && (
-          this.currentNode.nodeType === this.currentNode.ELEMENT_NODE ||
-          this.currentNode.nodeType === this.currentNode.TEXT_NODE)) {
+      if (
+        !this.options.hideInfoBar &&
+        (node.nodeType === node.ELEMENT_NODE ||
+          node.nodeType === node.TEXT_NODE)
+      ) {
         this._showInfobar();
       } else {
         this._hideInfobar();
       }
+      this._updateSimpleHighlighters();
       this._showBoxModel();
+
       shown = true;
     } else {
       // Nothing to highlight (0px rectangle like a <script> tag for instance)
       this._hide();
     }
 
-    setIgnoreLayoutChanges(false, this.highlighterEnv.window.document.documentElement);
+    setIgnoreLayoutChanges(
+      false,
+      this.highlighterEnv.window.document.documentElement
+    );
 
     return shown;
   }
@@ -369,21 +411,27 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
     this._hideBoxModel();
     this._hideInfobar();
 
-    setIgnoreLayoutChanges(false, this.highlighterEnv.window.document.documentElement);
+    setIgnoreLayoutChanges(
+      false,
+      this.highlighterEnv.window.document.documentElement
+    );
   }
 
   /**
    * Hide the infobar
    */
   _hideInfobar() {
-    this.getElement("infobar-container").setAttribute("hidden", "true");
+    this.getElement("box-model-infobar-container").setAttribute(
+      "hidden",
+      "true"
+    );
   }
 
   /**
    * Show the infobar
    */
   _showInfobar() {
-    this.getElement("infobar-container").removeAttribute("hidden");
+    this.getElement("box-model-infobar-container").removeAttribute("hidden");
     this._updateInfobar();
   }
 
@@ -391,14 +439,14 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * Hide the box model
    */
   _hideBoxModel() {
-    this.getElement("elements").setAttribute("hidden", "true");
+    this.getElement("box-model-elements").setAttribute("hidden", "true");
   }
 
   /**
    * Show the box model
    */
   _showBoxModel() {
-    this.getElement("elements").removeAttribute("hidden");
+    this.getElement("box-model-elements").removeAttribute("hidden");
   }
 
   /**
@@ -412,16 +460,16 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * @return {Object} A quad-like object {p1,p2,p3,p4,bounds}
    */
   _getOuterQuad(region) {
-    let quads = this.currentQuads[region];
-    if (!quads.length) {
+    const quads = this.currentQuads[region];
+    if (!quads || !quads.length) {
       return null;
     }
 
-    let quad = {
-      p1: {x: Infinity, y: Infinity},
-      p2: {x: -Infinity, y: Infinity},
-      p3: {x: -Infinity, y: -Infinity},
-      p4: {x: Infinity, y: -Infinity},
+    const quad = {
+      p1: { x: Infinity, y: Infinity },
+      p2: { x: -Infinity, y: Infinity },
+      p3: { x: -Infinity, y: -Infinity },
+      p4: { x: Infinity, y: -Infinity },
       bounds: {
         bottom: -Infinity,
         height: 0,
@@ -431,10 +479,10 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
         width: 0,
         x: 0,
         y: 0,
-      }
+      },
     };
 
-    for (let q of quads) {
+    for (const q of quads) {
       quad.p1.x = Math.min(quad.p1.x, q.p1.x);
       quad.p1.y = Math.min(quad.p1.y, q.p1.y);
       quad.p2.x = Math.max(quad.p2.x, q.p2.x);
@@ -464,7 +512,7 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    *         True if the current node has a box model to be highlighted
    */
   _updateBoxModel() {
-    let options = this.options;
+    const options = this.options;
     options.region = options.region || "content";
 
     if (!this._nodeNeedsHighlighting()) {
@@ -473,24 +521,18 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
     }
 
     for (let i = 0; i < BOX_MODEL_REGIONS.length; i++) {
-      let boxType = BOX_MODEL_REGIONS[i];
-      let nextBoxType = BOX_MODEL_REGIONS[i + 1];
-      let box = this.getElement(boxType);
-
-      if (this.regionFill[boxType]) {
-        box.setAttribute("style", "fill:" + this.regionFill[boxType]);
-      } else {
-        box.setAttribute("style", "");
-      }
+      const boxType = BOX_MODEL_REGIONS[i];
+      const nextBoxType = BOX_MODEL_REGIONS[i + 1];
+      const box = this.getElement("box-model-" + boxType);
 
       // Highlight all quads for this region by setting the "d" attribute of the
       // corresponding <path>.
-      let path = [];
+      const path = [];
       for (let j = 0; j < this.currentQuads[boxType].length; j++) {
-        let boxQuad = this.currentQuads[boxType][j];
-        let nextBoxQuad = this.currentQuads[nextBoxType]
-                          ? this.currentQuads[nextBoxType][j]
-                          : null;
+        const boxQuad = this.currentQuads[boxType][j];
+        const nextBoxQuad = this.currentQuads[nextBoxType]
+          ? this.currentQuads[nextBoxType][j]
+          : null;
         path.push(this._getBoxPathCoordinates(boxQuad, nextBoxQuad));
       }
 
@@ -515,36 +557,96 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
     }
 
     // Un-zoom the root wrapper if the page was zoomed.
-    let rootId = this.ID_CLASS_PREFIX + "elements";
-    this.markup.scaleRootElement(this.currentNode, rootId);
+    this.markup.scaleRootElement(this.currentNode, "box-model-elements");
 
     return true;
   }
 
   _getBoxPathCoordinates(boxQuad, nextBoxQuad) {
-    let {p1, p2, p3, p4} = boxQuad;
+    const { p1, p2, p3, p4 } = boxQuad;
 
     let path;
     if (!nextBoxQuad || !this.options.onlyRegionArea) {
       // If this is the content box (inner-most box) or if we're not being asked
       // to highlight only region areas, then draw a simple rectangle.
-      path = "M" + p1.x + "," + p1.y + " " +
-             "L" + p2.x + "," + p2.y + " " +
-             "L" + p3.x + "," + p3.y + " " +
-             "L" + p4.x + "," + p4.y;
+      path =
+        "M" +
+        p1.x +
+        "," +
+        p1.y +
+        " " +
+        "L" +
+        p2.x +
+        "," +
+        p2.y +
+        " " +
+        "L" +
+        p3.x +
+        "," +
+        p3.y +
+        " " +
+        "L" +
+        p4.x +
+        "," +
+        p4.y +
+        " " +
+        "L" +
+        p1.x +
+        "," +
+        p1.y;
     } else {
       // Otherwise, just draw the region itself, not a filled rectangle.
-      let {p1: np1, p2: np2, p3: np3, p4: np4} = nextBoxQuad;
-      path = "M" + p1.x + "," + p1.y + " " +
-             "L" + p2.x + "," + p2.y + " " +
-             "L" + p3.x + "," + p3.y + " " +
-             "L" + p4.x + "," + p4.y + " " +
-             "L" + p1.x + "," + p1.y + " " +
-             "L" + np1.x + "," + np1.y + " " +
-             "L" + np4.x + "," + np4.y + " " +
-             "L" + np3.x + "," + np3.y + " " +
-             "L" + np2.x + "," + np2.y + " " +
-             "L" + np1.x + "," + np1.y;
+      const { p1: np1, p2: np2, p3: np3, p4: np4 } = nextBoxQuad;
+      path =
+        "M" +
+        p1.x +
+        "," +
+        p1.y +
+        " " +
+        "L" +
+        p2.x +
+        "," +
+        p2.y +
+        " " +
+        "L" +
+        p3.x +
+        "," +
+        p3.y +
+        " " +
+        "L" +
+        p4.x +
+        "," +
+        p4.y +
+        " " +
+        "L" +
+        p1.x +
+        "," +
+        p1.y +
+        " " +
+        "L" +
+        np1.x +
+        "," +
+        np1.y +
+        " " +
+        "L" +
+        np4.x +
+        "," +
+        np4.y +
+        " " +
+        "L" +
+        np3.x +
+        "," +
+        np3.y +
+        " " +
+        "L" +
+        np2.x +
+        "," +
+        np2.y +
+        " " +
+        "L" +
+        np1.x +
+        "," +
+        np1.y;
     }
 
     return path;
@@ -555,25 +657,27 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * @return {Boolean}
    */
   _nodeNeedsHighlighting() {
-    return this.currentQuads.margin.length ||
-           this.currentQuads.border.length ||
-           this.currentQuads.padding.length ||
-           this.currentQuads.content.length;
+    return (
+      this.currentQuads.margin.length ||
+      this.currentQuads.border.length ||
+      this.currentQuads.padding.length ||
+      this.currentQuads.content.length
+    );
   }
 
   _getOuterBounds() {
-    for (let region of ["margin", "border", "padding", "content"]) {
-      let quad = this._getOuterQuad(region);
+    for (const region of ["margin", "border", "padding", "content"]) {
+      const quad = this._getOuterQuad(region);
 
       if (!quad) {
         // Invisible element such as a script tag.
         break;
       }
 
-      let {bottom, height, left, right, top, width, x, y} = quad.bounds;
+      const { bottom, height, left, right, top, width, x, y } = quad.bounds;
 
       if (width > 0 || height > 0) {
-        return {bottom, height, left, right, top, width, x, y};
+        return { bottom, height, left, right, top, width, x, y };
       }
     }
 
@@ -585,7 +689,7 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
       top: 0,
       width: 0,
       x: 0,
-      y: 0
+      y: 0,
     };
   }
 
@@ -595,16 +699,23 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * @param {String} region The region around which the guides should be shown.
    */
   _showGuides(region) {
-    let {p1, p2, p3, p4} = this._getOuterQuad(region);
+    const quad = this._getOuterQuad(region);
 
-    let allX = [p1.x, p2.x, p3.x, p4.x].sort((a, b) => a - b);
-    let allY = [p1.y, p2.y, p3.y, p4.y].sort((a, b) => a - b);
-    let toShowX = [];
-    let toShowY = [];
+    if (!quad) {
+      // Invisible element such as a script tag.
+      return;
+    }
 
-    for (let arr of [allX, allY]) {
+    const { p1, p2, p3, p4 } = quad;
+
+    const allX = [p1.x, p2.x, p3.x, p4.x].sort((a, b) => a - b);
+    const allY = [p1.y, p2.y, p3.y, p4.y].sort((a, b) => a - b);
+    const toShowX = [];
+    const toShowY = [];
+
+    for (const arr of [allX, allY]) {
       for (let i = 0; i < arr.length; i++) {
-        let val = arr[i];
+        const val = arr[i];
 
         if (i !== arr.lastIndexOf(val)) {
           if (arr === allX) {
@@ -620,13 +731,13 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
     // Move guide into place or hide it if no valid co-ordinate was found.
     this._updateGuide("top", Math.round(toShowY[0]));
     this._updateGuide("right", Math.round(toShowX[1]) - 1);
-    this._updateGuide("bottom", Math.round(toShowY[1] - 1));
+    this._updateGuide("bottom", Math.round(toShowY[1]) - 1);
     this._updateGuide("left", Math.round(toShowX[0]));
   }
 
   _hideGuides() {
-    for (let side of BOX_MODEL_SIDES) {
-      this.getElement("guide-" + side).setAttribute("hidden", "true");
+    for (const side of BOX_MODEL_SIDES) {
+      this.getElement("box-model-guide-" + side).setAttribute("hidden", "true");
     }
   }
 
@@ -639,10 +750,10 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * @param  {Integer} point
    *         x or y co-ordinate. If this is undefined we hide the guide.
    */
-  _updateGuide(side, point = -1) {
-    let guide = this.getElement("guide-" + side);
+  _updateGuide(side, point) {
+    const guide = this.getElement("box-model-guide-" + side);
 
-    if (point <= 0) {
+    if (!point || point <= 0) {
       guide.setAttribute("hidden", "true");
       return false;
     }
@@ -672,42 +783,99 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
       return;
     }
 
-    let {bindingElement: node, pseudo} =
-        getBindingElementAndPseudo(this.currentNode);
-
-    // Update the tag, id, classes, pseudo-classes and dimensions
-    let displayName = inspector.getNodeDisplayName(node);
-
-    let id = node.id ? "#" + node.id : "";
-
-    let classList = (node.classList || []).length
-                    ? "." + [...node.classList].join(".")
-                    : "";
-
-    let pseudos = this._getPseudoClasses(node).join("");
-    if (pseudo) {
-      // Display :after as ::after
-      pseudos += ":" + pseudo;
-    }
+    const { bindingElement: node } = getBindingElementAndPseudo(
+      this.currentNode
+    );
 
     // We want to display the original `width` and `height`, instead of the ones affected
     // by any zoom. Since the infobar can be displayed also for text nodes, we can't
     // access the computed style for that, and this is why we recalculate them here.
-    let zoom = getCurrentZoom(this.win);
-    let { width, height } = this._getOuterQuad("border").bounds;
-    let dim = parseFloat((width / zoom).toPrecision(6)) +
-              " \u00D7 " +
-              parseFloat((height / zoom).toPrecision(6));
+    const zoom = getCurrentZoom(this.win);
+    const quad = this._getOuterQuad("border");
 
-    this.getElement("infobar-tagname").setTextContent(displayName);
-    this.getElement("infobar-id").setTextContent(id);
-    this.getElement("infobar-classes").setTextContent(classList);
-    this.getElement("infobar-pseudo-classes").setTextContent(pseudos);
-    this.getElement("infobar-dimensions").setTextContent(dim);
+    if (!quad) {
+      return;
+    }
+
+    const { width, height } = quad.bounds;
+    const dim =
+      parseFloat((width / zoom).toPrecision(6)) +
+      " \u00D7 " +
+      parseFloat((height / zoom).toPrecision(6));
+
+    const { grid: gridType, flex: flexType } = getNodeGridFlexType(node);
+    const gridLayoutTextType = this._getLayoutTextType("gridtype", gridType);
+    const flexLayoutTextType = this._getLayoutTextType("flextype", flexType);
+
+    // Update the tag, id, classes, pseudo-classes and dimensions
+    this._updateInfobarNodeData();
+    this.getElement("box-model-infobar-dimensions").setTextContent(dim);
+    this.getElement("box-model-infobar-grid-type").setTextContent(
+      gridLayoutTextType
+    );
+    this.getElement("box-model-infobar-flex-type").setTextContent(
+      flexLayoutTextType
+    );
 
     this._moveInfobar();
   }
 
+  _updateInfobarNodeData() {
+    if (!this.currentNode) {
+      return;
+    }
+
+    // The binding element of a pseudo element can also be a pseudo element (for example
+    // ::before::marker), so walk up through the tree until we get a non pseudo binding
+    // element.
+    let node = this.currentNode,
+      pseudo = "";
+    while (true) {
+      const res = getBindingElementAndPseudo(node);
+
+      // Stop as soon as the binding element is the same as the passed node, meaning we
+      // found the ultimate originating element (https://drafts.csswg.org/selectors-4/#ultimate-originating-element).
+      if (res.bindingElement === node) {
+        break;
+      }
+
+      node = res.bindingElement;
+      pseudo = res.pseudo + pseudo;
+    }
+
+    // Update the tag, id, classes, pseudo-classes and dimensions
+    const displayName = getNodeDisplayName(node);
+
+    const id = node.id ? "#" + node.id : "";
+
+    const classList = node.classList?.length
+      ? "." + [...node.classList].join(".")
+      : "";
+
+    let pseudos = this._getPseudoClasses(node).join("");
+    if (pseudo) {
+      pseudos += pseudo;
+    }
+
+    this.getElement("box-model-infobar-tagname").setTextContent(displayName);
+    this.getElement("box-model-infobar-id").setTextContent(id);
+    this.getElement("box-model-infobar-classes").setTextContent(classList);
+    this.getElement("box-model-infobar-pseudo-classes").setTextContent(pseudos);
+  }
+
+  _getLayoutTextType(layoutTypeKey, { isContainer, isItem }) {
+    if (!isContainer && !isItem) {
+      return "";
+    }
+    if (isContainer && !isItem) {
+      return HighlightersBundle.formatValueSync(`${layoutTypeKey}-container`);
+    }
+    if (!isContainer && isItem) {
+      return HighlightersBundle.formatValueSync(`${layoutTypeKey}-item`);
+    }
+    return HighlightersBundle.formatValueSync(`${layoutTypeKey}-dual`);
+  }
+
   _getPseudoClasses(node) {
     if (node.nodeType !== nodeConstants.ELEMENT_NODE) {
       // hasPseudoClassLock can only be used on Elements.
@@ -721,8 +889,8 @@ class BoxModelHighlighter extends AutoRefreshHighlighter {
    * Move the Infobar to the right place in the highlighter.
    */
   _moveInfobar() {
-    let bounds = this._getOuterBounds();
-    let container = this.getElement("infobar-container");
+    const bounds = this._getOuterBounds();
+    const container = this.getElement("box-model-infobar-container");
 
     moveInfobar(container, bounds, this.win);
   }
