@@ -4,11 +4,21 @@
 
 "use strict";
 
-const { AutoRefreshHighlighter } = require("./auto-refresh");
-const { CanvasFrameAnonymousContentHelper, getComputedStyle,
-        createSVGNode, createNode } = require("./utils/markup");
-const { setIgnoreLayoutChanges, getAdjustedQuads } = require("devtools/shared/layout/utils");
-const { getCSSStyleRules } = require("devtools/shared/inspector/css-logic");
+const {
+  AutoRefreshHighlighter,
+} = require("resource://devtools/server/actors/highlighters/auto-refresh.js");
+const {
+  CanvasFrameAnonymousContentHelper,
+  getComputedStyle,
+} = require("resource://devtools/server/actors/highlighters/utils/markup.js");
+const {
+  setIgnoreLayoutChanges,
+  getAdjustedQuads,
+  getCurrentZoom,
+} = require("resource://devtools/shared/layout/utils.js");
+const {
+  getMatchingCSSRules,
+} = require("resource://devtools/shared/inspector/css-logic.js");
 
 const GEOMETRY_LABEL_SIZE = 6;
 
@@ -26,57 +36,57 @@ var GeoProp = {
   SIDES: ["top", "right", "bottom", "left"],
   SIZES: ["width", "height"],
 
-  allProps: function () {
+  allProps() {
     return [...this.SIDES, ...this.SIZES];
   },
 
-  isSide: function (name) {
-    return this.SIDES.indexOf(name) !== -1;
+  isSide(name) {
+    return this.SIDES.includes(name);
   },
 
-  isSize: function (name) {
-    return this.SIZES.indexOf(name) !== -1;
+  isSize(name) {
+    return this.SIZES.includes(name);
   },
 
-  containsSide: function (names) {
-    return names.some(name => this.SIDES.indexOf(name) !== -1);
+  containsSide(names) {
+    return names.some(name => this.SIDES.includes(name));
   },
 
-  containsSize: function (names) {
-    return names.some(name => this.SIZES.indexOf(name) !== -1);
+  containsSize(names) {
+    return names.some(name => this.SIZES.includes(name));
   },
 
-  isHorizontal: function (name) {
+  isHorizontal(name) {
     return name === "left" || name === "right" || name === "width";
   },
 
-  isInverted: function (name) {
+  isInverted(name) {
     return name === "right" || name === "bottom";
   },
 
-  mainAxisStart: function (name) {
+  mainAxisStart(name) {
     return this.isHorizontal(name) ? "left" : "top";
   },
 
-  crossAxisStart: function (name) {
+  crossAxisStart(name) {
     return this.isHorizontal(name) ? "top" : "left";
   },
 
-  mainAxisSize: function (name) {
+  mainAxisSize(name) {
     return this.isHorizontal(name) ? "width" : "height";
   },
 
-  crossAxisSize: function (name) {
+  crossAxisSize(name) {
     return this.isHorizontal(name) ? "height" : "width";
   },
 
-  axis: function (name) {
+  axis(name) {
     return this.isHorizontal(name) ? "x" : "y";
   },
 
-  crossAxis: function (name) {
+  crossAxis(name) {
     return this.isHorizontal(name) ? "y" : "x";
-  }
+  },
 };
 
 /**
@@ -92,11 +102,10 @@ var GeoProp = {
  * @return {Object}
  */
 function getOffsetParent(node) {
-  let win = node.ownerGlobal;
+  const win = node.ownerGlobal;
 
   let offsetParent = node.offsetParent;
-  if (offsetParent &&
-      getComputedStyle(offsetParent).position === "static") {
+  if (offsetParent && getComputedStyle(offsetParent).position === "static") {
     offsetParent = null;
   }
 
@@ -111,7 +120,7 @@ function getOffsetParent(node) {
 
   return {
     element: offsetParent,
-    dimension: {width, height}
+    dimension: { width, height },
   };
 }
 
@@ -119,27 +128,27 @@ function getOffsetParent(node) {
  * Get the list of geometry properties that are actually set on the provided
  * node.
  *
- * @param {nsIDOMNode} node The node to analyze.
+ * @param {Node} node The node to analyze.
  * @return {Map} A map indexed by property name and where the value is an
  * object having the cssRule property.
  */
 function getDefinedGeometryProperties(node) {
-  let props = new Map();
+  const props = new Map();
   if (!node) {
     return props;
   }
 
   // Get the list of css rules applying to the current node.
-  let cssRules = getCSSStyleRules(node);
+  const cssRules = getMatchingCSSRules(node);
   for (let i = 0; i < cssRules.length; i++) {
-    let rule = cssRules[i];
-    for (let name of GeoProp.allProps()) {
-      let value = rule.style.getPropertyValue(name);
+    const rule = cssRules[i];
+    for (const name of GeoProp.allProps()) {
+      const value = rule.style.getPropertyValue(name);
       if (value && value !== "auto") {
-        // getCSSStyleRules returns rules ordered from least to most specific
+        // getMatchingCSSRules returns rules ordered from least to most specific
         // so just override any previous properties we have set.
         props.set(name, {
-          cssRule: rule
+          cssRule: rule,
         });
       }
     }
@@ -148,13 +157,13 @@ function getDefinedGeometryProperties(node) {
   // Go through the inline styles last, only if the node supports inline style
   // (e.g. pseudo elements don't have a style property)
   if (node.style) {
-    for (let name of GeoProp.allProps()) {
-      let value = node.style.getPropertyValue(name);
+    for (const name of GeoProp.allProps()) {
+      const value = node.style.getPropertyValue(name);
       if (value && value !== "auto") {
         props.set(name, {
           // There's no cssRule to store here, so store the node instead since
           // node.style exists.
-          cssRule: node
+          cssRule: node,
         });
       }
     }
@@ -164,17 +173,17 @@ function getDefinedGeometryProperties(node) {
   // because of cases like relative positioning with both top and bottom where
   // only top will actually be used, but both exists in css rules and computed
   // styles.
-  let { position } = getComputedStyle(node);
-  for (let [name] of props) {
+  const { position } = getComputedStyle(node);
+  for (const [name] of props) {
     // Top/left/bottom/right on static positioned elements have no effect.
-    if (position === "static" && GeoProp.SIDES.indexOf(name) !== -1) {
+    if (position === "static" && GeoProp.SIDES.includes(name)) {
       props.delete(name);
     }
 
     // Bottom/right on relative positioned elements are only used if top/left
     // are not defined.
-    let hasRightAndLeft = name === "right" && props.has("left");
-    let hasBottomAndTop = name === "bottom" && props.has("top");
+    const hasRightAndLeft = name === "right" && props.has("left");
+    const hasBottomAndTop = name === "bottom" && props.has("top");
     if (position === "relative" && (hasRightAndLeft || hasBottomAndTop)) {
       props.delete(name);
     }
@@ -206,155 +215,155 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
   constructor(highlighterEnv) {
     super(highlighterEnv);
 
-    this.ID_CLASS_PREFIX = "geometry-editor-";
-
     // The list of element geometry properties that can be set.
     this.definedProperties = new Map();
 
-    this.markup = new CanvasFrameAnonymousContentHelper(highlighterEnv,
-      this._buildMarkup.bind(this));
+    this.markup = new CanvasFrameAnonymousContentHelper(
+      highlighterEnv,
+      this._buildMarkup.bind(this),
+      {
+        contentRootHostClassName: "devtools-highlighter-geometry-editor",
+      }
+    );
+    this.isReady = this.initialize();
 
-    let { pageListenerTarget } = this.highlighterEnv;
+    const { pageListenerTarget } = this.highlighterEnv;
 
     // Register the geometry editor instance to all events we're interested in.
     DOM_EVENTS.forEach(type => pageListenerTarget.addEventListener(type, this));
 
-    // Register the mousedown event for each Geometry Editor's handler.
-    // Those events are automatically removed when the markup is destroyed.
-    let onMouseDown = this.handleEvent.bind(this);
-
-    for (let side of GeoProp.SIDES) {
-      this.getElement("handler-" + side)
-        .addEventListener("mousedown", onMouseDown);
-    }
-
     this.onWillNavigate = this.onWillNavigate.bind(this);
 
     this.highlighterEnv.on("will-navigate", this.onWillNavigate);
   }
 
+  async initialize() {
+    await this.markup.initialize();
+    // Register the mousedown event for each Geometry Editor's handler.
+    // Those events are automatically removed when the markup is destroyed.
+    const onMouseDown = this.handleEvent.bind(this);
+
+    for (const side of GeoProp.SIDES) {
+      this.getElement("geometry-editor-handler-" + side).addEventListener(
+        "mousedown",
+        onMouseDown
+      );
+    }
+  }
+
   _buildMarkup() {
-    let container = createNode(this.win, {
-      attributes: {"class": "highlighter-container"}
+    const container = this.markup.createNode({
+      attributes: { class: "highlighter-container" },
     });
 
-    let root = createNode(this.win, {
+    this.rootEl = this.markup.createNode({
       parent: container,
       attributes: {
-        "id": "root",
-        "class": "root",
-        "hidden": "true"
+        id: "geometry-editor-root",
+        class: "geometry-editor-root",
+        hidden: "true",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
-    let svg = createSVGNode(this.win, {
+    const svg = this.markup.createSVGNode({
       nodeType: "svg",
-      parent: root,
+      parent: this.rootEl,
       attributes: {
-        "id": "elements",
-        "width": "100%",
-        "height": "100%"
+        id: "geometry-editor-elements",
+        width: "100%",
+        height: "100%",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     // Offset parent node highlighter.
-    createSVGNode(this.win, {
+    this.markup.createSVGNode({
       nodeType: "polygon",
       parent: svg,
       attributes: {
-        "class": "offset-parent",
-        "id": "offset-parent",
-        "hidden": "true"
+        class: "geometry-editor-offset-parent",
+        id: "geometry-editor-offset-parent",
+        hidden: "true",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     // Current node highlighter (margin box).
-    createSVGNode(this.win, {
+    this.markup.createSVGNode({
       nodeType: "polygon",
       parent: svg,
       attributes: {
-        "class": "current-node",
-        "id": "current-node",
-        "hidden": "true"
+        class: "geometry-editor-current-node",
+        id: "geometry-editor-current-node",
+        hidden: "true",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     // Build the 4 side arrows, handlers and labels.
-    for (let name of GeoProp.SIDES) {
-      createSVGNode(this.win, {
+    for (const name of GeoProp.SIDES) {
+      this.markup.createSVGNode({
         nodeType: "line",
         parent: svg,
         attributes: {
-          "class": "arrow " + name,
-          "id": "arrow-" + name,
-          "hidden": "true"
+          class: "geometry-editor-arrow " + name,
+          id: "geometry-editor-arrow-" + name,
+          hidden: "true",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
 
-      createSVGNode(this.win, {
+      this.markup.createSVGNode({
         nodeType: "circle",
         parent: svg,
         attributes: {
-          "class": "handler-" + name,
-          "id": "handler-" + name,
-          "r": "4",
+          class: "geometry-editor-handler-" + name,
+          id: "geometry-editor-handler-" + name,
+          r: "4",
           "data-side": name,
-          "hidden": "true"
+          hidden: "true",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
 
       // Labels are positioned by using a translated <g>. This group contains
       // a path and text that are themselves positioned using another translated
       // <g>. This is so that the label arrow points at the 0,0 coordinates of
       // parent <g>.
-      let labelG = createSVGNode(this.win, {
+      const labelG = this.markup.createSVGNode({
         nodeType: "g",
         parent: svg,
         attributes: {
-          "id": "label-" + name,
-          "hidden": "true"
+          id: "geometry-editor-label-" + name,
+          hidden: "true",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
 
-      let subG = createSVGNode(this.win, {
+      const subG = this.markup.createSVGNode({
         nodeType: "g",
         parent: labelG,
         attributes: {
-          "transform": GeoProp.isHorizontal(name)
-                       ? "translate(-30 -30)"
-                       : "translate(5 -10)"
-        }
+          transform: GeoProp.isHorizontal(name)
+            ? "translate(-30 -30)"
+            : "translate(5 -10)",
+        },
       });
 
-      createSVGNode(this.win, {
+      this.markup.createSVGNode({
         nodeType: "path",
         parent: subG,
         attributes: {
-          "class": "label-bubble",
-          "d": GeoProp.isHorizontal(name)
-               ? "M0 0 L60 0 L60 20 L35 20 L30 25 L25 20 L0 20z"
-               : "M5 0 L65 0 L65 20 L5 20 L5 15 L0 10 L5 5z"
+          class: "geometry-editor-label-bubble",
+          d: GeoProp.isHorizontal(name)
+            ? "M0 0 L60 0 L60 20 L35 20 L30 25 L25 20 L0 20z"
+            : "M5 0 L65 0 L65 20 L5 20 L5 15 L0 10 L5 5z",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
 
-      createSVGNode(this.win, {
+      this.markup.createSVGNode({
         nodeType: "text",
         parent: subG,
         attributes: {
-          "class": "label-text",
-          "id": "label-text-" + name,
-          "x": GeoProp.isHorizontal(name) ? "30" : "35",
-          "y": "10"
+          class: "geometry-editor-label-text",
+          id: "geometry-editor-label-text-" + name,
+          x: GeoProp.isHorizontal(name) ? "30" : "35",
+          y: "10",
         },
-        prefix: this.ID_CLASS_PREFIX
       });
     }
 
@@ -368,16 +377,19 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
       return;
     }
 
-    let { pageListenerTarget } = this.highlighterEnv;
+    const { pageListenerTarget } = this.highlighterEnv;
 
     if (pageListenerTarget) {
       DOM_EVENTS.forEach(type =>
-        pageListenerTarget.removeEventListener(type, this));
+        pageListenerTarget.removeEventListener(type, this)
+      );
     }
 
     AutoRefreshHighlighter.prototype.destroy.call(this);
 
     this.markup.destroy();
+    this.rootEl = null;
+
     this.definedProperties.clear();
     this.definedProperties = null;
     this.offsetParent = null;
@@ -385,7 +397,7 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
 
   handleEvent(event, id) {
     // No event handling if the highlighter is hidden
-    if (this.getElement("root").hasAttribute("hidden")) {
+    if (this.getElement("geometry-editor-root").hasAttribute("hidden")) {
       return;
     }
 
@@ -406,25 +418,27 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
           return;
         }
 
-        let handlerSide = this.markup.getElement(id).getAttribute("data-side");
+        const handlerSide = this.markup
+          .getElement(id)
+          .getAttribute("data-side");
 
         if (handlerSide) {
-          let side = handlerSide;
-          let sideProp = this.definedProperties.get(side);
+          const side = handlerSide;
+          const sideProp = this.definedProperties.get(side);
 
           if (!sideProp) {
             return;
           }
 
           let value = sideProp.cssRule.style.getPropertyValue(side);
-          let computedValue = this.computedStyle.getPropertyValue(side);
+          const computedValue = this.computedStyle.getPropertyValue(side);
 
-          let [unit] = value.match(/[^\d]+$/) || [""];
+          const [unit] = value.match(/[^\d]+$/) || [""];
 
           value = parseFloat(value);
 
-          let ratio = (value / parseFloat(computedValue)) || 1;
-          let dir = GeoProp.isInverted(side) ? -1 : 1;
+          const ratio = value / parseFloat(computedValue) || 1;
+          const dir = GeoProp.isInverted(side) ? -1 : 1;
 
           // Store all the initial values needed for drag & drop
           this[_dragging] = {
@@ -433,20 +447,27 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
             unit,
             x: pageX,
             y: pageY,
-            inc: ratio * dir
+            inc: ratio * dir,
           };
 
-          this.getElement("handler-" + side).classList.add("dragging");
+          this.getElement("geometry-editor-handler-" + side).classList?.add(
+            "dragging"
+          );
         }
 
-        this.getElement("root").setAttribute("dragging", "true");
+        this.getElement("geometry-editor-root").setAttribute(
+          "dragging",
+          "true"
+        );
         break;
       case "mouseup":
         // If we're dragging, drop it.
         if (this[_dragging]) {
-          let { side } = this[_dragging];
-          this.getElement("root").removeAttribute("dragging");
-          this.getElement("handler-" + side).classList.remove("dragging");
+          const { side } = this[_dragging];
+          this.getElement("geometry-editor-root").removeAttribute("dragging");
+          this.getElement("geometry-editor-handler-" + side).classList?.remove(
+            "dragging"
+          );
           this[_dragging] = null;
         }
         break;
@@ -455,14 +476,15 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
           return;
         }
 
-        let { side, x, y, value, unit, inc } = this[_dragging];
-        let sideProps = this.definedProperties.get(side);
+        const { side, x, y, value, unit, inc } = this[_dragging];
+        const sideProps = this.definedProperties.get(side);
 
         if (!sideProps) {
           return;
         }
 
-        let delta = (GeoProp.isHorizontal(side) ? pageX - x : pageY - y) * inc;
+        const delta =
+          (GeoProp.isHorizontal(side) ? pageX - x : pageY - y) * inc;
 
         // The inline style has usually the priority over any other CSS rule
         // set in stylesheets. However, if a rule has `!important` keyword,
@@ -470,32 +492,35 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
         // will always update the element, we have to add `!important` as
         // well.
         this.currentNode.style.setProperty(
-          side, (value + delta) + unit, "important");
+          side,
+          value + delta + unit,
+          "important"
+        );
 
         break;
     }
   }
 
   getElement(id) {
-    return this.markup.getElement(this.ID_CLASS_PREFIX + id);
+    return this.markup.getElement(id);
   }
 
   _show() {
     this.computedStyle = getComputedStyle(this.currentNode);
-    let pos = this.computedStyle.position;
+    const pos = this.computedStyle.position;
     // XXX: sticky positioning is ignored for now. To be implemented next.
     if (pos === "sticky") {
       this.hide();
       return false;
     }
 
-    let hasUpdated = this._update();
+    const hasUpdated = this._update();
     if (!hasUpdated) {
       this.hide();
       return false;
     }
 
-    this.getElement("root").removeAttribute("hidden");
+    this.getElement("geometry-editor-root").removeAttribute("hidden");
 
     return true;
   }
@@ -504,6 +529,9 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
     // At each update, the position or/and size may have changed, so get the
     // list of defined properties, and re-position the arrows and highlighters.
     this.definedProperties = getDefinedGeometryProperties(this.currentNode);
+    // We need the zoom factor to fix the original position of the node
+    // as well as the arrows.
+    this.zoomFactor = getCurrentZoom(this.currentNode);
 
     if (!this.definedProperties.size) {
       console.warn("The element does not have editable geometry properties");
@@ -518,8 +546,8 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
     this.updateArrows();
 
     // Avoid zooming the arrows when content is zoomed.
-    let node = this.currentNode;
-    this.markup.scaleRootElement(node, this.ID_CLASS_PREFIX + "root");
+    const node = this.currentNode;
+    this.markup.scaleRootElement(node, "geometry-editor-root");
 
     setIgnoreLayoutChanges(false, this.highlighterEnv.document.documentElement);
     return true;
@@ -542,32 +570,61 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
     this.offsetParent = getOffsetParent(this.currentNode);
     // And the offsetParent quads.
     this.parentQuads = getAdjustedQuads(
-        this.win, this.offsetParent.element, "padding");
+      this.win,
+      this.offsetParent.element,
+      "padding"
+    );
 
-    let el = this.getElement("offset-parent");
+    const el = this.getElement("geometry-editor-offset-parent");
 
-    let isPositioned = this.computedStyle.position === "absolute" ||
-                       this.computedStyle.position === "fixed";
-    let isRelative = this.computedStyle.position === "relative";
+    const isPositioned =
+      this.computedStyle.position === "absolute" ||
+      this.computedStyle.position === "fixed";
+    const isRelative = this.computedStyle.position === "relative";
     let isHighlighted = false;
 
     if (this.offsetParent.element && isPositioned) {
-      let {p1, p2, p3, p4} = this.parentQuads[0];
-      let points = p1.x + "," + p1.y + " " +
-                   p2.x + "," + p2.y + " " +
-                   p3.x + "," + p3.y + " " +
-                   p4.x + "," + p4.y;
+      const { p1, p2, p3, p4 } = this.parentQuads[0];
+      const points =
+        p1.x +
+        "," +
+        p1.y +
+        " " +
+        p2.x +
+        "," +
+        p2.y +
+        " " +
+        p3.x +
+        "," +
+        p3.y +
+        " " +
+        p4.x +
+        "," +
+        p4.y;
       el.setAttribute("points", points);
       isHighlighted = true;
     } else if (isRelative) {
-      let xDelta = parseFloat(this.computedStyle.left);
-      let yDelta = parseFloat(this.computedStyle.top);
+      const xDelta = parseFloat(this.computedStyle.left) * this.zoomFactor;
+      const yDelta = parseFloat(this.computedStyle.top) * this.zoomFactor;
       if (xDelta || yDelta) {
-        let {p1, p2, p3, p4} = this.currentQuads.margin[0];
-        let points = (p1.x - xDelta) + "," + (p1.y - yDelta) + " " +
-                     (p2.x - xDelta) + "," + (p2.y - yDelta) + " " +
-                     (p3.x - xDelta) + "," + (p3.y - yDelta) + " " +
-                     (p4.x - xDelta) + "," + (p4.y - yDelta);
+        const { p1, p2, p3, p4 } = this.currentQuads.margin[0];
+        const points =
+          p1.x -
+          xDelta +
+          "," +
+          (p1.y - yDelta) +
+          " " +
+          (p2.x - xDelta) +
+          "," +
+          (p2.y - yDelta) +
+          " " +
+          (p3.x - xDelta) +
+          "," +
+          (p3.y - yDelta) +
+          " " +
+          (p4.x - xDelta) +
+          "," +
+          (p4.y - yDelta);
         el.setAttribute("points", points);
         isHighlighted = true;
       }
@@ -581,12 +638,24 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
   }
 
   updateCurrentNode() {
-    let box = this.getElement("current-node");
-    let {p1, p2, p3, p4} = this.currentQuads.margin[0];
-    let attr = p1.x + "," + p1.y + " " +
-               p2.x + "," + p2.y + " " +
-               p3.x + "," + p3.y + " " +
-               p4.x + "," + p4.y;
+    const box = this.getElement("geometry-editor-current-node");
+    const { p1, p2, p3, p4 } = this.currentQuads.margin[0];
+    const attr =
+      p1.x +
+      "," +
+      p1.y +
+      " " +
+      p2.x +
+      "," +
+      p2.y +
+      " " +
+      p3.x +
+      "," +
+      p3.y +
+      " " +
+      p4.x +
+      "," +
+      p4.y;
     box.setAttribute("points", attr);
     box.removeAttribute("hidden");
   }
@@ -594,9 +663,15 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
   _hide() {
     setIgnoreLayoutChanges(true);
 
-    this.getElement("root").setAttribute("hidden", "true");
-    this.getElement("current-node").setAttribute("hidden", "true");
-    this.getElement("offset-parent").setAttribute("hidden", "true");
+    this.getElement("geometry-editor-root").setAttribute("hidden", "true");
+    this.getElement("geometry-editor-current-node").setAttribute(
+      "hidden",
+      "true"
+    );
+    this.getElement("geometry-editor-offset-parent").setAttribute(
+      "hidden",
+      "true"
+    );
     this.hideArrows();
 
     this.definedProperties.clear();
@@ -605,10 +680,19 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
   }
 
   hideArrows() {
-    for (let side of GeoProp.SIDES) {
-      this.getElement("arrow-" + side).setAttribute("hidden", "true");
-      this.getElement("label-" + side).setAttribute("hidden", "true");
-      this.getElement("handler-" + side).setAttribute("hidden", "true");
+    for (const side of GeoProp.SIDES) {
+      this.getElement("geometry-editor-arrow-" + side).setAttribute(
+        "hidden",
+        "true"
+      );
+      this.getElement("geometry-editor-label-" + side).setAttribute(
+        "hidden",
+        "true"
+      );
+      this.getElement("geometry-editor-handler-" + side).setAttribute(
+        "hidden",
+        "true"
+      );
     }
   }
 
@@ -616,7 +700,7 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
     this.hideArrows();
 
     // Position arrows always end at the node's margin box.
-    let marginBox = this.currentQuads.margin[0].bounds;
+    const marginBox = this.currentQuads.margin[0].bounds;
 
     // Position the side arrows which need to be visible.
     // Arrows always start at the offsetParent edge, and end at the middle
@@ -633,49 +717,65 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
     // |         +--------+--------+       |
     // |                  | bottom         |
     // +------------------+----------------+
-    let getSideArrowStartPos = side => {
-      // In case an offsetParent exists and is highlighted.
-      if (this.parentQuads && this.parentQuads.length) {
-        return this.parentQuads[0].bounds[side];
-      }
-
+    const getSideArrowStartPos = side => {
       // In case of relative positioning.
       if (this.computedStyle.position === "relative") {
         if (GeoProp.isInverted(side)) {
-          return marginBox[side] + parseFloat(this.computedStyle[side]);
+          return (
+            marginBox[side] +
+            parseFloat(this.computedStyle[side]) * this.zoomFactor
+          );
         }
-        return marginBox[side] - parseFloat(this.computedStyle[side]);
+        return (
+          marginBox[side] -
+          parseFloat(this.computedStyle[side]) * this.zoomFactor
+        );
+      }
+
+      // In case an offsetParent exists and is highlighted.
+      if (this.parentQuads && this.parentQuads.length) {
+        return this.parentQuads[0].bounds[side];
       }
 
       // In case the element is positioned in the viewport.
       if (GeoProp.isInverted(side)) {
         return this.offsetParent.dimension[GeoProp.mainAxisSize(side)];
       }
-      return -1 * this.currentNode.ownerGlobal["scroll" +
-                                              GeoProp.axis(side).toUpperCase()];
+      return (
+        -1 *
+        this.currentNode.ownerGlobal[
+          "scroll" + GeoProp.axis(side).toUpperCase()
+        ]
+      );
     };
 
-    for (let side of GeoProp.SIDES) {
-      let sideProp = this.definedProperties.get(side);
+    for (const side of GeoProp.SIDES) {
+      const sideProp = this.definedProperties.get(side);
       if (!sideProp) {
         continue;
       }
 
-      let mainAxisStartPos = getSideArrowStartPos(side);
-      let mainAxisEndPos = marginBox[side];
-      let crossAxisPos = marginBox[GeoProp.crossAxisStart(side)] +
-                         marginBox[GeoProp.crossAxisSize(side)] / 2;
+      const mainAxisStartPos = getSideArrowStartPos(side);
+      const mainAxisEndPos = marginBox[side];
+      const crossAxisPos =
+        marginBox[GeoProp.crossAxisStart(side)] +
+        marginBox[GeoProp.crossAxisSize(side)] / 2;
 
-      this.updateArrow(side, mainAxisStartPos, mainAxisEndPos, crossAxisPos,
-                       sideProp.cssRule.style.getPropertyValue(side));
+      this.updateArrow(
+        side,
+        mainAxisStartPos,
+        mainAxisEndPos,
+        crossAxisPos,
+        sideProp.cssRule.style.getPropertyValue(side)
+      );
     }
   }
 
   updateArrow(side, mainStart, mainEnd, crossPos, labelValue) {
-    let arrowEl = this.getElement("arrow-" + side);
-    let labelEl = this.getElement("label-" + side);
-    let labelTextEl = this.getElement("label-text-" + side);
-    let handlerEl = this.getElement("handler-" + side);
+    const arrowEl = this.getElement("geometry-editor-arrow-" + side);
+    const labelEl = this.getElement("geometry-editor-label-" + side);
+    const labelTextEl = this.getElement("geometry-editor-label-text-" + side);
+    const handlerEl = this.getElement("geometry-editor-handler-" + side);
 
     // Position the arrow <line>.
     arrowEl.setAttribute(GeoProp.axis(side) + "1", mainStart);
@@ -690,21 +790,26 @@ class GeometryEditorHighlighter extends AutoRefreshHighlighter {
 
     // Position the label <text> in the middle of the arrow (making sure it's
     // not hidden below the fold).
-    let capitalize = str => str[0].toUpperCase() + str.substring(1);
-    let winMain = this.win["inner" + capitalize(GeoProp.mainAxisSize(side))];
+    const capitalize = str => str[0].toUpperCase() + str.substring(1);
+    const winMain = this.win["inner" + capitalize(GeoProp.mainAxisSize(side))];
     let labelMain = mainStart + (mainEnd - mainStart) / 2;
-    if ((mainStart > 0 && mainStart < winMain) ||
-        (mainEnd > 0 && mainEnd < winMain)) {
+    if (
+      (mainStart > 0 && mainStart < winMain) ||
+      (mainEnd > 0 && mainEnd < winMain)
+    ) {
       if (labelMain < GEOMETRY_LABEL_SIZE) {
         labelMain = GEOMETRY_LABEL_SIZE;
       } else if (labelMain > winMain - GEOMETRY_LABEL_SIZE) {
         labelMain = winMain - GEOMETRY_LABEL_SIZE;
       }
     }
-    let labelCross = crossPos;
-    labelEl.setAttribute("transform", GeoProp.isHorizontal(side)
-                         ? "translate(" + labelMain + " " + labelCross + ")"
-                         : "translate(" + labelCross + " " + labelMain + ")");
+    const labelCross = crossPos;
+    labelEl.setAttribute(
+      "transform",
+      GeoProp.isHorizontal(side)
+        ? "translate(" + labelMain + " " + labelCross + ")"
+        : "translate(" + labelCross + " " + labelMain + ")"
+    );
     labelEl.removeAttribute("hidden");
     labelTextEl.setTextContent(labelValue);
   }
