@@ -4,7 +4,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "FrameAnimator.h"
-#include "GeckoProfiler.h"
 
 #include <utility>
 
@@ -12,6 +11,7 @@
 #include "RasterImage.h"
 #include "imgIContainer.h"
 #include "mozilla/CheckedInt.h"
+#include "mozilla/ProfilerLabels.h"
 #include "mozilla/StaticPrefs_image.h"
 
 namespace mozilla {
@@ -318,7 +318,6 @@ RefreshResult FrameAnimator::AdvanceFrame(AnimationState& aState,
 
   // Set currentAnimationFrameIndex at the last possible moment
   aState.mCurrentAnimationFrameIndex = nextFrameIndex;
-  aState.mCompositedFrameRequested = false;
   aCurrentFrame = std::move(nextFrame);
   aFrames.Advance(nextFrameIndex);
 
@@ -333,19 +332,23 @@ void FrameAnimator::ResetAnimation(AnimationState& aState) {
 
   // Our surface provider is synchronized to our state, so we need to reset its
   // state as well, if we still have one.
-  LookupResult result = SurfaceCache::Lookup(
+  SurfaceCache::ResetAnimation(
       ImageKey(mImage),
-      RasterSurfaceKey(mSize, DefaultSurfaceFlags(), PlaybackType::eAnimated),
-      /* aMarkUsed = */ false);
-  if (!result) {
-    return;
-  }
-
-  result.Surface().Reset();
+      RasterSurfaceKey(mSize, DefaultSurfaceFlags(), PlaybackType::eAnimated));
 
   // Calling Reset on the surface of the animation can cause discarding surface
   // providers to throw out all their frames so refresh our state.
-  aState.UpdateStateInternal(result, mSize);
+  OrientedIntRect rect =
+      OrientedIntRect::FromUnknownRect(aState.UpdateState(mImage, mSize));
+
+  if (!rect.IsEmpty()) {
+    nsCOMPtr<nsIEventTarget> eventTarget = do_GetMainThread();
+    RefPtr<RasterImage> image = mImage;
+    nsCOMPtr<nsIRunnable> ev = NS_NewRunnableFunction(
+        "FrameAnimator::ResetAnimation",
+        [=]() -> void { image->NotifyProgress(NoProgress, rect); });
+    eventTarget->Dispatch(ev.forget(), NS_DISPATCH_NORMAL);
+  }
 }
 
 RefreshResult FrameAnimator::RequestRefresh(AnimationState& aState,
@@ -395,7 +398,7 @@ RefreshResult FrameAnimator::RequestRefresh(AnimationState& aState,
   // If nothing has accessed the composited frame since the last time we
   // advanced, then there is no point in continuing to advance the animation.
   // This has the effect of freezing the animation while not in view.
-  if (!aState.mCompositedFrameRequested &&
+  if (!result.Surface().MayAdvance() &&
       aState.MaybeAdvanceAnimationFrameTime(aTime)) {
     return ret;
   }
@@ -443,13 +446,18 @@ RefreshResult FrameAnimator::RequestRefresh(AnimationState& aState,
 
 LookupResult FrameAnimator::GetCompositedFrame(AnimationState& aState,
                                                bool aMarkUsed) {
-  aState.mCompositedFrameRequested = true;
-
   LookupResult result = SurfaceCache::Lookup(
       ImageKey(mImage),
       RasterSurfaceKey(mSize, DefaultSurfaceFlags(), PlaybackType::eAnimated),
       aMarkUsed);
 
+  if (result) {
+    // If we are getting the frame directly (e.g. through tests or canvas), we
+    // need to ensure the animation is marked to allow advancing to the next
+    // frame.
+    result.Surface().MarkMayAdvance();
+  }
+
   if (aState.mCompositedFrameInvalid) {
     MOZ_ASSERT(StaticPrefs::image_mem_animated_discardable_AtStartup());
     MOZ_ASSERT(aState.GetHasRequestedDecode());
@@ -467,7 +475,7 @@ LookupResult FrameAnimator::GetCompositedFrame(AnimationState& aState,
       // getting called which calls UpdateState. The reason we care about this
       // is that img.decode promises won't resolve until GetCompositedFrame
       // returns a frame.
-      UnorientedIntRect rect = UnorientedIntRect::FromUnknownRect(
+      OrientedIntRect rect = OrientedIntRect::FromUnknownRect(
           aState.UpdateStateInternal(result, mSize));
 
       if (!rect.IsEmpty()) {
