@@ -6,7 +6,6 @@
 
 #include "imgFrame.h"
 #include "ImageRegion.h"
-#include "ShutdownTracker.h"
 #include "SurfaceCache.h"
 
 #include "prenv.h"
@@ -17,18 +16,13 @@
 
 #include "gfxUtils.h"
 
-#include "GeckoProfiler.h"
 #include "MainThreadUtils.h"
 #include "mozilla/CheckedInt.h"
-#include "mozilla/gfx/gfxVars.h"
 #include "mozilla/gfx/Tools.h"
-#include "mozilla/gfx/SourceSurfaceRawData.h"
-#include "mozilla/layers/SourceSurfaceSharedData.h"
-#include "mozilla/layers/SourceSurfaceVolatileData.h"
 #include "mozilla/Likely.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/ProfilerLabels.h"
 #include "mozilla/StaticPrefs_browser.h"
-#include "mozilla/StaticPrefs_image.h"
 #include "nsMargin.h"
 #include "nsRefreshDriver.h"
 #include "nsThreadUtils.h"
@@ -56,90 +50,25 @@ class RecyclingSourceSurfaceSharedData final : public SourceSurfaceSharedData {
   }
 };
 
-static int32_t VolatileSurfaceStride(const IntSize& size,
-                                     SurfaceFormat format) {
+static already_AddRefed<SourceSurfaceSharedData> AllocateBufferForImage(
+    const IntSize& size, SurfaceFormat format, bool aShouldRecycle = false) {
   // Stride must be a multiple of four or cairo will complain.
-  return (size.width * BytesPerPixel(format) + 0x3) & ~0x3;
-}
+  int32_t stride = (size.width * BytesPerPixel(format) + 0x3) & ~0x3;
 
-static already_AddRefed<DataSourceSurface> CreateLockedSurface(
-    DataSourceSurface* aSurface, const IntSize& size, SurfaceFormat format) {
-  switch (aSurface->GetType()) {
-    case SurfaceType::DATA_SHARED:
-    case SurfaceType::DATA_RECYCLING_SHARED:
-    case SurfaceType::DATA_ALIGNED: {
-      // Shared memory is never released until the surface itself is released.
-      // Similar for aligned/heap surfaces.
-      RefPtr<DataSourceSurface> surf(aSurface);
-      return surf.forget();
-    }
-    default: {
-      // Volatile memory requires us to map it first, and it is fallible.
-      DataSourceSurface::ScopedMap smap(aSurface,
-                                        DataSourceSurface::READ_WRITE);
-      if (smap.IsMapped()) {
-        return MakeAndAddRef<SourceSurfaceMappedData>(std::move(smap), size,
-                                                      format);
-      }
-      break;
-    }
-  }
-
-  return nullptr;
-}
-
-static bool ShouldUseHeap(const IntSize& aSize, int32_t aStride,
-                          bool aIsAnimated) {
-  // On some platforms (i.e. Android), a volatile buffer actually keeps a file
-  // handle active. We would like to avoid too many since we could easily
-  // exhaust the pool. However, other platforms we do not have the file handle
-  // problem, and additionally we may avoid a superfluous memset since the
-  // volatile memory starts out as zero-filled. Hence the knobs below.
-
-  // For as long as an animated image is retained, its frames will never be
-  // released to let the OS purge volatile buffers.
-  if (aIsAnimated && StaticPrefs::image_mem_animated_use_heap()) {
-    return true;
-  }
-
-  // Lets us avoid too many small images consuming all of the handles. The
-  // actual allocation checks for overflow.
-  int32_t bufferSize = (aStride * aSize.height) / 1024;
-  return bufferSize < StaticPrefs::image_mem_volatile_min_threshold_kb();
-}
-
-static already_AddRefed<DataSourceSurface> AllocateBufferForImage(
-    const IntSize& size, SurfaceFormat format, bool aShouldRecycle = false,
-    bool aIsAnimated = false) {
-  int32_t stride = VolatileSurfaceStride(size, format);
-
-  if (gfxVars::GetUseWebRenderOrDefault() && StaticPrefs::image_mem_shared()) {
-    RefPtr<SourceSurfaceSharedData> newSurf;
-    if (aShouldRecycle) {
-      newSurf = new RecyclingSourceSurfaceSharedData();
-    } else {
-      newSurf = new SourceSurfaceSharedData();
-    }
-    if (newSurf->Init(size, stride, format)) {
-      return newSurf.forget();
-    }
-  } else if (ShouldUseHeap(size, stride, aIsAnimated)) {
-    RefPtr<SourceSurfaceAlignedRawData> newSurf =
-        new SourceSurfaceAlignedRawData();
-    if (newSurf->Init(size, format, false, 0, stride)) {
-      return newSurf.forget();
-    }
+  RefPtr<SourceSurfaceSharedData> newSurf;
+  if (aShouldRecycle) {
+    newSurf = new RecyclingSourceSurfaceSharedData();
   } else {
-    RefPtr<SourceSurfaceVolatileData> newSurf = new SourceSurfaceVolatileData();
-    if (newSurf->Init(size, stride, format)) {
-      return newSurf.forget();
-    }
+    newSurf = new SourceSurfaceSharedData();
   }
-  return nullptr;
+  if (!newSurf->Init(size, stride, format)) {
+    return nullptr;
+  }
+  return newSurf.forget();
 }
 
-static bool GreenSurface(DataSourceSurface* aSurface, const IntSize& aSize,
-                         SurfaceFormat aFormat) {
+static bool GreenSurface(SourceSurfaceSharedData* aSurface,
+                         const IntSize& aSize, SurfaceFormat aFormat) {
   int32_t stride = aSurface->Stride();
   uint32_t* surfaceData = reinterpret_cast<uint32_t*>(aSurface->GetData());
   uint32_t surfaceDataLength = (stride * aSize.height) / sizeof(uint32_t);
@@ -171,8 +100,8 @@ static bool GreenSurface(DataSourceSurface* aSurface, const IntSize& aSize,
   return true;
 }
 
-static bool ClearSurface(DataSourceSurface* aSurface, const IntSize& aSize,
-                         SurfaceFormat aFormat) {
+static bool ClearSurface(SourceSurfaceSharedData* aSurface,
+                         const IntSize& aSize, SurfaceFormat aFormat) {
   int32_t stride = aSurface->Stride();
   uint8_t* data = aSurface->GetData();
   MOZ_ASSERT(data);
@@ -196,10 +125,8 @@ static bool ClearSurface(DataSourceSurface* aSurface, const IntSize& aSize,
 imgFrame::imgFrame()
     : mMonitor("imgFrame"),
       mDecoded(0, 0, 0, 0),
-      mLockCount(0),
       mAborted(false),
       mFinished(false),
-      mOptimizable(false),
       mShouldRecycle(false),
       mTimeout(FrameTimeout::FromRawMilliseconds(100)),
       mDisposalMethod(DisposalMethod::NOT_SPECIFIED),
@@ -218,11 +145,13 @@ imgFrame::~imgFrame() {
 nsresult imgFrame::InitForDecoder(const nsIntSize& aImageSize,
                                   SurfaceFormat aFormat, bool aNonPremult,
                                   const Maybe<AnimationParams>& aAnimParams,
-                                  bool aShouldRecycle) {
+                                  bool aShouldRecycle,
+                                  uint32_t* aImageDataLength) {
   // Assert for properties that should be verified by decoders,
   // warn for properties related to bad content.
   if (!SurfaceCache::IsLegalSize(aImageSize)) {
     NS_WARNING("Should have legal image size");
+    MonitorAutoLock lock(mMonitor);
     mAborted = true;
     return NS_ERROR_FAILURE;
   }
@@ -254,13 +183,13 @@ nsresult imgFrame::InitForDecoder(const nsIntSize& aImageSize,
   }
 
   mNonPremult = aNonPremult;
+
+  MonitorAutoLock lock(mMonitor);
   mShouldRecycle = aShouldRecycle;
 
-  MOZ_ASSERT(!mLockedSurface, "Called imgFrame::InitForDecoder() twice?");
+  MOZ_ASSERT(!mRawSurface, "Called imgFrame::InitForDecoder() twice?");
 
-  bool postFirstFrame = aAnimParams && aAnimParams->mFrameNum > 0;
-  mRawSurface = AllocateBufferForImage(mImageSize, mFormat, mShouldRecycle,
-                                       postFirstFrame);
+  mRawSurface = AllocateBufferForImage(mImageSize, mFormat, mShouldRecycle);
   if (!mRawSurface) {
     mAborted = true;
     return NS_ERROR_OUT_OF_MEMORY;
@@ -275,23 +204,6 @@ nsresult imgFrame::InitForDecoder(const nsIntSize& aImageSize,
     }
   }
 
-  mLockedSurface = CreateLockedSurface(mRawSurface, mImageSize, mFormat);
-  if (!mLockedSurface) {
-    NS_WARNING("Failed to create LockedSurface");
-    mAborted = true;
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  if (mBlankRawSurface) {
-    mBlankLockedSurface =
-        CreateLockedSurface(mBlankRawSurface, mImageSize, mFormat);
-    if (!mBlankLockedSurface) {
-      NS_WARNING("Failed to create BlankLockedSurface");
-      mAborted = true;
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-  }
-
   if (!ClearSurface(mRawSurface, mImageSize, mFormat)) {
     NS_WARNING("Could not clear allocated buffer");
     mAborted = true;
@@ -306,16 +218,20 @@ nsresult imgFrame::InitForDecoder(const nsIntSize& aImageSize,
     }
   }
 
+  if (aImageDataLength) {
+    *aImageDataLength = GetImageDataLength();
+  }
+
   return NS_OK;
 }
 
-nsresult imgFrame::InitForDecoderRecycle(const AnimationParams& aAnimParams) {
+nsresult imgFrame::InitForDecoderRecycle(const AnimationParams& aAnimParams,
+                                         uint32_t* aImageDataLength) {
   // We want to recycle this frame, but there is no guarantee that consumers are
   // done with it in a timely manner. Let's ensure they are done with it first.
   MonitorAutoLock lock(mMonitor);
 
-  MOZ_ASSERT(mLockCount > 0);
-  MOZ_ASSERT(mLockedSurface);
+  MOZ_ASSERT(mRawSurface);
 
   if (!mShouldRecycle) {
     // This frame either was never marked as recyclable, or the flag was cleared
@@ -325,14 +241,11 @@ nsresult imgFrame::InitForDecoderRecycle(const AnimationParams& aAnimParams) {
 
   // Ensure we account for all internal references to the surface.
   MozRefCountType internalRefs = 1;
-  if (mRawSurface == mLockedSurface) {
-    ++internalRefs;
-  }
-  if (mOptSurface == mLockedSurface) {
+  if (mOptSurface == mRawSurface) {
     ++internalRefs;
   }
 
-  if (mLockedSurface->refCount() > internalRefs) {
+  if (mRawSurface->refCount() > internalRefs) {
     if (NS_IsMainThread()) {
       // We should never be both decoding and recycling on the main thread. Sync
       // decoding can only be used to produce the first set of frames. Those
@@ -355,14 +268,14 @@ nsresult imgFrame::InitForDecoderRecycle(const AnimationParams& aAnimParams) {
     // is still in use for some other purpose, it won't be returned to the pool
     // and its owner can hold onto it forever without additional impact here.
     int32_t refreshInterval =
-        std::max(std::min(nsRefreshDriver::DefaultInterval(), 20), 4);
+        std::clamp(nsRefreshDriver::DefaultInterval(), 4, 20);
     TimeDuration waitInterval =
         TimeDuration::FromMilliseconds(refreshInterval >> 2);
     TimeStamp timeout =
         TimeStamp::Now() + TimeDuration::FromMilliseconds(refreshInterval);
     while (true) {
       mMonitor.Wait(waitInterval);
-      if (mLockedSurface->refCount() <= internalRefs) {
+      if (mRawSurface->refCount() <= internalRefs) {
         break;
       }
 
@@ -380,6 +293,10 @@ nsresult imgFrame::InitForDecoderRecycle(const AnimationParams& aAnimParams) {
   mDisposalMethod = aAnimParams.mDisposalMethod;
   mDirtyRect = GetRect();
 
+  if (aImageDataLength) {
+    *aImageDataLength = GetImageDataLength();
+  }
+
   return NS_OK;
 }
 
@@ -393,6 +310,7 @@ nsresult imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
   // warn for properties related to bad content.
   if (!SurfaceCache::IsLegalSize(aSize)) {
     NS_WARNING("Should have legal image size");
+    MonitorAutoLock lock(mMonitor);
     mAborted = true;
     return NS_ERROR_FAILURE;
   }
@@ -404,9 +322,10 @@ nsresult imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
 
   bool canUseDataSurface = Factory::DoesBackendSupportDataDrawtarget(aBackend);
   if (canUseDataSurface) {
+    MonitorAutoLock lock(mMonitor);
     // It's safe to use data surfaces for content on this platform, so we can
     // get away with using volatile buffers.
-    MOZ_ASSERT(!mLockedSurface, "Called imgFrame::InitWithDrawable() twice?");
+    MOZ_ASSERT(!mRawSurface, "Called imgFrame::InitWithDrawable() twice?");
 
     mRawSurface = AllocateBufferForImage(mImageSize, mFormat);
     if (!mRawSurface) {
@@ -414,13 +333,6 @@ nsresult imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    mLockedSurface = CreateLockedSurface(mRawSurface, mImageSize, mFormat);
-    if (!mLockedSurface) {
-      NS_WARNING("Failed to create LockedSurface");
-      mAborted = true;
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-
     if (!ClearSurface(mRawSurface, mImageSize, mFormat)) {
       NS_WARNING("Could not clear allocated buffer");
       mAborted = true;
@@ -428,14 +340,18 @@ nsresult imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
     }
 
     target = gfxPlatform::CreateDrawTargetForData(
-        mLockedSurface->GetData(), mImageSize, mLockedSurface->Stride(),
-        mFormat);
+        mRawSurface->GetData(), mImageSize, mRawSurface->Stride(), mFormat);
   } else {
     // We can't use data surfaces for content, so we'll create an offscreen
     // surface instead.  This means if someone later calls RawAccessRef(), we
     // may have to do an expensive readback, but we warned callers about that in
     // the documentation for this method.
-    MOZ_ASSERT(!mOptSurface, "Called imgFrame::InitWithDrawable() twice?");
+#ifdef DEBUG
+    {
+      MonitorAutoLock lock(mMonitor);
+      MOZ_ASSERT(!mOptSurface, "Called imgFrame::InitWithDrawable() twice?");
+    }
+#endif
 
     if (gfxPlatform::GetPlatform()->SupportsAzureContentForType(aBackend)) {
       target = gfxPlatform::GetPlatform()->CreateDrawTargetForBackend(
@@ -447,19 +363,21 @@ nsresult imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
   }
 
   if (!target || !target->IsValid()) {
+    MonitorAutoLock lock(mMonitor);
     mAborted = true;
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
   // Draw using the drawable the caller provided.
-  RefPtr<gfxContext> ctx = gfxContext::CreateOrNull(target);
-  MOZ_ASSERT(ctx);  // Already checked the draw target above.
-  gfxUtils::DrawPixelSnapped(ctx, aDrawable, SizeDouble(mImageSize),
+  gfxContext ctx(target);
+
+  gfxUtils::DrawPixelSnapped(&ctx, aDrawable, SizeDouble(mImageSize),
                              ImageRegion::Create(ThebesRect(GetRect())),
                              mFormat, aSamplingFilter, aImageFlags);
 
-  if (canUseDataSurface && !mLockedSurface) {
-    NS_WARNING("Failed to create VolatileDataSourceSurface");
+  MonitorAutoLock lock(mMonitor);
+  if (canUseDataSurface && !mRawSurface) {
+    NS_WARNING("Failed to create SourceSurfaceSharedData");
     mAborted = true;
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -469,97 +387,23 @@ nsresult imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
     // imgFrame's perspective.
     mOptSurface = target->Snapshot();
   } else {
-    FinalizeSurface();
+    FinalizeSurfaceInternal();
   }
 
   // If we reach this point, we should regard ourselves as complete.
   mDecoded = GetRect();
   mFinished = true;
 
-#ifdef DEBUG
-  MonitorAutoLock lock(mMonitor);
   MOZ_ASSERT(AreAllPixelsWritten());
-#endif
-
-  return NS_OK;
-}
-
-nsresult imgFrame::Optimize(DrawTarget* aTarget) {
-  MOZ_ASSERT(NS_IsMainThread());
-  mMonitor.AssertCurrentThreadOwns();
-
-  if (mLockCount > 0 || !mOptimizable) {
-    // Don't optimize right now.
-    return NS_OK;
-  }
-
-  // Check whether image optimization is disabled -- not thread safe!
-  static bool gDisableOptimize = false;
-  static bool hasCheckedOptimize = false;
-  if (!hasCheckedOptimize) {
-    if (PR_GetEnv("MOZ_DISABLE_IMAGE_OPTIMIZE")) {
-      gDisableOptimize = true;
-    }
-    hasCheckedOptimize = true;
-  }
-
-  // Don't optimize during shutdown because gfxPlatform may not be available.
-  if (ShutdownTracker::ShutdownHasStarted()) {
-    return NS_OK;
-  }
-
-  if (gDisableOptimize) {
-    return NS_OK;
-  }
-
-  if (mOptSurface) {
-    return NS_OK;
-  }
-
-  // XXX(seth): It's currently unclear if there's any reason why we can't
-  // optimize non-premult surfaces. We should look into removing this.
-  if (mNonPremult) {
-    return NS_OK;
-  }
-  if (!gfxVars::UseWebRender()) {
-    mOptSurface = aTarget->OptimizeSourceSurface(mLockedSurface);
-  } else {
-    mOptSurface = gfxPlatform::GetPlatform()
-                      ->ScreenReferenceDrawTarget()
-                      ->OptimizeSourceSurface(mLockedSurface);
-  }
-  if (mOptSurface == mLockedSurface) {
-    mOptSurface = nullptr;
-  }
-
-  if (mOptSurface) {
-    // There's no reason to keep our original surface around if we have an
-    // optimized surface. Release our reference to it. This will leave
-    // |mLockedSurface| as the only thing keeping it alive, so it'll get freed
-    // below.
-    mRawSurface = nullptr;
-  }
-
-  // Release all strong references to the surface's memory. If the underlying
-  // surface is volatile, this will allow the operating system to free the
-  // memory if it needs to.
-  mLockedSurface = nullptr;
-  mOptimizable = false;
 
   return NS_OK;
 }
 
 DrawableFrameRef imgFrame::DrawableRef() { return DrawableFrameRef(this); }
 
-RawAccessFrameRef imgFrame::RawAccessRef(bool aOnlyFinished /*= false*/) {
-  return RawAccessFrameRef(this, aOnlyFinished);
-}
-
-void imgFrame::SetRawAccessOnly() {
-  AssertImageDataLocked();
-
-  // Lock our data and throw away the key.
-  LockImageData(false);
+RawAccessFrameRef imgFrame::RawAccessRef(
+    gfx::DataSourceSurface::MapType aMapType) {
+  return RawAccessFrameRef(this, aMapType);
 }
 
 imgFrame::SurfaceWithFormat imgFrame::SurfaceForDrawing(
@@ -627,11 +471,6 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
   {
     MonitorAutoLock lock(mMonitor);
 
-    // Possibly convert this image into a GPU texture, this may also cause our
-    // mLockedSurface to be released and the OS to release the underlying
-    // memory.
-    Optimize(aContext->GetDrawTarget());
-
     bool doPartialDecode = !AreAllPixelsWritten();
 
     // Most draw targets will just use the surface only during DrawPixelSnapped
@@ -692,31 +531,43 @@ nsresult imgFrame::ImageUpdatedInternal(const nsIntRect& aUpdateRect) {
   if (mRawSurface) {
     mRawSurface->Invalidate(updateRect);
   }
-  if (mLockedSurface && mRawSurface != mLockedSurface) {
-    mLockedSurface->Invalidate(updateRect);
-  }
   return NS_OK;
 }
 
 void imgFrame::Finish(Opacity aFrameOpacity /* = Opacity::SOME_TRANSPARENCY */,
-                      bool aFinalize /* = true */) {
+                      bool aFinalize /* = true */,
+                      bool aOrientationSwapsWidthAndHeight /* = false */) {
   MonitorAutoLock lock(mMonitor);
-  MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
 
   IntRect frameRect(GetRect());
   if (!mDecoded.IsEqualEdges(frameRect)) {
     // The decoder should have produced rows starting from either the bottom or
     // the top of the image. We need to calculate the region for which we have
-    // not yet invalidated.
+    // not yet invalidated. And if the orientation swaps width and height then
+    // its from the left or right.
     IntRect delta(0, 0, frameRect.width, 0);
-    if (mDecoded.y == 0) {
-      delta.y = mDecoded.height;
-      delta.height = frameRect.height - mDecoded.height;
-    } else if (mDecoded.y + mDecoded.height == frameRect.height) {
-      delta.height = frameRect.height - mDecoded.y;
+    if (!aOrientationSwapsWidthAndHeight) {
+      delta.width = frameRect.width;
+      if (mDecoded.y == 0) {
+        delta.y = mDecoded.height;
+        delta.height = frameRect.height - mDecoded.height;
+      } else if (mDecoded.y + mDecoded.height == frameRect.height) {
+        delta.height = frameRect.height - mDecoded.y;
+      } else {
+        MOZ_ASSERT_UNREACHABLE("Decoder only updated middle of image!");
+        delta = frameRect;
+      }
     } else {
-      MOZ_ASSERT_UNREACHABLE("Decoder only updated middle of image!");
-      delta = frameRect;
+      delta.height = frameRect.height;
+      if (mDecoded.x == 0) {
+        delta.x = mDecoded.width;
+        delta.width = frameRect.width - mDecoded.width;
+      } else if (mDecoded.x + mDecoded.width == frameRect.width) {
+        delta.width = frameRect.width - mDecoded.x;
+      } else {
+        MOZ_ASSERT_UNREACHABLE("Decoder only updated middle of image!");
+        delta = frameRect;
+      }
     }
 
     ImageUpdatedInternal(delta);
@@ -748,92 +599,6 @@ uint32_t imgFrame::GetImageDataLength() const {
   return GetImageBytesPerRow() * mImageSize.height;
 }
 
-void imgFrame::GetImageData(uint8_t** aData, uint32_t* aLength) const {
-  MonitorAutoLock lock(mMonitor);
-  GetImageDataInternal(aData, aLength);
-}
-
-void imgFrame::GetImageDataInternal(uint8_t** aData, uint32_t* aLength) const {
-  mMonitor.AssertCurrentThreadOwns();
-  MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
-  MOZ_ASSERT(mLockedSurface);
-
-  if (mLockedSurface) {
-    // TODO: This is okay for now because we only realloc shared surfaces on
-    // the main thread after decoding has finished, but if animations want to
-    // read frame data off the main thread, we will need to reconsider this.
-    *aData = mLockedSurface->GetData();
-    MOZ_ASSERT(
-        *aData,
-        "mLockedSurface is non-null, but GetData is null in GetImageData");
-  } else {
-    *aData = nullptr;
-  }
-
-  *aLength = GetImageDataLength();
-}
-
-uint8_t* imgFrame::GetImageData() const {
-  uint8_t* data;
-  uint32_t length;
-  GetImageData(&data, &length);
-  return data;
-}
-
-uint8_t* imgFrame::LockImageData(bool aOnlyFinished) {
-  MonitorAutoLock lock(mMonitor);
-
-  MOZ_ASSERT(mLockCount >= 0, "Unbalanced locks and unlocks");
-  if (mLockCount < 0 || (aOnlyFinished && !mFinished)) {
-    return nullptr;
-  }
-
-  uint8_t* data;
-  if (mLockedSurface) {
-    data = mLockedSurface->GetData();
-  } else {
-    data = nullptr;
-  }
-
-  // If the raw data is still available, we should get a valid pointer for it.
-  if (!data) {
-    MOZ_ASSERT_UNREACHABLE("It's illegal to re-lock an optimized imgFrame");
-    return nullptr;
-  }
-
-  ++mLockCount;
-  return data;
-}
-
-void imgFrame::AssertImageDataLocked() const {
-#ifdef DEBUG
-  MonitorAutoLock lock(mMonitor);
-  MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
-#endif
-}
-
-nsresult imgFrame::UnlockImageData() {
-  MonitorAutoLock lock(mMonitor);
-
-  MOZ_ASSERT(mLockCount > 0, "Unlocking an unlocked image!");
-  if (mLockCount <= 0) {
-    return NS_ERROR_FAILURE;
-  }
-
-  MOZ_ASSERT(mLockCount > 1 || mFinished || mAborted,
-             "Should have Finish()'d or aborted before unlocking");
-
-  mLockCount--;
-
-  return NS_OK;
-}
-
-void imgFrame::SetOptimizable() {
-  AssertImageDataLocked();
-  MonitorAutoLock lock(mMonitor);
-  mOptimizable = true;
-}
-
 void imgFrame::FinalizeSurface() {
   MonitorAutoLock lock(mMonitor);
   FinalizeSurfaceInternal();
@@ -868,26 +633,16 @@ already_AddRefed<SourceSurface> imgFrame::GetSourceSurfaceInternal() {
     mOptSurface = nullptr;
   }
 
-  if (mBlankLockedSurface) {
+  if (mBlankRawSurface) {
     // We are going to return the blank surface because of the flags.
     // We are including comments here that are copied from below
     // just so that we are on the same page!
-    RefPtr<SourceSurface> surf(mBlankLockedSurface);
+    RefPtr<SourceSurface> surf(mBlankRawSurface);
     return surf.forget();
   }
 
-  if (mLockedSurface) {
-    RefPtr<SourceSurface> surf(mLockedSurface);
-    return surf.forget();
-  }
-
-  MOZ_ASSERT(!mShouldRecycle, "Should recycle but no locked surface!");
-
-  if (!mRawSurface) {
-    return nullptr;
-  }
-
-  return CreateLockedSurface(mRawSurface, mImageSize, mFormat);
+  RefPtr<SourceSurface> surf(mRawSurface);
+  return surf.forget();
 }
 
 void imgFrame::Abort() {
@@ -933,14 +688,8 @@ void imgFrame::AddSizeOfExcludingThis(MallocSizeOf aMallocSizeOf,
   MonitorAutoLock lock(mMonitor);
 
   AddSizeOfCbData metadata;
-
   metadata.mFinished = mFinished;
-  if (mLockedSurface) {
-    // The locked surface should only be present if we have mRawSurface. Hence
-    // we only need to get its allocation size to avoid double counting.
-    metadata.mHeapBytes += aMallocSizeOf(mLockedSurface);
-    metadata.AddType(mLockedSurface->GetType());
-  }
+
   if (mOptSurface) {
     metadata.mHeapBytes += aMallocSizeOf(mOptSurface);
 
