@@ -24,8 +24,6 @@
 #include "nsIInputStream.h"
 #include "nsIMultiPartChannel.h"
 #include "nsIHttpChannel.h"
-#include "nsIApplicationCache.h"
-#include "nsIApplicationCacheChannel.h"
 #include "nsMimeTypes.h"
 
 #include "nsIInterfaceRequestorUtils.h"
@@ -33,13 +31,14 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
+#include "nsEscape.h"
 
-#include "plstr.h"   // PL_strcasestr(...)
 #include "prtime.h"  // for PR_Now
 #include "nsNetUtil.h"
 #include "nsIProtocolHandler.h"
 #include "imgIRequest.h"
 #include "nsProperties.h"
+#include "nsIURL.h"
 
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/SizeOfState.h"
@@ -59,19 +58,20 @@ imgRequest::imgRequest(imgLoader* aLoader, const ImageCacheKey& aCacheKey)
       mLoadId(nullptr),
       mFirstProxy(nullptr),
       mValidator(nullptr),
-      mInnerWindowId(0),
-      mCORSMode(imgIRequest::CORS_NONE),
+      mCORSMode(CORS_NONE),
       mImageErrorCode(NS_OK),
       mImageAvailable(false),
       mIsDeniedCrossSiteCORSRequest(false),
       mIsCrossSiteNoCORSRequest(false),
+      mShouldReportRenderTimeForLCP(false),
       mMutex("imgRequest"),
       mProgressTracker(new ProgressTracker()),
       mIsMultiPartChannel(false),
       mIsInCache(false),
       mDecodeRequested(false),
       mNewPartPending(false),
-      mHadInsecureRedirect(false) {
+      mHadInsecureRedirect(false),
+      mInnerWindowId(0) {
   LOG_FUNC(gImgLog, "imgRequest::imgRequest()");
 }
 
@@ -85,13 +85,15 @@ imgRequest::~imgRequest() {
     LOG_FUNC(gImgLog, "imgRequest::~imgRequest()");
 }
 
-nsresult imgRequest::Init(nsIURI* aURI, nsIURI* aFinalURI,
-                          bool aHadInsecureRedirect, nsIRequest* aRequest,
-                          nsIChannel* aChannel, imgCacheEntry* aCacheEntry,
-                          mozilla::dom::Document* aLoadingDocument,
-                          nsIPrincipal* aTriggeringPrincipal, int32_t aCORSMode,
-                          nsIReferrerInfo* aReferrerInfo) {
+nsresult imgRequest::Init(
+    nsIURI* aURI, nsIURI* aFinalURI, bool aHadInsecureRedirect,
+    nsIRequest* aRequest, nsIChannel* aChannel, imgCacheEntry* aCacheEntry,
+    mozilla::dom::Document* aLoadingDocument,
+    nsIPrincipal* aTriggeringPrincipal, mozilla::CORSMode aCORSMode,
+    nsIReferrerInfo* aReferrerInfo) MOZ_NO_THREAD_SAFETY_ANALYSIS {
   MOZ_ASSERT(NS_IsMainThread(), "Cannot use nsIURI off main thread!");
+  // Init() can only be called once, and that's before it can be used off
+  // mainthread
 
   LOG_FUNC(gImgLog, "imgRequest::Init");
 
@@ -174,6 +176,11 @@ bool imgRequest::CanReuseWithoutValidation(dom::Document* aDoc) const {
 
 void imgRequest::ClearLoader() { mLoader = nullptr; }
 
+already_AddRefed<nsIPrincipal> imgRequest::GetTriggeringPrincipal() const {
+  nsCOMPtr<nsIPrincipal> principal = mTriggeringPrincipal;
+  return principal.forget();
+}
+
 already_AddRefed<ProgressTracker> imgRequest::GetProgressTracker() const {
   MutexAutoLock lock(mMutex);
 
@@ -275,6 +282,16 @@ nsresult imgRequest::RemoveProxy(imgRequestProxy* proxy, nsresult aStatus) {
   return NS_OK;
 }
 
+uint64_t imgRequest::InnerWindowID() const {
+  MutexAutoLock lock(mMutex);
+  return mInnerWindowId;
+}
+
+void imgRequest::SetInnerWindowID(uint64_t aInnerWindowId) {
+  MutexAutoLock lock(mMutex);
+  mInnerWindowId = aInnerWindowId;
+}
+
 void imgRequest::CancelAndAbort(nsresult aStatus) {
   LOG_SCOPE(gImgLog, "imgRequest::CancelAndAbort");
 
@@ -317,8 +334,7 @@ void imgRequest::Cancel(nsresult aStatus) {
   if (NS_IsMainThread()) {
     ContinueCancel(aStatus);
   } else {
-    RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
-    nsCOMPtr<nsIEventTarget> eventTarget = progressTracker->GetEventTarget();
+    nsCOMPtr<nsIEventTarget> eventTarget = GetMainThreadSerialEventTarget();
     nsCOMPtr<nsIRunnable> ev = new imgRequestMainThreadCancel(this, aStatus);
     eventTarget->Dispatch(ev.forget(), NS_DISPATCH_NORMAL);
   }
@@ -333,7 +349,7 @@ void imgRequest::ContinueCancel(nsresult aStatus) {
   RemoveFromCache();
 
   if (mRequest && !(progressTracker->GetProgress() & FLAG_LAST_PART_COMPLETE)) {
-    mRequest->Cancel(aStatus);
+    mRequest->CancelWithReason(aStatus, "imgRequest::ContinueCancel"_ns);
   }
 }
 
@@ -451,6 +467,30 @@ already_AddRefed<image::Image> imgRequest::GetImage() const {
   return image.forget();
 }
 
+void imgRequest::GetFileName(nsACString& aFileName) {
+  nsAutoString fileName;
+
+  nsCOMPtr<nsISupportsCString> supportscstr;
+  if (NS_SUCCEEDED(mProperties->Get("content-disposition",
+                                    NS_GET_IID(nsISupportsCString),
+                                    getter_AddRefs(supportscstr))) &&
+      supportscstr) {
+    nsAutoCString cdHeader;
+    supportscstr->GetData(cdHeader);
+    NS_GetFilenameFromDisposition(fileName, cdHeader);
+  }
+
+  if (fileName.IsEmpty()) {
+    nsCOMPtr<nsIURL> imgUrl(do_QueryInterface(mURI));
+    if (imgUrl) {
+      imgUrl->GetFileName(aFileName);
+      NS_UnescapeURL(aFileName);
+    }
+  } else {
+    aFileName = NS_ConvertUTF16toUTF8(fileName);
+  }
+}
+
 int32_t imgRequest::Priority() const {
   int32_t priority = nsISupportsPriority::PRIORITY_NORMAL;
   nsCOMPtr<nsISupportsPriority> p = do_QueryInterface(mRequest);
@@ -539,23 +579,32 @@ void imgRequest::UpdateCacheEntrySize() {
 }
 
 void imgRequest::SetCacheValidation(imgCacheEntry* aCacheEntry,
-                                    nsIRequest* aRequest) {
-  /* get the expires info */
-  if (!aCacheEntry || aCacheEntry->GetExpiryTime() != 0) {
+                                    nsIRequest* aRequest,
+                                    bool aForceTouch /* = false */) {
+  if (!aCacheEntry) {
     return;
   }
 
-  auto info = nsContentUtils::GetSubresourceCacheValidationInfo(aRequest);
+  RefPtr<imgRequest> req = aCacheEntry->GetRequest();
+  MOZ_ASSERT(req);
+  RefPtr<nsIURI> uri;
+  req->GetURI(getter_AddRefs(uri));
+  // TODO(emilio): Seems we should be able to assert `uri` is not null, but we
+  // get here in such cases sometimes (like for some redirects, see
+  // docshell/test/chrome/test_bug89419.xhtml).
+  //
+  // We have the original URI in the cache key though, probably we should be
+  // using that instead of relying on Init() getting called.
+  auto info = nsContentUtils::GetSubresourceCacheValidationInfo(aRequest, uri);
 
   // Expiration time defaults to 0. We set the expiration time on our entry if
   // it hasn't been set yet.
   if (!info.mExpirationTime) {
     // If the channel doesn't support caching, then ensure this expires the
     // next time it is used.
-    info.mExpirationTime.emplace(nsContentUtils::SecondsFromPRTime(PR_Now()) -
-                                 1);
+    info.mExpirationTime.emplace(CacheExpirationTime::AlreadyExpired());
   }
-  aCacheEntry->SetExpiryTime(*info.mExpirationTime);
+  aCacheEntry->AccumulateExpiryTime(*info.mExpirationTime, aForceTouch);
   // Cache entries default to not needing to validate. We ensure that
   // multiple calls to this function don't override an earlier decision to
   // validate by making validation a one-way decision.
@@ -564,66 +613,6 @@ void imgRequest::SetCacheValidation(imgCacheEntry* aCacheEntry,
   }
 }
 
-namespace {
-
-already_AddRefed<nsIApplicationCache> GetApplicationCache(
-    nsIRequest* aRequest) {
-  nsresult rv;
-
-  nsCOMPtr<nsIApplicationCacheChannel> appCacheChan =
-      do_QueryInterface(aRequest);
-  if (!appCacheChan) {
-    return nullptr;
-  }
-
-  bool fromAppCache;
-  rv = appCacheChan->GetLoadedFromApplicationCache(&fromAppCache);
-  NS_ENSURE_SUCCESS(rv, nullptr);
-
-  if (!fromAppCache) {
-    return nullptr;
-  }
-
-  nsCOMPtr<nsIApplicationCache> appCache;
-  rv = appCacheChan->GetApplicationCache(getter_AddRefs(appCache));
-  NS_ENSURE_SUCCESS(rv, nullptr);
-
-  return appCache.forget();
-}
-
-}  // namespace
-
-bool imgRequest::CacheChanged(nsIRequest* aNewRequest) {
-  nsCOMPtr<nsIApplicationCache> newAppCache = GetApplicationCache(aNewRequest);
-
-  // Application cache not involved at all or the same app cache involved
-  // in both of the loads (original and new).
-  if (newAppCache == mApplicationCache) {
-    return false;
-  }
-
-  // In a rare case it may happen that two objects still refer
-  // the same application cache version.
-  if (newAppCache && mApplicationCache) {
-    nsresult rv;
-
-    nsAutoCString oldAppCacheClientId, newAppCacheClientId;
-    rv = mApplicationCache->GetClientID(oldAppCacheClientId);
-    NS_ENSURE_SUCCESS(rv, true);
-    rv = newAppCache->GetClientID(newAppCacheClientId);
-    NS_ENSURE_SUCCESS(rv, true);
-
-    if (oldAppCacheClientId == newAppCacheClientId) {
-      return false;
-    }
-  }
-
-  // When we get here, app caches differ or app cache is involved
-  // just in one of the loads what we also consider as a change
-  // in a loading cache.
-  return true;
-}
-
 bool imgRequest::GetMultipart() const {
   MutexAutoLock lock(mMutex);
   return mIsMultiPartChannel;
@@ -651,6 +640,7 @@ imgRequest::OnStartRequest(nsIRequest* aRequest) {
     mIsCrossSiteNoCORSRequest = loadInfo->GetTainting() == LoadTainting::Opaque;
   }
 
+  UpdateShouldReportRenderTimeForLCP();
   // Figure out if we're multipart.
   nsCOMPtr<nsIMultiPartChannel> multiPartChannel = do_QueryInterface(aRequest);
   {
@@ -699,9 +689,7 @@ imgRequest::OnStartRequest(nsIRequest* aRequest) {
     }
   }
 
-  SetCacheValidation(mCacheEntry, aRequest);
-
-  mApplicationCache = GetApplicationCache(aRequest);
+  SetCacheValidation(mCacheEntry, aRequest, /* aForceTouch = */ true);
 
   // Shouldn't we be dead already if this gets hit?
   // Probably multipart/x-mixed-replace...
@@ -722,16 +710,17 @@ imgRequest::OnStartRequest(nsIRequest* aRequest) {
     nsAutoCString mimeType;
     nsresult rv = channel->GetContentType(mimeType);
     if (NS_SUCCEEDED(rv) && !mimeType.EqualsLiteral(IMAGE_SVG_XML)) {
+      mOffMainThreadData = true;
       // Retarget OnDataAvailable to the DecodePool's IO thread.
-      nsCOMPtr<nsIEventTarget> target =
+      nsCOMPtr<nsISerialEventTarget> target =
           DecodePool::Singleton()->GetIOEventTarget();
       rv = retargetable->RetargetDeliveryTo(target);
+      MOZ_LOG(gImgLog, LogLevel::Warning,
+              ("[this=%p] imgRequest::OnStartRequest -- "
+               "RetargetDeliveryTo rv %" PRIu32 "=%s\n",
+               this, static_cast<uint32_t>(rv),
+               NS_SUCCEEDED(rv) ? "succeeded" : "failed"));
     }
-    MOZ_LOG(gImgLog, LogLevel::Warning,
-            ("[this=%p] imgRequest::OnStartRequest -- "
-             "RetargetDeliveryTo rv %" PRIu32 "=%s\n",
-             this, static_cast<uint32_t>(rv),
-             NS_SUCCEEDED(rv) ? "succeeded" : "failed"));
   }
 
   return NS_OK;
@@ -742,14 +731,22 @@ imgRequest::OnStopRequest(nsIRequest* aRequest, nsresult status) {
   LOG_FUNC(gImgLog, "imgRequest::OnStopRequest");
   MOZ_ASSERT(NS_IsMainThread(), "Can't send notifications off-main-thread");
 
-  RefPtr<Image> image = GetImage();
-
   RefPtr<imgRequest> strongThis = this;
 
-  if (mIsMultiPartChannel && mNewPartPending) {
+  bool isMultipart = false;
+  bool newPartPending = false;
+  {
+    MutexAutoLock lock(mMutex);
+    isMultipart = mIsMultiPartChannel;
+    newPartPending = mNewPartPending;
+  }
+  if (isMultipart && newPartPending) {
     OnDataAvailable(aRequest, nullptr, 0, 0);
   }
 
+  // Get this after OnDataAvailable because that might have created the image.
+  RefPtr<Image> image = GetImage();
+
   // XXXldb What if this is a non-last part of a multipart request?
   // xxx before we release our reference to mRequest, lets
   // save the last status that we saw so that the
@@ -781,8 +778,7 @@ imgRequest::OnStopRequest(nsIRequest* aRequest, nsresult status) {
   // trigger a failure, since the image might be waiting for more non-optional
   // data and this is the point where we break the news that it's not coming.
   if (image) {
-    nsresult rv =
-        image->OnImageDataComplete(aRequest, nullptr, status, lastPart);
+    nsresult rv = image->OnImageDataComplete(aRequest, status, lastPart);
 
     // If we got an error in the OnImageDataComplete() call, we don't want to
     // proceed as if nothing bad happened. However, we also want to give
@@ -839,11 +835,12 @@ static nsresult sniff_mimetype_callback(nsIInputStream* in, void* closure,
 /** nsThreadRetargetableStreamListener methods **/
 NS_IMETHODIMP
 imgRequest::CheckListenerChain() {
-  // TODO Might need more checking here.
-  NS_ASSERTION(NS_IsMainThread(), "Should be on the main thread!");
-  return NS_OK;
+  return mOffMainThreadData ? NS_OK : NS_ERROR_NO_INTERFACE;
 }
 
+NS_IMETHODIMP
+imgRequest::OnDataFinished(nsresult) { return NS_OK; }
+
 /** nsIStreamListener methods **/
 
 struct NewPartResult final {
@@ -866,7 +863,7 @@ static NewPartResult PrepareForNewPart(nsIRequest* aRequest,
                                        nsIURI* aURI, bool aIsMultipart,
                                        image::Image* aExistingImage,
                                        ProgressTracker* aProgressTracker,
-                                       uint32_t aInnerWindowId) {
+                                       uint64_t aInnerWindowId) {
   NewPartResult result(aExistingImage);
 
   if (aInStr) {
@@ -1005,6 +1002,7 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsIInputStream* aInStr,
   RefPtr<ProgressTracker> progressTracker;
   bool isMultipart = false;
   bool newPartPending = false;
+  uint64_t innerWindowId = 0;
 
   // Retrieve and update our state.
   {
@@ -1014,6 +1012,7 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsIInputStream* aInStr,
     isMultipart = mIsMultiPartChannel;
     newPartPending = mNewPartPending;
     mNewPartPending = false;
+    innerWindowId = mInnerWindowId;
   }
 
   // If this is a new part, we need to sniff its content type and create an
@@ -1021,31 +1020,28 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsIInputStream* aInStr,
   if (newPartPending) {
     NewPartResult result =
         PrepareForNewPart(aRequest, aInStr, aCount, mURI, isMultipart, image,
-                          progressTracker, mInnerWindowId);
+                          progressTracker, innerWindowId);
     bool succeeded = result.mSucceeded;
 
     if (result.mImage) {
       image = result.mImage;
-      nsCOMPtr<nsIEventTarget> eventTarget;
 
       // Update our state to reflect this new part.
       {
         MutexAutoLock lock(mMutex);
         mImage = image;
 
-        // We only get an event target if we are not on the main thread, because
-        // we have to dispatch in that case. If we are on the main thread, but
-        // on a different scheduler group than ProgressTracker would give us,
-        // that is okay because nothing in imagelib requires that, just our
-        // listeners (which have their own checks).
-        if (!NS_IsMainThread()) {
-          eventTarget = mProgressTracker->GetEventTarget();
-          MOZ_ASSERT(eventTarget);
-        }
-
         mProgressTracker = nullptr;
       }
 
+      // We only get an event target if we are not on the main thread, because
+      // we have to dispatch in that case.
+      nsCOMPtr<nsIEventTarget> eventTarget;
+      if (!NS_IsMainThread()) {
+        eventTarget = GetMainThreadSerialEventTarget();
+        MOZ_ASSERT(eventTarget);
+      }
+
       // Some property objects are not threadsafe, and we need to send
       // OnImageAvailable on the main thread, so finish on the main thread.
       if (!eventTarget) {
@@ -1054,7 +1050,7 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsIInputStream* aInStr,
       } else {
         nsCOMPtr<nsIRunnable> runnable =
             new FinishPreparingForNewPartRunnable(this, std::move(result));
-        eventTarget->Dispatch(CreateMediumHighRunnable(runnable.forget()),
+        eventTarget->Dispatch(CreateRenderBlockingRunnable(runnable.forget()),
                               NS_DISPATCH_NORMAL);
       }
     }
@@ -1069,7 +1065,7 @@ imgRequest::OnDataAvailable(nsIRequest* aRequest, nsIInputStream* aInStr,
   // Notify the image that it has new data.
   if (aInStr) {
     nsresult rv =
-        image->OnImageDataAvailable(aRequest, nullptr, aInStr, aOffset, aCount);
+        image->OnImageDataAvailable(aRequest, aInStr, aOffset, aCount);
 
     if (NS_FAILED(rv)) {
       MOZ_LOG(gImgLog, LogLevel::Warning,
@@ -1223,17 +1219,8 @@ imgRequest::OnRedirectVerifyCallback(nsresult result) {
 
   // Make sure we have a protocol that returns data rather than opens an
   // external application, e.g. 'mailto:'.
-  bool doesNotReturnData = false;
-  nsresult rv = NS_URIChainHasFlags(
-      mFinalURI, nsIProtocolHandler::URI_DOES_NOT_RETURN_DATA,
-      &doesNotReturnData);
-
-  if (NS_SUCCEEDED(rv) && doesNotReturnData) {
-    rv = NS_ERROR_ABORT;
-  }
-
-  if (NS_FAILED(rv)) {
-    mRedirectCallback->OnRedirectVerifyCallback(rv);
+  if (nsContentUtils::IsExternalProtocol(mFinalURI)) {
+    mRedirectCallback->OnRedirectVerifyCallback(NS_ERROR_ABORT);
     mRedirectCallback = nullptr;
     return NS_OK;
   }
@@ -1242,3 +1229,13 @@ imgRequest::OnRedirectVerifyCallback(nsresult result) {
   mRedirectCallback = nullptr;
   return NS_OK;
 }
+
+void imgRequest::UpdateShouldReportRenderTimeForLCP() {
+  if (mTimedChannel) {
+    bool allRedirectPassTAO = false;
+    mTimedChannel->GetAllRedirectsPassTimingAllowCheck(&allRedirectPassTAO);
+    mShouldReportRenderTimeForLCP =
+        mTimedChannel->TimingAllowCheck(mTriggeringPrincipal) &&
+        allRedirectPassTAO;
+  }
+}
