@@ -181,18 +181,17 @@ struct InefficientNonFlatteningStringHashPolicy {
 namespace JS {
 
 struct ClassInfo {
-#define FOR_EACH_SIZE(MACRO)                                     \
-  MACRO(Objects, GCHeapUsed, objectsGCHeap)                      \
-  MACRO(Objects, MallocHeap, objectsMallocHeapSlots)             \
-  MACRO(Objects, MallocHeap, objectsMallocHeapElementsNormal)    \
-  MACRO(Objects, MallocHeap, objectsMallocHeapElementsAsmJS)     \
-  MACRO(Objects, MallocHeap, objectsMallocHeapGlobalData)        \
-  MACRO(Objects, MallocHeap, objectsMallocHeapGlobalVarNamesSet) \
-  MACRO(Objects, MallocHeap, objectsMallocHeapMisc)              \
-  MACRO(Objects, NonHeap, objectsNonHeapElementsNormal)          \
-  MACRO(Objects, NonHeap, objectsNonHeapElementsShared)          \
-  MACRO(Objects, NonHeap, objectsNonHeapElementsWasm)            \
-  MACRO(Objects, NonHeap, objectsNonHeapElementsWasmShared)      \
+#define FOR_EACH_SIZE(MACRO)                                  \
+  MACRO(Objects, GCHeapUsed, objectsGCHeap)                   \
+  MACRO(Objects, MallocHeap, objectsMallocHeapSlots)          \
+  MACRO(Objects, MallocHeap, objectsMallocHeapElementsNormal) \
+  MACRO(Objects, MallocHeap, objectsMallocHeapElementsAsmJS)  \
+  MACRO(Objects, MallocHeap, objectsMallocHeapGlobalData)     \
+  MACRO(Objects, MallocHeap, objectsMallocHeapMisc)           \
+  MACRO(Objects, NonHeap, objectsNonHeapElementsNormal)       \
+  MACRO(Objects, NonHeap, objectsNonHeapElementsShared)       \
+  MACRO(Objects, NonHeap, objectsNonHeapElementsWasm)         \
+  MACRO(Objects, NonHeap, objectsNonHeapElementsWasmShared)   \
   MACRO(Objects, NonHeap, objectsNonHeapCodeWasm)
 
   ClassInfo() = default;
@@ -462,7 +461,6 @@ struct NotableScriptSourceInfo : public ScriptSourceInfo {
 struct HelperThreadStats {
 #define FOR_EACH_SIZE(MACRO)           \
   MACRO(_, MallocHeap, stateData)      \
-  MACRO(_, MallocHeap, parseTask)      \
   MACRO(_, MallocHeap, ionCompileTask) \
   MACRO(_, MallocHeap, wasmCompile)    \
   MACRO(_, MallocHeap, contexts)
@@ -555,7 +553,8 @@ struct UnusedGCThingSizes {
   MACRO(Other, GCHeapUnused, bigInt)       \
   MACRO(Other, GCHeapUnused, jitcode)      \
   MACRO(Other, GCHeapUnused, scope)        \
-  MACRO(Other, GCHeapUnused, regExpShared)
+  MACRO(Other, GCHeapUnused, regExpShared) \
+  MACRO(Other, GCHeapUnused, smallBuffer)
 
   UnusedGCThingSizes() = default;
   UnusedGCThingSizes(UnusedGCThingSizes&& other) = default;
@@ -626,6 +625,36 @@ struct UnusedGCThingSizes {
 #undef FOR_EACH_SIZE
 };
 
+struct GCBufferStats {
+#define FOR_EACH_SIZE(MACRO)          \
+  MACRO(Other, MallocHeap, usedBytes) \
+  MACRO(Other, MallocHeap, freeBytes) \
+  MACRO(Other, MallocHeap, adminBytes)
+
+  GCBufferStats() = default;
+  GCBufferStats(GCBufferStats&& other) = default;
+
+  void addSizes(const GCBufferStats& other) { FOR_EACH_SIZE(ADD_OTHER_SIZE); }
+
+  size_t totalSize() const {
+    size_t n = 0;
+    FOR_EACH_SIZE(ADD_SIZE_TO_N);
+    return n;
+  }
+
+  void addToTabSizes(JS::TabSizes* sizes) const {
+    FOR_EACH_SIZE(ADD_TO_TAB_SIZES);
+  }
+
+  void addToServoSizes(JS::ServoSizes* sizes) const {
+    FOR_EACH_SIZE(ADD_TO_SERVO_SIZES);
+  }
+
+  FOR_EACH_SIZE(DECL_SIZE_ZERO);
+
+#undef FOR_EACH_SIZE
+};
+
 struct ZoneStats {
 #define FOR_EACH_SIZE(MACRO)                               \
   MACRO(Other, GCHeapUsed, symbolsGCHeap)                  \
@@ -643,9 +672,11 @@ struct ZoneStats {
   MACRO(Other, MallocHeap, scopesMallocHeap)               \
   MACRO(Other, GCHeapUsed, regExpSharedsGCHeap)            \
   MACRO(Other, MallocHeap, regExpSharedsMallocHeap)        \
+  MACRO(Other, MallocHeap, zoneObject)                     \
   MACRO(Other, MallocHeap, regexpZone)                     \
   MACRO(Other, MallocHeap, jitZone)                        \
-  MACRO(Other, MallocHeap, baselineStubsOptimized)         \
+  MACRO(Other, MallocHeap, cacheIRStubs)                   \
+  MACRO(Other, MallocHeap, objectFuses)                    \
   MACRO(Other, MallocHeap, uniqueIdMap)                    \
   MACRO(Other, MallocHeap, initialPropMapTable)            \
   MACRO(Other, MallocHeap, shapeTables)                    \
@@ -662,6 +693,7 @@ struct ZoneStats {
   void addSizes(const ZoneStats& other) {
     MOZ_ASSERT(isTotals);
     FOR_EACH_SIZE(ADD_OTHER_SIZE);
+    gcBuffers.addSizes(other.gcBuffers);
     unusedGCThings.addSizes(other.unusedGCThings);
     stringInfo.add(other.stringInfo);
     shapeInfo.add(other.shapeInfo);
@@ -679,6 +711,7 @@ struct ZoneStats {
   void addToTabSizes(JS::TabSizes* sizes) const {
     MOZ_ASSERT(isTotals);
     FOR_EACH_SIZE(ADD_TO_TAB_SIZES);
+    gcBuffers.addToTabSizes(sizes);
     unusedGCThings.addToTabSizes(sizes);
     stringInfo.addToTabSizes(sizes);
     shapeInfo.addToTabSizes(sizes);
@@ -687,6 +720,7 @@ struct ZoneStats {
   void addToServoSizes(JS::ServoSizes* sizes) const {
     MOZ_ASSERT(isTotals);
     FOR_EACH_SIZE(ADD_TO_SERVO_SIZES);
+    gcBuffers.addToServoSizes(sizes);
     unusedGCThings.addToServoSizes(sizes);
     stringInfo.addToServoSizes(sizes);
     shapeInfo.addToServoSizes(sizes);
@@ -695,6 +729,8 @@ struct ZoneStats {
 
   FOR_EACH_SIZE(DECL_SIZE_ZERO);
 
+  GCBufferStats gcBuffers;
+
   // These string measurements are initially for all strings.  At the end,
   // if the measurement granularity is FineGrained, we subtract the
   // measurements of the notable script sources and move them into
@@ -731,7 +767,7 @@ struct RealmStats {
   MACRO(Other, GCHeapUsed, scriptsGCHeap)         \
   MACRO(Other, MallocHeap, scriptsMallocHeapData) \
   MACRO(Other, MallocHeap, baselineData)          \
-  MACRO(Other, MallocHeap, baselineStubsFallback) \
+  MACRO(Other, MallocHeap, allocSites)            \
   MACRO(Other, MallocHeap, ionData)               \
   MACRO(Other, MallocHeap, jitScripts)            \
   MACRO(Other, MallocHeap, realmObject)           \
@@ -894,15 +930,11 @@ extern JS_PUBLIC_API size_t UserRealmCount(JSContext* cx);
 
 extern JS_PUBLIC_API size_t PeakSizeOfTemporary(const JSContext* cx);
 
-extern JS_PUBLIC_API bool AddSizeOfTab(JSContext* cx, JS::HandleObject obj,
+extern JS_PUBLIC_API bool AddSizeOfTab(JSContext* cx, JS::Zone* zone,
                                        mozilla::MallocSizeOf mallocSizeOf,
                                        ObjectPrivateVisitor* opv,
-                                       TabSizes* sizes);
-
-extern JS_PUBLIC_API bool AddServoSizeOf(JSContext* cx,
-                                         mozilla::MallocSizeOf mallocSizeOf,
-                                         ObjectPrivateVisitor* opv,
-                                         ServoSizes* sizes);
+                                       TabSizes* sizes,
+                                       const JS::AutoRequireNoGC& nogc);
 
 }  // namespace JS
 
