@@ -20,6 +20,7 @@
 
 #include "gc/GC.h"
 #include "js/AllocPolicy.h"
+#include "js/ArrayBuffer.h"
 #include "js/CharacterEncoding.h"
 #include "js/Conversions.h"
 #include "js/Equality.h"      // JS::SameValue
@@ -71,15 +72,78 @@ inline JSAPITestString operator+(const JSAPITestString& a,
   return result;
 }
 
+// A singly linked list of tests which doesn't require runtime intialization.
+//
+// We rely on the runtime initialization of global variables containing test
+// instances to add themselves to these lists, which depends on them already
+// being in a valid state.
+template <typename T>
+class JSAPITestList {
+  T* first = nullptr;
+  T* last = nullptr;
+
+ public:
+  T* getFirst() const { return first; }
+
+  void pushBack(T* element) {
+    MOZ_ASSERT(!element->next);
+    MOZ_ASSERT(bool(first) == bool(last));
+
+    if (!first) {
+      first = element;
+      last = element;
+      return;
+    }
+
+    last->next = element;
+    last = element;
+  }
+};
+
+class JSAPIRuntimeTest;
+
 class JSAPITest {
  public:
-  static JSAPITest* list;
-  JSAPITest* next;
+  bool knownFail;
+  JSAPITestString msgs;
+
+  JSAPITest() : knownFail(false) {}
+
+  virtual ~JSAPITest() {}
+
+  virtual const char* name() = 0;
+
+  virtual void maybeAppendException(JSAPITestString& message) {}
+
+  bool fail(const JSAPITestString& msg = JSAPITestString(),
+            const char* filename = "-", int lineno = 0) {
+    char location[256];
+    SprintfLiteral(location, "%s:%d:", filename, lineno);
+
+    JSAPITestString message(location);
+    message += msg;
+
+    maybeAppendException(message);
+
+    fprintf(stderr, "%.*s\n", int(message.length()), message.begin());
+
+    if (msgs.length() != 0) {
+      msgs += " | ";
+    }
+    msgs += message;
+    return false;
+  }
+
+  JSAPITestString messages() const { return msgs; }
+};
+
+class JSAPIRuntimeTest : public JSAPITest {
+ public:
+  static JSAPITestList<JSAPIRuntimeTest> list;
+  JSAPIRuntimeTest* next = nullptr;
 
   JSContext* cx;
   JS::PersistentRootedObject global;
-  bool knownFail;
-  JSAPITestString msgs;
 
   // Whether this test is willing to skip its init() and reuse a global (and
   // JSContext etc.) from a previous test that also has reuseGlobal=true. It
@@ -87,12 +151,11 @@ class JSAPITest {
   // another reuseGlobal test.
   bool reuseGlobal;
 
-  JSAPITest() : cx(nullptr), knownFail(false), reuseGlobal(false) {
-    next = list;
-    list = this;
+  JSAPIRuntimeTest() : JSAPITest(), cx(nullptr), reuseGlobal(false) {
+    list.pushBack(this);
   }
 
-  virtual ~JSAPITest() {
+  virtual ~JSAPIRuntimeTest() {
     MOZ_RELEASE_ASSERT(!cx);
     MOZ_RELEASE_ASSERT(!global);
   }
@@ -112,7 +175,6 @@ class JSAPITest {
   virtual bool init() { return true; }
   virtual void uninit();
 
-  virtual const char* name() = 0;
   virtual bool run(JS::HandleObject global) = 0;
 
 #define EXEC(s)                                     \
@@ -134,8 +196,9 @@ class JSAPITest {
                 JS::MutableHandleValue vp);
 
   JSAPITestString jsvalToSource(JS::HandleValue v) {
-    if (JSString* str = JS_ValueToSource(cx, v)) {
-      if (JS::UniqueChars bytes = JS_EncodeStringToLatin1(cx, str)) {
+    JS::Rooted<JSString*> str(cx, JS_ValueToSource(cx, v));
+    if (str) {
+      if (JS::UniqueChars bytes = JS_EncodeStringToUTF8(cx, str)) {
         return JSAPITestString(bytes.get());
       }
     }
@@ -208,6 +271,9 @@ class JSAPITest {
     if (flags.unicode()) {
       str += "u";
     }
+    if (flags.unicodeSets()) {
+      str += "v";
+    }
     if (flags.sticky()) {
       str += "y";
     }
@@ -286,14 +352,7 @@ class JSAPITest {
                   __LINE__);                                         \
   } while (false)
 
-  bool fail(const JSAPITestString& msg = JSAPITestString(),
-            const char* filename = "-", int lineno = 0) {
-    char location[256];
-    SprintfLiteral(location, "%s:%d:", filename, lineno);
-
-    JSAPITestString message(location);
-    message += msg;
-
+  void maybeAppendException(JSAPITestString& message) override {
     if (JS_IsExceptionPending(cx)) {
       message += " -- ";
 
@@ -301,28 +360,21 @@ class JSAPITest {
       JS::RootedValue v(cx);
       JS_GetPendingException(cx, &v);
       JS_ClearPendingException(cx);
-      JSString* s = JS::ToString(cx, v);
+      JS::Rooted<JSString*> s(cx, JS::ToString(cx, v));
       if (s) {
         if (JS::UniqueChars bytes = JS_EncodeStringToLatin1(cx, s)) {
           message += bytes.get();
         }
       }
     }
-
-    fprintf(stderr, "%.*s\n", int(message.length()), message.begin());
-
-    if (msgs.length() != 0) {
-      msgs += " | ";
-    }
-    msgs += message;
-    return false;
   }
 
-  JSAPITestString messages() const { return msgs; }
-
   static const JSClass* basicGlobalClass() {
-    static const JSClass c = {"global", JSCLASS_GLOBAL_FLAGS,
-                              &JS::DefaultGlobalClassOps};
+    static const JSClass c = {
+        "global",
+        JSCLASS_GLOBAL_FLAGS,
+        &JS::DefaultGlobalClassOps,
+    };
     return &c;
   }
 
@@ -330,12 +382,13 @@ class JSAPITest {
   static bool print(JSContext* cx, unsigned argc, JS::Value* vp) {
     JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
 
+    JS::Rooted<JSString*> str(cx);
     for (unsigned i = 0; i < args.length(); i++) {
-      JSString* str = JS::ToString(cx, args[i]);
+      str = JS::ToString(cx, args[i]);
       if (!str) {
         return false;
       }
-      JS::UniqueChars bytes = JS_EncodeStringToLatin1(cx, str);
+      JS::UniqueChars bytes = JS_EncodeStringToUTF8(cx, str);
       if (!bytes) {
         return false;
       }
@@ -363,7 +416,7 @@ class JSAPITest {
     MOZ_RELEASE_ASSERT(report->isWarning());
 
     fprintf(stderr, "%s:%u:%s\n",
-            report->filename ? report->filename : "<no filename>",
+            report->filename ? report->filename.c_str() : "<no filename>",
             (unsigned int)report->lineno, report->message().c_str());
   }
 
@@ -372,8 +425,23 @@ class JSAPITest {
   virtual JSObject* createGlobal(JSPrincipals* principals = nullptr);
 };
 
+class JSAPIFrontendTest : public JSAPITest {
+ public:
+  static JSAPITestList<JSAPIFrontendTest> list;
+  JSAPIFrontendTest* next = nullptr;
+
+  JSAPIFrontendTest() : JSAPITest() { list.pushBack(this); }
+
+  virtual ~JSAPIFrontendTest() {}
+
+  virtual bool init() { return true; }
+  virtual void uninit() {}
+
+  virtual bool run() = 0;
+};
+
 #define BEGIN_TEST_WITH_ATTRIBUTES_AND_EXTRA(testname, attrs, extra) \
-  class cls_##testname : public JSAPITest {                          \
+  class cls_##testname : public JSAPIRuntimeTest {                   \
    public:                                                           \
     virtual const char* name() override { return #testname; }        \
     extra virtual bool run(JS::HandleObject global) override attrs
@@ -383,19 +451,31 @@ class JSAPITest {
 
 #define BEGIN_TEST(testname) BEGIN_TEST_WITH_ATTRIBUTES(testname, )
 
+#define BEGIN_FRONTEND_TEST_WITH_ATTRIBUTES_AND_EXTRA(testname, attrs, extra) \
+  class cls_##testname : public JSAPIFrontendTest {                           \
+   public:                                                                    \
+    virtual const char* name() override { return #testname; }                 \
+    extra virtual bool run() override attrs
+
+#define BEGIN_FRONTEND_TEST_WITH_ATTRIBUTES(testname, attrs) \
+  BEGIN_FRONTEND_TEST_WITH_ATTRIBUTES_AND_EXTRA(testname, attrs, )
+
+#define BEGIN_FRONTEND_TEST(testname) \
+  BEGIN_FRONTEND_TEST_WITH_ATTRIBUTES(testname, )
+
 #define BEGIN_REUSABLE_TEST(testname)   \
   BEGIN_TEST_WITH_ATTRIBUTES_AND_EXTRA( \
-      testname, , cls_##testname()      \
-      : JSAPITest() { reuseGlobal = true; })
+      testname, ,                       \
+      cls_##testname() : JSAPIRuntimeTest() { reuseGlobal = true; })
 
 #define END_TEST(testname) \
   }                        \
   ;                        \
-  static cls_##testname cls_##testname##_instance;
+  MOZ_RUNINIT static cls_##testname cls_##testname##_instance;
 
 /*
- * A "fixture" is a subclass of JSAPITest that holds common definitions for a
- * set of tests. Each test that wants to use the fixture should use
+ * A "fixture" is a subclass of JSAPIRuntimeTest that holds common definitions
+ * for a set of tests. Each test that wants to use the fixture should use
  * BEGIN_FIXTURE_TEST and END_FIXTURE_TEST, just as one would use BEGIN_TEST and
  * END_TEST, but include the fixture class as the first argument. The fixture
  * class's declarations are then in scope for the test bodies.
@@ -410,7 +490,7 @@ class JSAPITest {
 #define END_FIXTURE_TEST(fixture, testname) \
   }                                         \
   ;                                         \
-  static cls_##testname cls_##testname##_instance;
+  MOZ_RUNINIT static cls_##testname cls_##testname##_instance;
 
 /*
  * A class for creating and managing one temporary file.
@@ -490,6 +570,7 @@ class TestJSPrincipals : public JSPrincipals {
 class ExternalData {
   char* contents_;
   size_t len_;
+  bool uniquePointerCreated_ = false;
 
  public:
   explicit ExternalData(const char* str)
@@ -506,6 +587,13 @@ class ExternalData {
     contents_ = nullptr;
   }
 
+  mozilla::UniquePtr<void, JS::BufferContentsDeleter> pointer() {
+    MOZ_ASSERT(!uniquePointerCreated_,
+               "Not allowed to create multiple unique pointers to contents");
+    uniquePointerCreated_ = true;
+    return {contents_, {ExternalData::freeCallback, this}};
+  }
+
   static void freeCallback(void* contents, void* userData) {
     auto self = static_cast<ExternalData*>(userData);
     MOZ_ASSERT(self->contents() == contents);
@@ -540,22 +628,22 @@ class AutoLeaveZeal {
  public:
   explicit AutoLeaveZeal(JSContext* cx) : cx_(cx), zealBits_(0), frequency_(0) {
     uint32_t dummy;
-    JS_GetGCZealBits(cx_, &zealBits_, &frequency_, &dummy);
-    JS_SetGCZeal(cx_, 0, 0);
+    JS::GetGCZealBits(cx_, &zealBits_, &frequency_, &dummy);
+    JS::SetGCZeal(cx_, 0, 0);
     JS::PrepareForFullGC(cx_);
     JS::NonIncrementalGC(cx_, JS::GCOptions::Normal, JS::GCReason::DEBUG_GC);
   }
   ~AutoLeaveZeal() {
-    JS_SetGCZeal(cx_, 0, 0);
+    JS::SetGCZeal(cx_, 0, 0);
     for (size_t i = 0; i < sizeof(zealBits_) * 8; i++) {
       if (zealBits_ & (1 << i)) {
-        JS_SetGCZeal(cx_, i, frequency_);
+        JS::SetGCZeal(cx_, i, frequency_);
       }
     }
 
 #  ifdef DEBUG
     uint32_t zealBitsAfter, frequencyAfter, dummy;
-    JS_GetGCZealBits(cx_, &zealBitsAfter, &frequencyAfter, &dummy);
+    JS::GetGCZealBits(cx_, &zealBitsAfter, &frequencyAfter, &dummy);
     MOZ_ASSERT(zealBitsAfter == zealBits_);
     MOZ_ASSERT(frequencyAfter == frequency_);
 #  endif
