@@ -7,6 +7,7 @@
 #include "shell/ModuleLoader.h"
 
 #include "mozilla/DebugOnly.h"
+#include "mozilla/ScopeExit.h"
 #include "mozilla/TextUtils.h"
 
 #include "jsapi.h"
@@ -27,6 +28,8 @@
 #include "vm/JSContext.h"
 #include "vm/StringType.h"
 
+#include "vm/NativeObject-inl.h"
+
 using namespace js;
 using namespace js::shell;
 
@@ -59,25 +62,20 @@ bool ModuleLoader::init(JSContext* cx, HandleString loadPath) {
   }
 
   JSRuntime* rt = cx->runtime();
-  JS::SetModuleResolveHook(rt, ModuleLoader::ResolveImportedModule);
+  JS::SetModuleLoadHook(rt, ModuleLoader::LoadImportedModule);
   JS::SetModuleMetadataHook(rt, ModuleLoader::GetImportMetaProperties);
-  JS::SetModuleDynamicImportHook(rt, ModuleLoader::ImportModuleDynamically);
-
-  JS::ImportAssertionVector assertions;
-  MOZ_ALWAYS_TRUE(assertions.reserve(1));
-  assertions.infallibleAppend(JS::ImportAssertion::Type);
-  JS::SetSupportedImportAssertions(rt, assertions);
-
   return true;
 }
 
 // static
-JSObject* ModuleLoader::ResolveImportedModule(
-    JSContext* cx, JS::HandleValue referencingPrivate,
-    JS::HandleObject moduleRequest) {
+bool ModuleLoader::LoadImportedModule(JSContext* cx,
+                                      JS::Handle<JSScript*> referrer,
+                                      JS::Handle<JSObject*> moduleRequest,
+                                      JS::HandleValue hostDefined,
+                                      JS::HandleValue payload) {
   ShellContext* scx = GetShellContext(cx);
-  return scx->moduleLoader->resolveImportedModule(cx, referencingPrivate,
-                                                  moduleRequest);
+  return scx->moduleLoader->loadImportedModule(cx, referrer, moduleRequest,
+                                               payload);
 }
 
 // static
@@ -116,34 +114,9 @@ bool ModuleLoader::ImportMetaResolve(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
-// static
-bool ModuleLoader::ImportModuleDynamically(JSContext* cx,
-                                           JS::HandleValue referencingPrivate,
-                                           JS::HandleObject moduleRequest,
-                                           JS::HandleObject promise) {
-  ShellContext* scx = GetShellContext(cx);
-  return scx->moduleLoader->dynamicImport(cx, referencingPrivate, moduleRequest,
-                                          promise);
-}
-
-// static
-bool ModuleLoader::GetSupportedImportAssertions(
-    JSContext* cx, JS::ImportAssertionVector& values) {
-  MOZ_ASSERT(values.empty());
-
-  if (!values.reserve(1)) {
-    ReportOutOfMemory(cx);
-    return false;
-  }
-
-  values.infallibleAppend(JS::ImportAssertion::Type);
-
-  return true;
-}
-
 bool ModuleLoader::loadRootModule(JSContext* cx, HandleString path) {
   RootedValue rval(cx);
-  if (!loadAndExecute(cx, path, &rval)) {
+  if (!loadAndExecute(cx, path, nullptr, &rval)) {
     return false;
   }
 
@@ -157,8 +130,7 @@ bool ModuleLoader::loadRootModule(JSContext* cx, HandleString path) {
 
 bool ModuleLoader::registerTestModule(JSContext* cx, HandleObject moduleRequest,
                                       Handle<ModuleObject*> module) {
-  Rooted<JSLinearString*> path(
-      cx, resolve(cx, moduleRequest, UndefinedHandleValue));
+  Rooted<JSLinearString*> path(cx, resolve(cx, moduleRequest, nullptr));
   if (!path) {
     return false;
   }
@@ -168,7 +140,10 @@ bool ModuleLoader::registerTestModule(JSContext* cx, HandleObject moduleRequest,
     return false;
   }
 
-  return addModuleToRegistry(cx, path, module);
+  JS::ModuleType moduleType =
+      moduleRequest->as<ModuleRequestObject>().moduleType();
+
+  return addModuleToRegistry(cx, moduleType, path, module);
 }
 
 void ModuleLoader::clearModules(JSContext* cx) {
@@ -177,29 +152,68 @@ void ModuleLoader::clearModules(JSContext* cx) {
 }
 
 bool ModuleLoader::loadAndExecute(JSContext* cx, HandleString path,
+                                  HandleObject moduleRequestArg,
                                   MutableHandleValue rval) {
-  RootedObject module(cx, loadAndParse(cx, path));
+  RootedObject module(cx, loadAndParse(cx, path, moduleRequestArg));
   if (!module) {
     return false;
   }
 
-  if (!JS::ModuleLink(cx, module)) {
+  return loadAndExecute(cx, module, rval);
+}
+
+bool ModuleLoader::loadAndExecute(JSContext* cx, HandleObject module,
+                                  MutableHandleValue rval) {
+  MOZ_ASSERT(module);
+
+  RootedValue hostDefined(cx, ObjectValue(*module));
+  if (!JS::LoadRequestedModules(cx, module, hostDefined, LoadResolved,
+                                LoadRejected)) {
+    return false;
+  }
+
+  if (JS_IsExceptionPending(cx)) {
     return false;
   }
 
   return JS::ModuleEvaluate(cx, module, rval);
 }
 
-JSObject* ModuleLoader::resolveImportedModule(
-    JSContext* cx, JS::HandleValue referencingPrivate,
-    JS::HandleObject moduleRequest) {
-  Rooted<JSLinearString*> path(cx,
-                               resolve(cx, moduleRequest, referencingPrivate));
-  if (!path) {
-    return nullptr;
+/* static */
+bool ModuleLoader::LoadResolved(JSContext* cx, HandleValue hostDefined) {
+  RootedObject module(cx, &hostDefined.toObject());
+  return JS::ModuleLink(cx, module);
+}
+
+/* static */
+bool ModuleLoader::LoadRejected(JSContext* cx, HandleValue hostDefined,
+                                HandleValue error) {
+  JS_SetPendingException(cx, error);
+  return true;
+}
+
+bool ModuleLoader::loadImportedModule(JSContext* cx,
+                                      JS::Handle<JSScript*> referrer,
+                                      JS::Handle<JSObject*> moduleRequest,
+                                      JS::HandleValue payload) {
+  // TODO: Bug 1968904: Update HostLoadImportedModule
+  if (payload.isObject() && payload.toObject().is<PromiseObject>()) {
+    // This is a dynamic import.
+    return dynamicImport(cx, referrer, moduleRequest, payload);
   }
 
-  return loadAndParse(cx, path);
+  Rooted<JSLinearString*> path(cx, resolve(cx, moduleRequest, referrer));
+  if (!path) {
+    return false;
+  }
+
+  RootedObject module(cx, loadAndParse(cx, path, moduleRequest));
+  if (!module) {
+    return false;
+  }
+
+  return JS::FinishLoadingImportedModule(cx, referrer, moduleRequest, payload,
+                                         module, false);
 }
 
 bool ModuleLoader::populateImportMeta(JSContext* cx,
@@ -251,26 +265,38 @@ bool ModuleLoader::importMetaResolve(JSContext* cx,
   return true;
 }
 
-bool ModuleLoader::dynamicImport(JSContext* cx,
-                                 JS::HandleValue referencingPrivate,
+bool ModuleLoader::dynamicImport(JSContext* cx, JS::HandleScript referrer,
                                  JS::HandleObject moduleRequest,
-                                 JS::HandleObject promise) {
+                                 JS::HandleValue payload) {
   // To make this more realistic, use a promise to delay the import and make it
   // happen asynchronously. This method packages up the arguments and creates a
   // resolved promise, which on fullfillment calls doDynamicImport with the
   // original arguments.
 
-  MOZ_ASSERT(promise);
   RootedValue moduleRequestValue(cx, ObjectValue(*moduleRequest));
-  RootedValue promiseValue(cx, ObjectValue(*promise));
   RootedObject closure(cx, JS_NewObjectWithGivenProto(cx, nullptr, nullptr));
+  RootedValue referrerValue(cx);
+  if (referrer) {
+    referrerValue = PrivateGCThingValue(referrer);
+  } else {
+    RootedScript script(cx);
+    const char* filename;
+    uint32_t lineno;
+    uint32_t pcOffset;
+    bool mutedErrors;
+    DescribeScriptedCallerForCompilation(cx, &script, &filename, &lineno,
+                                         &pcOffset, &mutedErrors);
+    MOZ_ASSERT(script);
+    referrerValue = PrivateGCThingValue(script);
+  }
+  MOZ_ASSERT(!referrerValue.isUndefined());
+
   if (!closure ||
-      !JS_DefineProperty(cx, closure, "referencingPrivate", referencingPrivate,
+      !JS_DefineProperty(cx, closure, "referrer", referrerValue,
                          JSPROP_ENUMERATE) ||
       !JS_DefineProperty(cx, closure, "moduleRequest", moduleRequestValue,
                          JSPROP_ENUMERATE) ||
-      !JS_DefineProperty(cx, closure, "promise", promiseValue,
-                         JSPROP_ENUMERATE)) {
+      !JS_DefineProperty(cx, closure, "payload", payload, JSPROP_ENUMERATE)) {
     return false;
   }
 
@@ -301,21 +327,21 @@ bool ModuleLoader::DynamicImportDelayFulfilled(JSContext* cx, unsigned argc,
   CallArgs args = CallArgsFromVp(argc, vp);
   RootedObject closure(cx, &args[0].toObject());
 
-  RootedValue referencingPrivate(cx);
+  RootedValue referrerValue(cx);
   RootedValue moduleRequestValue(cx);
-  RootedValue promiseValue(cx);
-  if (!JS_GetProperty(cx, closure, "referencingPrivate", &referencingPrivate) ||
+  RootedValue payload(cx);
+  if (!JS_GetProperty(cx, closure, "referrer", &referrerValue) ||
       !JS_GetProperty(cx, closure, "moduleRequest", &moduleRequestValue) ||
-      !JS_GetProperty(cx, closure, "promise", &promiseValue)) {
+      !JS_GetProperty(cx, closure, "payload", &payload)) {
     return false;
   }
 
   RootedObject moduleRequest(cx, &moduleRequestValue.toObject());
-  RootedObject promise(cx, &promiseValue.toObject());
+  RootedScript referrer(cx, static_cast<JSScript*>(referrerValue.toGCThing()));
 
   ShellContext* scx = GetShellContext(cx);
-  return scx->moduleLoader->doDynamicImport(cx, referencingPrivate,
-                                            moduleRequest, promise);
+  return scx->moduleLoader->doDynamicImport(cx, referrer, moduleRequest,
+                                            payload);
 }
 
 bool ModuleLoader::DynamicImportDelayRejected(JSContext* cx, unsigned argc,
@@ -323,38 +349,47 @@ bool ModuleLoader::DynamicImportDelayRejected(JSContext* cx, unsigned argc,
   MOZ_CRASH("This promise should never be rejected");
 }
 
-bool ModuleLoader::doDynamicImport(JSContext* cx,
-                                   JS::HandleValue referencingPrivate,
+bool ModuleLoader::doDynamicImport(JSContext* cx, JS::HandleScript referrer,
                                    JS::HandleObject moduleRequest,
-                                   JS::HandleObject promise) {
+                                   JS::HandleValue payload) {
   // Exceptions during dynamic import are handled by calling
-  // FinishDynamicModuleImport with a pending exception on the context.
-  RootedValue rval(cx);
-  bool ok =
-      tryDynamicImport(cx, referencingPrivate, moduleRequest, promise, &rval);
-  JSObject* evaluationObject = ok ? &rval.toObject() : nullptr;
-  RootedObject evaluationPromise(cx, evaluationObject);
-  return JS::FinishDynamicModuleImport(
-      cx, evaluationPromise, referencingPrivate, moduleRequest, promise);
-}
-
-bool ModuleLoader::tryDynamicImport(JSContext* cx,
-                                    JS::HandleValue referencingPrivate,
-                                    JS::HandleObject moduleRequest,
-                                    JS::HandleObject promise,
-                                    JS::MutableHandleValue rval) {
-  Rooted<JSLinearString*> path(cx,
-                               resolve(cx, moduleRequest, referencingPrivate));
+  // FinishLoadingImportedModule with a pending exception on the context.
+  Rooted<JSLinearString*> path(cx, resolve(cx, moduleRequest, referrer));
   if (!path) {
-    return false;
+    return JS::FinishLoadingImportedModuleFailedWithPendingException(cx,
+                                                                     payload);
   }
 
-  return loadAndExecute(cx, path, rval);
+  RootedObject module(cx, loadAndParse(cx, path, moduleRequest));
+  if (!module) {
+    return JS::FinishLoadingImportedModuleFailedWithPendingException(cx,
+                                                                     payload);
+  }
+
+  RootedValue hostDefined(cx, ObjectValue(*module));
+  if (!JS::LoadRequestedModules(cx, module, hostDefined, LoadResolved,
+                                LoadRejected)) {
+    return JS::FinishLoadingImportedModuleFailedWithPendingException(cx,
+                                                                     payload);
+  }
+
+  if (JS_IsExceptionPending(cx)) {
+    return JS::FinishLoadingImportedModuleFailedWithPendingException(cx,
+                                                                     payload);
+  }
+
+  return JS::FinishLoadingImportedModule(cx, nullptr, moduleRequest, payload,
+                                         module, false);
 }
 
 JSLinearString* ModuleLoader::resolve(JSContext* cx,
                                       HandleObject moduleRequestArg,
-                                      HandleValue referencingInfo) {
+                                      HandleScript referrer) {
+  RootedValue referencingInfo(cx);
+  if (referrer) {
+    referencingInfo = GetScriptPrivate(referrer);
+  }
+
   ModuleRequestObject* moduleRequest =
       &moduleRequestArg->as<ModuleRequestObject>();
   if (moduleRequest->specifier()->length() == 0) {
@@ -439,7 +474,8 @@ JSLinearString* ModuleLoader::resolve(JSContext* cx, HandleString specifier,
   return normalizePath(cx, linear);
 }
 
-JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
+JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg,
+                                     JS::HandleObject moduleRequestArg) {
   Rooted<JSLinearString*> path(cx, JS_EnsureLinearString(cx, pathArg));
   if (!path) {
     return nullptr;
@@ -450,8 +486,13 @@ JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
     return nullptr;
   }
 
+  JS::ModuleType moduleType = JS::ModuleType::JavaScript;
+  if (moduleRequestArg) {
+    moduleType = moduleRequestArg->as<ModuleRequestObject>().moduleType();
+  }
+
   RootedObject module(cx);
-  if (!lookupModuleInRegistry(cx, path, &module)) {
+  if (!lookupModuleInRegistry(cx, moduleType, path, &module)) {
     return nullptr;
   }
 
@@ -482,30 +523,46 @@ JSObject* ModuleLoader::loadAndParse(JSContext* cx, HandleString pathArg) {
     return nullptr;
   }
 
-  module = JS::CompileModule(cx, options, srcBuf);
-  if (!module) {
-    return nullptr;
+  switch (moduleType) {
+    case JS::ModuleType::Unknown:
+      JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                                JSMSG_BAD_MODULE_TYPE);
+      return nullptr;
+    case JS::ModuleType::JavaScript: {
+      module = JS::CompileModule(cx, options, srcBuf);
+      if (!module) {
+        return nullptr;
+      }
+
+      RootedObject info(cx, js::CreateScriptPrivate(cx, path));
+      if (!info) {
+        return nullptr;
+      }
+
+      JS::SetModulePrivate(module, ObjectValue(*info));
+    } break;
+    case JS::ModuleType::JSON:
+      module = JS::CompileJsonModule(cx, options, srcBuf);
+      if (!module) {
+        return nullptr;
+      }
+      break;
   }
 
-  RootedObject info(cx, js::CreateScriptPrivate(cx, path));
-  if (!info) {
-    return nullptr;
-  }
-
-  JS::SetModulePrivate(module, ObjectValue(*info));
-
-  if (!addModuleToRegistry(cx, path, module)) {
+  if (!addModuleToRegistry(cx, moduleType, path, module)) {
     return nullptr;
   }
 
   return module;
 }
 
-bool ModuleLoader::lookupModuleInRegistry(JSContext* cx, HandleString path,
+bool ModuleLoader::lookupModuleInRegistry(JSContext* cx,
+                                          JS::ModuleType moduleType,
+                                          HandleString path,
                                           MutableHandleObject moduleOut) {
   moduleOut.set(nullptr);
 
-  RootedObject registry(cx, getOrCreateModuleRegistry(cx));
+  RootedObject registry(cx, getOrCreateModuleRegistry(cx, moduleType));
   if (!registry) {
     return false;
   }
@@ -523,9 +580,9 @@ bool ModuleLoader::lookupModuleInRegistry(JSContext* cx, HandleString path,
   return true;
 }
 
-bool ModuleLoader::addModuleToRegistry(JSContext* cx, HandleString path,
-                                       HandleObject module) {
-  RootedObject registry(cx, getOrCreateModuleRegistry(cx));
+bool ModuleLoader::addModuleToRegistry(JSContext* cx, JS::ModuleType moduleType,
+                                       HandleString path, HandleObject module) {
+  RootedObject registry(cx, getOrCreateModuleRegistry(cx, moduleType));
   if (!registry) {
     return false;
   }
@@ -535,22 +592,48 @@ bool ModuleLoader::addModuleToRegistry(JSContext* cx, HandleString path,
   return JS::MapSet(cx, registry, pathValue, moduleValue);
 }
 
-JSObject* ModuleLoader::getOrCreateModuleRegistry(JSContext* cx) {
+static ArrayObject* GetOrCreateRootRegistry(JSContext* cx) {
   Handle<GlobalObject*> global = cx->global();
   RootedValue value(cx, global->getReservedSlot(GlobalAppSlotModuleRegistry));
   if (!value.isUndefined()) {
-    return &value.toObject();
+    return &value.toObject().as<ArrayObject>();
   }
 
-  JSObject* registry = JS::NewMapObject(cx);
+  uint32_t numberOfModuleTypes = uint32_t(JS::ModuleType::Limit) + 1;
+
+  Rooted<ArrayObject*> registry(
+      cx, NewDenseFullyAllocatedArray(cx, numberOfModuleTypes, TenuredObject));
   if (!registry) {
     return nullptr;
   }
+  registry->ensureDenseInitializedLength(0, numberOfModuleTypes);
+
+  Rooted<JSObject*> innerRegistry(cx);
+  for (size_t i = 0; i < numberOfModuleTypes; ++i) {
+    innerRegistry = JS::NewMapObject(cx);
+    if (!innerRegistry) {
+      return nullptr;
+    }
+    registry->initDenseElement(i, ObjectValue(*innerRegistry));
+  }
 
   global->setReservedSlot(GlobalAppSlotModuleRegistry, ObjectValue(*registry));
+
   return registry;
 }
 
+JSObject* ModuleLoader::getOrCreateModuleRegistry(JSContext* cx,
+                                                  JS::ModuleType moduleType) {
+  Rooted<ArrayObject*> rootRegistry(cx, GetOrCreateRootRegistry(cx));
+  if (!rootRegistry) {
+    return nullptr;
+  }
+
+  uint32_t index = uint32_t(moduleType);
+  MOZ_ASSERT(rootRegistry->containsDenseElement(index));
+  return &rootRegistry->getDenseElement(index).toObject();
+}
+
 bool ModuleLoader::getScriptPath(JSContext* cx, HandleValue privateValue,
                                  MutableHandle<JSLinearString*> pathOut) {
   pathOut.set(nullptr);
