@@ -16,7 +16,7 @@
 #include "js/CallArgs.h"              // JS::CallArgs
 #include "js/CallNonGenericMethod.h"  // CallNonGenericMethod
 #include "js/Class.h"                 // JSClass, JSCLASS_*
-#include "js/ColumnNumber.h"          // JS::ColumnNumberZeroOrigin
+#include "js/ColumnNumber.h"          // JS::ColumnNumberOneOrigin
 #include "js/ErrorReport.h"           // JS_ReportErrorASCII
 #include "js/PropertyAndElement.h"    // JS_GetProperty
 #include "js/PropertySpec.h"  // JSPropertySpec, JS_PSG, JS_PS_END, JSFunctionSpec, JS_FN, JS_FN_END
@@ -102,6 +102,48 @@ bool IdentFilter(JSContext* cx, JS::Handle<JS::Value> from,
   return true;
 }
 
+bool GetModuleStatusName(JSContext* cx, JS::Handle<JS::Value> from,
+                         JS::MutableHandle<JS::Value> to) {
+  if (!from.isInt32()) {
+    return false;
+  }
+
+  const char* statusStr = nullptr;
+  switch (static_cast<ModuleStatus>(from.toInt32())) {
+    case ModuleStatus::New:
+      statusStr = "New";
+      break;
+    case ModuleStatus::Unlinked:
+      statusStr = "Unlinked";
+      break;
+    case ModuleStatus::Linking:
+      statusStr = "Linking";
+      break;
+    case ModuleStatus::Linked:
+      statusStr = "Linked";
+      break;
+    case ModuleStatus::Evaluating:
+      statusStr = "Evaluating";
+      break;
+    case ModuleStatus::EvaluatingAsync:
+      statusStr = "EvaluatingAsync";
+      break;
+    case ModuleStatus::Evaluated:
+      statusStr = "Evaluated";
+      break;
+    default:
+      MOZ_CRASH("Unknown ModuleStatus value");
+  }
+
+  JS::Rooted<JSString*> str(cx, JS_NewStringCopyZ(cx, statusStr));
+  if (!str) {
+    return false;
+  }
+
+  to.setString(str);
+  return true;
+}
+
 template <class T>
 bool SingleFilter(JSContext* cx, JS::Handle<JS::Value> from,
                   JS::MutableHandle<JS::Value> to) {
@@ -206,8 +248,8 @@ static Value Uint32OrUndefinedValue(mozilla::Maybe<uint32_t> x) {
   return Uint32Value(x.value());
 }
 
-static Value ColumnNumberZeroOriginValue(JS::ColumnNumberZeroOrigin x) {
-  uint32_t column = x.zeroOriginValue();
+static Value ColumnNumberOneOriginValue(JS::ColumnNumberOneOrigin x) {
+  uint32_t column = x.oneOriginValue();
   MOZ_ASSERT(column <= INT32_MAX);
   return Int32Value(column);
 }
@@ -228,6 +270,13 @@ template <class T, typename RawGetterT, typename FilterT>
 bool ShellModuleWrapperGetter(JSContext* cx, const JS::CallArgs& args,
                               RawGetterT rawGetter, FilterT filter) {
   JS::Rooted<T*> wrapper(cx, &args.thisv().toObject().as<T>());
+  if constexpr (std::is_same_v<T, ShellModuleObjectWrapper>) {
+    if (!wrapper->get()->hasCyclicModuleFields()) {
+      args.rval().set(UndefinedValue());
+      return true;
+    }
+  }
+
   JS::Rooted<JS::Value> raw(cx, rawGetter(wrapper->get()));
 
   JS::Rooted<JS::Value> filtered(cx);
@@ -285,6 +334,13 @@ template <class T, typename RawGetterT, typename FilterT>
 bool ShellModuleNativeWrapperGetter(JSContext* cx, const JS::CallArgs& args,
                                     RawGetterT rawGetter, FilterT filter) {
   JS::Rooted<T*> wrapper(cx, &args.thisv().toObject().as<T>());
+  if constexpr (std::is_same_v<T, ShellModuleObjectWrapper>) {
+    if (!wrapper->get()->hasCyclicModuleFields()) {
+      args.rval().set(UndefinedValue());
+      return true;
+    }
+  }
+
   JS::Rooted<typename T::Target*> owner(cx, wrapper->get());
 
   JS::Rooted<JS::Value> filtered(cx);
@@ -296,6 +352,25 @@ bool ShellModuleNativeWrapperGetter(JSContext* cx, const JS::CallArgs& args,
   return true;
 }
 
+bool ModuleTypeToString(JSContext* cx, JS::Handle<JSObject*> owner,
+                        JS::ModuleType moduleType,
+                        JS::MutableHandle<JS::Value> to) {
+  switch (moduleType) {
+    case JS::ModuleType::Unknown:
+      to.setString(cx->names().unknown);
+      break;
+    case JS::ModuleType::JavaScript:
+      to.setString(cx->names().js);
+      break;
+    case JS::ModuleType::JSON:
+      to.setString(cx->names().json);
+      break;
+  }
+
+  MOZ_ASSERT(!to.isUndefined());
+  return true;
+}
+
 #define DEFINE_NATIVE_GETTER_FUNCTIONS(CLASS, PROP, FILTER)                    \
   static auto Shell##CLASS##Wrapper_##PROP##Getter_raw(CLASS* obj) {           \
     return obj->PROP();                                                        \
@@ -315,20 +390,27 @@ bool ShellModuleNativeWrapperGetter(JSContext* cx, const JS::CallArgs& args,
 
 DEFINE_GETTER_FUNCTIONS(ModuleRequestObject, specifier, StringOrNullValue,
                         IdentFilter)
-DEFINE_GETTER_FUNCTIONS(ModuleRequestObject, assertions, ObjectOrNullValue,
-                        IdentFilter)
+DEFINE_GETTER_FUNCTIONS(ModuleRequestObject, getFirstUnsupportedAttributeKey,
+                        StringOrNullValue, IdentFilter)
+DEFINE_NATIVE_GETTER_FUNCTIONS(ModuleRequestObject, moduleType,
+                               ModuleTypeToString);
 
 static const JSPropertySpec ShellModuleRequestObjectWrapper_accessors[] = {
     JS_PSG("specifier", ShellModuleRequestObjectWrapper_specifierGetter, 0),
-    JS_PSG("assertions", ShellModuleRequestObjectWrapper_assertionsGetter, 0),
-    JS_PS_END};
+    JS_PSG("moduleType", ShellModuleRequestObjectWrapper_moduleTypeGetter, 0),
+    JS_PSG(
+        "firstUnsupportedAttributeKey",
+        ShellModuleRequestObjectWrapper_getFirstUnsupportedAttributeKeyGetter,
+        0),
+    JS_PS_END,
+};
 
 DEFINE_GETTER_FUNCTIONS(ImportEntry, moduleRequest, ObjectOrNullValue,
                         SingleFilter<ShellModuleRequestObjectWrapper>)
 DEFINE_GETTER_FUNCTIONS(ImportEntry, importName, StringOrNullValue, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ImportEntry, localName, StringValue, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ImportEntry, lineNumber, Uint32Value, IdentFilter)
-DEFINE_GETTER_FUNCTIONS(ImportEntry, columnNumber, ColumnNumberZeroOriginValue,
+DEFINE_GETTER_FUNCTIONS(ImportEntry, columnNumber, ColumnNumberOneOriginValue,
                         IdentFilter)
 
 static const JSPropertySpec ShellImportEntryWrapper_accessors[] = {
@@ -337,7 +419,8 @@ static const JSPropertySpec ShellImportEntryWrapper_accessors[] = {
     JS_PSG("localName", ShellImportEntryWrapper_localNameGetter, 0),
     JS_PSG("lineNumber", ShellImportEntryWrapper_lineNumberGetter, 0),
     JS_PSG("columnNumber", ShellImportEntryWrapper_columnNumberGetter, 0),
-    JS_PS_END};
+    JS_PS_END,
+};
 
 DEFINE_GETTER_FUNCTIONS(ExportEntry, exportName, StringOrNullValue, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ExportEntry, moduleRequest, ObjectOrNullValue,
@@ -345,7 +428,7 @@ DEFINE_GETTER_FUNCTIONS(ExportEntry, moduleRequest, ObjectOrNullValue,
 DEFINE_GETTER_FUNCTIONS(ExportEntry, importName, StringOrNullValue, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ExportEntry, localName, StringOrNullValue, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ExportEntry, lineNumber, Uint32Value, IdentFilter)
-DEFINE_GETTER_FUNCTIONS(ExportEntry, columnNumber, ColumnNumberZeroOriginValue,
+DEFINE_GETTER_FUNCTIONS(ExportEntry, columnNumber, ColumnNumberOneOriginValue,
                         IdentFilter)
 
 static const JSPropertySpec ShellExportEntryWrapper_accessors[] = {
@@ -355,23 +438,25 @@ static const JSPropertySpec ShellExportEntryWrapper_accessors[] = {
     JS_PSG("localName", ShellExportEntryWrapper_localNameGetter, 0),
     JS_PSG("lineNumber", ShellExportEntryWrapper_lineNumberGetter, 0),
     JS_PSG("columnNumber", ShellExportEntryWrapper_columnNumberGetter, 0),
-    JS_PS_END};
+    JS_PS_END,
+};
 
 DEFINE_GETTER_FUNCTIONS(RequestedModule, moduleRequest, ObjectOrNullValue,
                         SingleFilter<ShellModuleRequestObjectWrapper>)
 DEFINE_GETTER_FUNCTIONS(RequestedModule, lineNumber, Uint32Value, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(RequestedModule, columnNumber,
-                        ColumnNumberZeroOriginValue, IdentFilter)
+                        ColumnNumberOneOriginValue, IdentFilter)
 
 static const JSPropertySpec ShellRequestedModuleWrapper_accessors[] = {
     JS_PSG("moduleRequest", ShellRequestedModuleWrapper_moduleRequestGetter, 0),
     JS_PSG("lineNumber", ShellRequestedModuleWrapper_lineNumberGetter, 0),
     JS_PSG("columnNumber", ShellRequestedModuleWrapper_columnNumberGetter, 0),
-    JS_PS_END};
+    JS_PS_END,
+};
 
 DEFINE_GETTER_FUNCTIONS(ModuleObject, namespace_, ObjectOrNullValue,
                         IdentFilter)
-DEFINE_GETTER_FUNCTIONS(ModuleObject, status, StatusValue, IdentFilter)
+DEFINE_GETTER_FUNCTIONS(ModuleObject, status, StatusValue, GetModuleStatusName)
 DEFINE_GETTER_FUNCTIONS(ModuleObject, maybeEvaluationError, Value, IdentFilter)
 DEFINE_NATIVE_GETTER_FUNCTIONS(ModuleObject, requestedModules,
                                SpanToArrayFilter<ShellRequestedModuleWrapper>)
@@ -429,7 +514,8 @@ static const JSPropertySpec ShellModuleObjectWrapper_accessors[] = {
            ShellModuleObjectWrapper_asyncParentModulesGetter, 0),
     JS_PSG("pendingAsyncDependencies",
            ShellModuleObjectWrapper_maybePendingAsyncDependenciesGetter, 0),
-    JS_PS_END};
+    JS_PS_END,
+};
 
 #undef DEFINE_GETTER_FUNCTIONS
 #undef DEFINE_NATIVE_GETTER_FUNCTIONS
@@ -469,6 +555,7 @@ static const JSPropertySpec ShellModuleObjectWrapper_accessors[] = {
 
 DEFINE_CREATE(ModuleRequestObject, ShellModuleRequestObjectWrapper_accessors,
               nullptr)
+
 DEFINE_NATIVE_CREATE(ImportEntry, ShellImportEntryWrapper_accessors, nullptr)
 DEFINE_NATIVE_CREATE(ExportEntry, ShellExportEntryWrapper_accessors, nullptr)
 DEFINE_NATIVE_CREATE(RequestedModule, ShellRequestedModuleWrapper_accessors,
