@@ -36,7 +36,7 @@ RareArgumentsData* RareArgumentsData::create(JSContext* cx,
                                              ArgumentsObject* obj) {
   size_t bytes = RareArgumentsData::bytesRequired(obj->initialLength());
 
-  uint8_t* data = AllocateObjectBuffer<uint8_t>(cx, obj, bytes);
+  uint8_t* data = AllocNurseryOrMallocBuffer<uint8_t>(cx, obj, bytes);
   if (!data) {
     return nullptr;
   }
@@ -48,6 +48,12 @@ RareArgumentsData* RareArgumentsData::create(JSContext* cx,
   return new (data) RareArgumentsData();
 }
 
+ArgumentsData::ArgumentsData(uint32_t numArgs) : args(numArgs) {
+  // |args| must be the last field.
+  static_assert(offsetof(ArgumentsData, args) + sizeof(args) ==
+                sizeof(ArgumentsData));
+}
+
 bool ArgumentsObject::createRareData(JSContext* cx) {
   MOZ_ASSERT(!data()->rareData);
 
@@ -80,7 +86,8 @@ void ArgumentsObject::MaybeForwardToCallObject(AbstractFramePtr frame,
     obj->initFixedSlot(MAYBE_CALL_SLOT, ObjectValue(frame.callObj()));
     for (PositionalFormalParameterIter fi(script); fi; fi++) {
       if (fi.closedOver()) {
-        data->args[fi.argumentSlot()] = MagicEnvSlotValue(fi.location().slot());
+        data->args.setElement(obj, fi.argumentSlot(),
+                              MagicEnvSlotValue(fi.location().slot()));
         obj->markArgumentForwarded();
       }
     }
@@ -98,7 +105,8 @@ void ArgumentsObject::MaybeForwardToCallObject(JSFunction* callee,
     obj->initFixedSlot(MAYBE_CALL_SLOT, ObjectValue(*callObj));
     for (PositionalFormalParameterIter fi(script); fi; fi++) {
       if (fi.closedOver()) {
-        data->args[fi.argumentSlot()] = MagicEnvSlotValue(fi.location().slot());
+        data->args.setElement(obj, fi.argumentSlot(),
+                              MagicEnvSlotValue(fi.location().slot()));
         obj->markArgumentForwarded();
       }
     }
@@ -110,16 +118,20 @@ struct CopyFrameArgs {
 
   explicit CopyFrameArgs(AbstractFramePtr frame) : frame_(frame) {}
 
-  void copyActualArgs(GCPtr<Value>* dst, unsigned numActuals) const {
+  void copyActualArgs(ArgumentsObject* owner, GCOwnedArray<Value>& args,
+                      unsigned numActuals) const {
     MOZ_ASSERT_IF(frame_.isInterpreterFrame(),
                   !frame_.asInterpreterFrame()->runningInJit());
 
     // Copy arguments.
     Value* src = frame_.argv();
     Value* end = src + numActuals;
-    while (src != end) {
-      (dst++)->init(*src++);
-    }
+    args.withOwner(owner, [&](auto& args) {
+      auto* dst = args.begin();
+      while (src != end) {
+        (dst++)->init(*src++);
+      }
+    });
   }
 
   /*
@@ -138,14 +150,18 @@ struct CopyJitFrameArgs {
   CopyJitFrameArgs(jit::JitFrameLayout* frame, HandleObject callObj)
       : frame_(frame), callObj_(callObj) {}
 
-  void copyActualArgs(GCPtr<Value>* dst, unsigned numActuals) const {
+  void copyActualArgs(ArgumentsObject* owner, GCOwnedArray<Value>& args,
+                      unsigned numActuals) const {
     MOZ_ASSERT(frame_->numActualArgs() == numActuals);
 
     Value* src = frame_->actualArgs();
     Value* end = src + numActuals;
-    while (src != end) {
-      (dst++)->init(*src++);
-    }
+    args.withOwner(owner, [&](auto& args) {
+      auto* dst = args.begin();
+      while (src != end) {
+        (dst++)->init(*src++);
+      }
+    });
   }
 
   /*
@@ -180,12 +196,16 @@ struct CopyScriptFrameIterArgs {
     return true;
   }
 
-  void copyActualArgs(GCPtr<Value>* dst, unsigned numActuals) const {
+  void copyActualArgs(ArgumentsObject* owner, GCOwnedArray<Value>& args,
+                      unsigned numActuals) const {
     MOZ_ASSERT(actualArgs_.length() == numActuals);
 
-    for (Value v : actualArgs_) {
-      (dst++)->init(v);
-    }
+    args.withOwner(owner, [&](auto& args) {
+      auto* dst = args.begin();
+      for (Value v : actualArgs_) {
+        (dst++)->init(v);
+      }
+    });
   }
 
   /*
@@ -209,12 +229,16 @@ struct CopyInlinedArgs {
                   HandleFunction callee)
       : args_(args), callObj_(callObj), callee_(callee) {}
 
-  void copyActualArgs(GCPtr<Value>* dst, unsigned numActuals) const {
+  void copyActualArgs(ArgumentsObject* owner, GCOwnedArray<Value>& args,
+                      unsigned numActuals) const {
     MOZ_ASSERT(numActuals <= args_.length());
 
-    for (uint32_t i = 0; i < numActuals; i++) {
-      (dst++)->init(args_[i]);
-    }
+    args.withOwner(owner, [&](auto& args) {
+      auto* dst = args.begin();
+      for (uint32_t i = 0; i < numActuals; i++) {
+        (dst++)->init(args_[i]);
+      }
+    });
   }
 
   /*
@@ -242,13 +266,12 @@ ArgumentsObject* ArgumentsObject::createTemplateObject(JSContext* cx,
   }
 
   AutoSetNewObjectMetadata metadata(cx);
-  JSObject* base =
-      NativeObject::create(cx, FINALIZE_KIND, gc::Heap::Tenured, shape);
-  if (!base) {
+  auto* obj = NativeObject::create<ArgumentsObject>(cx, FINALIZE_KIND,
+                                                    gc::Heap::Tenured, shape);
+  if (!obj) {
     return nullptr;
   }
 
-  ArgumentsObject* obj = &base->as<js::ArgumentsObject>();
   obj->initFixedSlot(ArgumentsObject::DATA_SLOT, PrivateValue(nullptr));
   return obj;
 }
@@ -262,7 +285,7 @@ ArgumentsObject* GlobalObject::maybeArgumentsTemplateObject(bool mapped) const {
 ArgumentsObject* GlobalObject::getOrCreateArgumentsTemplateObject(JSContext* cx,
                                                                   bool mapped) {
   GlobalObjectData& data = cx->global()->data();
-  HeapPtr<ArgumentsObject*>& obj =
+  GCPtr<ArgumentsObject*>& obj =
       mapped ? data.mappedArgumentsTemplate : data.unmappedArgumentsTemplate;
 
   ArgumentsObject* templateObj = obj;
@@ -301,23 +324,21 @@ ArgumentsObject* ArgumentsObject::create(JSContext* cx, HandleFunction callee,
   unsigned numBytes = ArgumentsData::bytesRequired(numArgs);
 
   AutoSetNewObjectMetadata metadata(cx);
-  JSObject* base =
-      NativeObject::create(cx, FINALIZE_KIND, gc::Heap::Default, shape);
-  if (!base) {
+  auto* obj = NativeObject::create<ArgumentsObject>(cx, FINALIZE_KIND,
+                                                    gc::Heap::Default, shape);
+  if (!obj) {
     return nullptr;
   }
-  ArgumentsObject* obj = &base->as<ArgumentsObject>();
 
   ArgumentsData* data = reinterpret_cast<ArgumentsData*>(
-      AllocateObjectBuffer<uint8_t>(cx, obj, numBytes));
+      AllocNurseryOrMallocBuffer<uint8_t>(cx, obj, numBytes));
   if (!data) {
     // Make the object safe for GC.
     obj->initFixedSlot(DATA_SLOT, PrivateValue(nullptr));
     return nullptr;
   }
 
-  data->numArgs = numArgs;
-  data->rareData = nullptr;
+  new (data) ArgumentsData(numArgs);
 
   InitReservedSlot(obj, DATA_SLOT, data, numBytes, MemoryUse::ArgumentsData);
   obj->initFixedSlot(CALLEE_SLOT, ObjectValue(*callee));
@@ -325,13 +346,14 @@ ArgumentsObject* ArgumentsObject::create(JSContext* cx, HandleFunction callee,
                      Int32Value(numActuals << PACKED_BITS_COUNT));
 
   // Copy [0, numActuals) into data->args.
-  GCPtr<Value>* args = data->args;
-  copy.copyActualArgs(args, numActuals);
+  copy.copyActualArgs(obj, data->args, numActuals);
 
   // Fill in missing arguments with |undefined|.
-  for (size_t i = numActuals; i < numArgs; i++) {
-    args[i].init(UndefinedValue());
-  }
+  data->args.withOwner(obj, [&](auto& args) {
+    for (size_t i = numActuals; i < numArgs; i++) {
+      args[i].init(UndefinedValue());
+    }
+  });
 
   copy.maybeForwardToCallObject(obj, data);
 
@@ -417,7 +439,7 @@ ArgumentsObject* ArgumentsObject::finishPure(
   unsigned numBytes = ArgumentsData::bytesRequired(numArgs);
 
   ArgumentsData* data = reinterpret_cast<ArgumentsData*>(
-      AllocateObjectBuffer<uint8_t>(cx, obj, numBytes));
+      AllocNurseryOrMallocBuffer<uint8_t>(cx, obj, numBytes));
   if (!data) {
     // Make the object safe for GC. Don't report OOM, the slow path will
     // retry the allocation.
@@ -426,8 +448,7 @@ ArgumentsObject* ArgumentsObject::finishPure(
     return nullptr;
   }
 
-  data->numArgs = numArgs;
-  data->rareData = nullptr;
+  new (data) ArgumentsData(numArgs);
 
   obj->initFixedSlot(INITIAL_LENGTH_SLOT,
                      Int32Value(numActuals << PACKED_BITS_COUNT));
@@ -436,13 +457,14 @@ ArgumentsObject* ArgumentsObject::finishPure(
   obj->initFixedSlot(MAYBE_CALL_SLOT, UndefinedValue());
   obj->initFixedSlot(CALLEE_SLOT, ObjectValue(*callee));
 
-  GCPtr<Value>* args = data->args;
-  copy.copyActualArgs(args, numActuals);
+  copy.copyActualArgs(obj, data->args, numActuals);
 
   // Fill in missing arguments with |undefined|.
-  for (size_t i = numActuals; i < numArgs; i++) {
-    args[i].init(UndefinedValue());
-  }
+  data->args.withOwner(obj, [&](auto& args) {
+    for (size_t i = numActuals; i < numArgs; i++) {
+      args[i].init(UndefinedValue());
+    }
+  });
 
   if (callObj && callee->needsCallObject()) {
     copy.maybeForwardToCallObject(obj, data);
@@ -668,7 +690,7 @@ static bool ResolveArgumentsProperty(JSContext* cx,
 /* static */
 bool MappedArgumentsObject::obj_resolve(JSContext* cx, HandleObject obj,
                                         HandleId id, bool* resolvedp) {
-  Rooted<MappedArgumentsObject*> argsobj(cx, &obj->as<MappedArgumentsObject>());
+  auto argsobj = obj.as<MappedArgumentsObject>();
 
   if (id.isWellKnownSymbol(JS::SymbolCode::iterator)) {
     if (argsobj->hasOverriddenIterator()) {
@@ -710,7 +732,7 @@ bool MappedArgumentsObject::obj_resolve(JSContext* cx, HandleObject obj,
 
 /* static */
 bool MappedArgumentsObject::obj_enumerate(JSContext* cx, HandleObject obj) {
-  Rooted<MappedArgumentsObject*> argsobj(cx, &obj->as<MappedArgumentsObject>());
+  auto argsobj = obj.as<MappedArgumentsObject>();
 
   RootedId id(cx);
   bool found;
@@ -815,7 +837,7 @@ bool MappedArgumentsObject::obj_defineProperty(JSContext* cx, HandleObject obj,
                                                Handle<PropertyDescriptor> desc,
                                                ObjectOpResult& result) {
   // Step 1.
-  Rooted<MappedArgumentsObject*> argsobj(cx, &obj->as<MappedArgumentsObject>());
+  auto argsobj = obj.as<MappedArgumentsObject>();
 
   // Steps 2-3.
   bool isMapped = false;
@@ -941,8 +963,7 @@ bool js::UnmappedArgSetter(JSContext* cx, HandleObject obj, HandleId id,
 /* static */
 bool UnmappedArgumentsObject::obj_resolve(JSContext* cx, HandleObject obj,
                                           HandleId id, bool* resolvedp) {
-  Rooted<UnmappedArgumentsObject*> argsobj(cx,
-                                           &obj->as<UnmappedArgumentsObject>());
+  auto argsobj = obj.as<UnmappedArgumentsObject>();
 
   if (id.isWellKnownSymbol(JS::SymbolCode::iterator)) {
     if (argsobj->hasOverriddenIterator()) {
@@ -995,8 +1016,7 @@ bool UnmappedArgumentsObject::obj_resolve(JSContext* cx, HandleObject obj,
 
 /* static */
 bool UnmappedArgumentsObject::obj_enumerate(JSContext* cx, HandleObject obj) {
-  Rooted<UnmappedArgumentsObject*> argsobj(cx,
-                                           &obj->as<UnmappedArgumentsObject>());
+  auto argsobj = obj.as<UnmappedArgumentsObject>();
 
   RootedId id(cx);
   bool found;
@@ -1035,16 +1055,16 @@ void ArgumentsObject::finalize(JS::GCContext* gcx, JSObject* obj) {
                RareArgumentsData::bytesRequired(argsobj.initialLength()),
                MemoryUse::RareArgumentsData);
     gcx->free_(&argsobj, argsobj.data(),
-               ArgumentsData::bytesRequired(argsobj.data()->numArgs),
+               ArgumentsData::bytesRequired(argsobj.data()->numArgs()),
                MemoryUse::ArgumentsData);
   }
 }
 
 void ArgumentsObject::trace(JSTracer* trc, JSObject* obj) {
   ArgumentsObject& argsobj = obj->as<ArgumentsObject>();
-  if (ArgumentsData* data =
-          argsobj.data()) {  // Template objects have no ArgumentsData.
-    TraceRange(trc, data->numArgs, data->begin(), "arguments");
+  // Template objects have no ArgumentsData.
+  if (ArgumentsData* data = argsobj.data()) {
+    data->args.trace(trc);
   }
 }
 
@@ -1061,44 +1081,27 @@ size_t ArgumentsObject::objectMoved(JSObject* dst, JSObject* src) {
   Nursery& nursery = dst->runtimeFromMainThread()->gc.nursery();
 
   size_t nbytesTotal = 0;
-  uint32_t nDataBytes = ArgumentsData::bytesRequired(nsrc->data()->numArgs);
-  if (!nursery.isInside(nsrc->data())) {
-    nursery.removeMallocedBufferDuringMinorGC(nsrc->data());
-  } else {
-    AutoEnterOOMUnsafeRegion oomUnsafe;
-    uint8_t* data = nsrc->zone()->pod_malloc<uint8_t>(nDataBytes);
-    if (!data) {
-      oomUnsafe.crash(
-          "Failed to allocate ArgumentsObject data while tenuring.");
-    }
-    ndst->initFixedSlot(DATA_SLOT, PrivateValue(data));
 
-    mozilla::PodCopy(data, reinterpret_cast<uint8_t*>(nsrc->data()),
-                     nDataBytes);
+  ArgumentsData* data = nsrc->data();
+  uint32_t nDataBytes = ArgumentsData::bytesRequired(nsrc->data()->numArgs());
+  Nursery::WasBufferMoved result =
+      nursery.maybeMoveNurseryOrMallocBufferOnPromotion(
+          &data, dst, nDataBytes, MemoryUse::ArgumentsData);
+  if (result == Nursery::BufferMoved) {
+    ndst->initFixedSlot(DATA_SLOT, PrivateValue(data));
     nbytesTotal += nDataBytes;
   }
 
-  AddCellMemory(ndst, nDataBytes, MemoryUse::ArgumentsData);
-
-  if (RareArgumentsData* srcRareData = nsrc->maybeRareData()) {
-    uint32_t nbytes = RareArgumentsData::bytesRequired(nsrc->initialLength());
-    if (!nursery.isInside(srcRareData)) {
-      nursery.removeMallocedBufferDuringMinorGC(srcRareData);
-    } else {
-      AutoEnterOOMUnsafeRegion oomUnsafe;
-      uint8_t* dstRareData = nsrc->zone()->pod_malloc<uint8_t>(nbytes);
-      if (!dstRareData) {
-        oomUnsafe.crash(
-            "Failed to allocate RareArgumentsData data while tenuring.");
-      }
-      ndst->data()->rareData = (RareArgumentsData*)dstRareData;
-
-      mozilla::PodCopy(dstRareData, reinterpret_cast<uint8_t*>(srcRareData),
-                       nbytes);
-      nbytesTotal += nbytes;
+  if (RareArgumentsData* rareData = nsrc->maybeRareData()) {
+    uint32_t nRareBytes =
+        RareArgumentsData::bytesRequired(nsrc->initialLength());
+    Nursery::WasBufferMoved result =
+        nursery.maybeMoveNurseryOrMallocBufferOnPromotion(
+            &rareData, dst, nRareBytes, MemoryUse::RareArgumentsData);
+    if (result == Nursery::BufferMoved) {
+      ndst->data()->rareData = rareData;
+      nbytesTotal += nRareBytes;
     }
-
-    AddCellMemory(ndst, nbytes, MemoryUse::RareArgumentsData);
   }
 
   return nbytesTotal;
@@ -1148,7 +1151,8 @@ const JSClass MappedArgumentsObject::class_ = {
     &MappedArgumentsObject::classOps_,
     nullptr,
     &MappedArgumentsObject::classExt_,
-    &MappedArgumentsObject::objectOps_};
+    &MappedArgumentsObject::objectOps_,
+};
 
 /*
  * Unmapped arguments is significantly less magical than mapped arguments, so
@@ -1177,5 +1181,7 @@ const JSClass UnmappedArgumentsObject::class_ = {
         JSCLASS_HAS_RESERVED_SLOTS(UnmappedArgumentsObject::RESERVED_SLOTS) |
         JSCLASS_HAS_CACHED_PROTO(JSProto_Object) |
         JSCLASS_SKIP_NURSERY_FINALIZE | JSCLASS_BACKGROUND_FINALIZE,
-    &UnmappedArgumentsObject::classOps_, nullptr,
-    &UnmappedArgumentsObject::classExt_};
+    &UnmappedArgumentsObject::classOps_,
+    nullptr,
+    &UnmappedArgumentsObject::classExt_,
+};
