@@ -16,10 +16,9 @@
 #include "jit/BaselineFrame.h"   // js::jit::BaselineFrame
 #include "jit/JitFrames.h"       // js::jit::EnsureUnwoundJitExitFrame
 #include "jit/JSJitFrameIter.h"  // js::jit::{FrameType,InlineFrameIterator,JSJitFrameIter,MaybeReadFallback,SnapshotIterator}
-#include "js/ColumnNumber.h"  // JS::LimitedColumnNumberZeroOrigin, JS::TaggedColumnNumberZeroOrigin
+#include "js/ColumnNumber.h"  // JS::LimitedColumnNumberOneOrigin, JS::TaggedColumnNumberOneOrigin
 #include "js/GCAPI.h"              // JS::AutoSuppressGCAnalysis
 #include "js/Principals.h"         // JSSubsumesOp
-#include "js/RootingAPI.h"         // JS::Rooted
 #include "vm/Activation.h"         // js::Activation{,Iterator}
 #include "vm/EnvironmentObject.h"  // js::CallObject
 #include "vm/JitActivation.h"      // js::jit::JitActivation
@@ -43,7 +42,6 @@ class ArgumentsObject;
 }  // namespace js
 
 using JS::Realm;
-using JS::Rooted;
 using JS::Value;
 
 using js::AbstractFramePtr;
@@ -124,7 +122,12 @@ JS::Realm* JitFrameIter::realm() const {
     return asWasm().instance()->realm();
   }
 
-  return asJSJit().script()->realm();
+  if (asJSJit().isScripted()) {
+    return asJSJit().script()->realm();
+  }
+
+  MOZ_RELEASE_ASSERT(asJSJit().isTrampolineNative());
+  return asJSJit().callee()->realm();
 }
 
 uint8_t* JitFrameIter::resumePCinCurrentFrame() const {
@@ -181,7 +184,7 @@ void JitFrameIter::settle() {
 
   if (isWasm()) {
     const wasm::WasmFrameIter& wasmFrame = asWasm();
-    if (!wasmFrame.hasUnwoundJitFrame()) {
+    if (!wasmFrame.done() || !wasmFrame.unwoundCallerFPIsJSJit()) {
       return;
     }
 
@@ -195,16 +198,14 @@ void JitFrameIter::settle() {
     //
     // The wasm iterator has saved the previous jit frame pointer for us.
 
-    MOZ_ASSERT(wasmFrame.done());
     uint8_t* prevFP = wasmFrame.unwoundCallerFP();
-    jit::FrameType prevFrameType = wasmFrame.unwoundJitFrameType();
 
     if (mustUnwindActivation_) {
       act_->setJSExitFP(prevFP);
     }
 
     iter_.destroy();
-    iter_.construct<jit::JSJitFrameIter>(act_, prevFrameType, prevFP);
+    iter_.construct<jit::JSJitFrameIter>(act_, prevFP, mustUnwindActivation_);
     MOZ_ASSERT(!asJSJit().done());
     return;
   }
@@ -399,6 +400,7 @@ void FrameIter::nextJitFrame() {
   }
 
   MOZ_ASSERT(isWasm());
+  wasmFrame().enableInlinedFrames();
   data_.pc_ = nullptr;
 }
 
@@ -569,7 +571,7 @@ JSAtom* FrameIter::maybeFunctionDisplayAtom() const {
         return wasmFrame().functionDisplayAtom();
       }
       if (isFunctionFrame()) {
-        return calleeTemplate()->displayAtom();
+        return calleeTemplate()->fullDisplayAtom();
       }
       return nullptr;
   }
@@ -619,8 +621,7 @@ const char16_t* FrameIter::displayURL() const {
   MOZ_CRASH("Unexpected state");
 }
 
-unsigned FrameIter::computeLine(
-    JS::TaggedColumnNumberZeroOrigin* column) const {
+unsigned FrameIter::computeLine(JS::TaggedColumnNumberOneOrigin* column) const {
   switch (data_.state_) {
     case DONE:
       break;
@@ -629,10 +630,10 @@ unsigned FrameIter::computeLine(
       if (isWasm()) {
         return wasmFrame().computeLine(column);
       }
-      JS::LimitedColumnNumberZeroOrigin columnNumber;
+      JS::LimitedColumnNumberOneOrigin columnNumber;
       unsigned lineNumber = PCToLineNumber(script(), pc(), &columnNumber);
       if (column) {
-        *column = JS::TaggedColumnNumberZeroOrigin(columnNumber);
+        *column = JS::TaggedColumnNumberOneOrigin(columnNumber);
       }
       return lineNumber;
   }
@@ -779,7 +780,7 @@ void FrameIter::wasmUpdateBytecodeOffset() {
 
   // Relookup the current frame, updating the bytecode offset in the process.
   data_.jitFrames_ = JitFrameIter(data_.activations_->asJit());
-  while (wasmFrame().debugFrame() != frame) {
+  while (!isWasm() || wasmFrame().debugFrame() != frame) {
     ++data_.jitFrames_;
   }
 
@@ -825,7 +826,7 @@ bool FrameIter::matchCallee(JSContext* cx, JS::Handle<JSFunction*> fun) const {
   // Use the calleeTemplate to rule out a match without needing to invalidate to
   // find the actual callee. The real callee my be a clone of the template which
   // should *not* be considered a match.
-  Rooted<JSFunction*> currentCallee(cx, calleeTemplate());
+  JSFunction* currentCallee = calleeTemplate();
 
   if (currentCallee->nargs() != fun->nargs()) {
     return false;
@@ -841,7 +842,7 @@ bool FrameIter::matchCallee(JSContext* cx, JS::Handle<JSFunction*> fun) const {
   // group and Ion will not inline them interchangeably.
   //
   // See: js::jit::InlineFrameIterator::findNextFrame()
-  if (currentCallee->hasBaseScript()) {
+  if (currentCallee->hasBaseScript() && fun->hasBaseScript()) {
     if (currentCallee->baseScript() != fun->baseScript()) {
       return false;
     }
