@@ -27,7 +27,7 @@
 
 #include "frontend/ScriptIndex.h"  // ScriptIndex
 #include "gc/Barrier.h"
-#include "js/ColumnNumber.h"  // JS::LimitedColumnNumberZeroOrigin
+#include "js/ColumnNumber.h"  // JS::LimitedColumnNumberOneOrigin, JS::LimitedColumnNumberOneOrigin
 #include "js/CompileOptions.h"
 #include "js/Transcoding.h"
 #include "js/UbiNode.h"
@@ -86,13 +86,14 @@ namespace frontend {
 struct CompilationStencil;
 struct ExtensibleCompilationStencil;
 struct CompilationGCOutput;
+struct InitialStencilAndDelazifications;
 struct CompilationStencilMerger;
 class StencilXDR;
 }  // namespace frontend
 
 class ScriptCounts {
  public:
-  typedef mozilla::Vector<PCCounts, 0, SystemAllocPolicy> PCCountsVector;
+  using PCCountsVector = mozilla::Vector<PCCounts, 0, SystemAllocPolicy>;
 
   inline ScriptCounts();
   inline explicit ScriptCounts(PCCountsVector&& jumpTargets);
@@ -191,29 +192,6 @@ using ScriptFinalWarmUpCountMap =
                        DefaultHasher<HeapPtr<BaseScript*>>, SystemAllocPolicy>;
 #endif
 
-// As we execute JS sources that used lazy parsing, we may generate additional
-// bytecode that we would like to include in caches if they are being used.
-// There is a dependency cycle between JSScript / ScriptSource /
-// CompilationStencil for this scenario so introduce this smart-ptr wrapper to
-// avoid needing the full details of the stencil-merger in this file.
-class StencilIncrementalEncoderPtr {
- public:
-  frontend::CompilationStencilMerger* merger_ = nullptr;
-
-  StencilIncrementalEncoderPtr() = default;
-  ~StencilIncrementalEncoderPtr() { reset(); }
-
-  bool hasEncoder() const { return bool(merger_); }
-
-  void reset();
-
-  bool setInitial(JSContext* cx,
-                  UniquePtr<frontend::ExtensibleCompilationStencil>&& initial);
-
-  bool addDelazification(JSContext* cx,
-                         const frontend::CompilationStencil& delazification);
-};
-
 struct ScriptSourceChunk {
   ScriptSource* ss = nullptr;
   uint32_t chunk = 0;
@@ -254,8 +232,8 @@ using SourceData = mozilla::UniquePtr<void, JS::FreePolicy>;
 
 template <typename Unit>
 inline SourceData ToSourceData(EntryUnits<Unit> chars) {
-  static_assert(std::is_same_v<SourceData::DeleterType,
-                               typename EntryUnits<Unit>::DeleterType>,
+  static_assert(std::is_same_v<SourceData::deleter_type,
+                               typename EntryUnits<Unit>::deleter_type>,
                 "EntryUnits and SourceData must share the same deleter "
                 "type, that need not know the type of the data being freed, "
                 "for the upcast below to be safe");
@@ -597,11 +575,6 @@ class ScriptSource {
   SharedImmutableTwoByteString displayURL_;
   SharedImmutableTwoByteString sourceMapURL_;
 
-  // The bytecode cache encoder is used to encode only the content of function
-  // which are delazified.  If this value is not nullptr, then each delazified
-  // function should be recorded before their first execution.
-  StencilIncrementalEncoderPtr xdrEncoder_;
-
   // A string indicating how this source code was introduced into the system.
   // This is a constant, statically allocated C string, so does not need memory
   // management.
@@ -623,7 +596,7 @@ class ScriptSource {
   uint32_t startLine_ = 0;
   // Column number within the file where this source starts,
   // in UTF-16 code units.
-  JS::LimitedColumnNumberZeroOrigin startColumn_;
+  JS::LimitedColumnNumberOneOrigin startColumn_;
 
   // See: CompileOptions::mutedErrors.
   bool mutedErrors_ = false;
@@ -909,7 +882,7 @@ class ScriptSource {
   JSLinearString* substringDontDeflate(JSContext* cx, size_t start,
                                        size_t stop);
 
-  [[nodiscard]] bool appendSubstring(JSContext* cx, js::StringBuffer& buf,
+  [[nodiscard]] bool appendSubstring(JSContext* cx, js::StringBuilder& buf,
                                      size_t start, size_t stop);
 
   void setParameterListEnd(uint32_t parameterListEnd) {
@@ -1030,6 +1003,9 @@ class ScriptSource {
   [[nodiscard]] bool setFilename(FrontendContext* fc, const char* filename);
   [[nodiscard]] bool setFilename(FrontendContext* fc, UniqueChars&& filename);
 
+  bool hasIntroducerFilename() const {
+    return introducerFilename_ ? true : false;
+  }
   const char* introducerFilename() const {
     return introducerFilename_ ? introducerFilename_.chars() : filename();
   }
@@ -1063,7 +1039,7 @@ class ScriptSource {
   bool mutedErrors() const { return mutedErrors_; }
 
   uint32_t startLine() const { return startLine_; }
-  JS::LimitedColumnNumberZeroOrigin startColumn() const { return startColumn_; }
+  JS::LimitedColumnNumberOneOrigin startColumn() const { return startColumn_; }
 
   JS::DelazificationOption delazificationMode() const {
     return delazificationMode_;
@@ -1076,24 +1052,6 @@ class ScriptSource {
     MOZ_ASSERT(offset <= (uint32_t)INT32_MAX);
     introductionOffset_.emplace(offset);
   }
-
-  // Return wether an XDR encoder is present or not.
-  bool hasEncoder() const { return xdrEncoder_.hasEncoder(); }
-
-  [[nodiscard]] bool startIncrementalEncoding(
-      JSContext* cx,
-      UniquePtr<frontend::ExtensibleCompilationStencil>&& initial);
-
-  [[nodiscard]] bool addDelazificationToIncrementalEncoding(
-      JSContext* cx, const frontend::CompilationStencil& stencil);
-
-  // Linearize the encoded content in the |buffer| provided as argument to
-  // |xdrEncodeTopLevel|, and free the XDR encoder.  In case of errors, the
-  // |buffer| is considered undefined.
-  bool xdrFinalizeEncoder(JSContext* cx, JS::TranscodeBuffer& buffer);
-
-  // Discard the incremental encoding data and free the XDR encoder.
-  void xdrAbortEncoder();
 };
 
 // [SMDOC] ScriptSourceObject
@@ -1166,12 +1124,75 @@ class ScriptSourceObject : public NativeObject {
 #endif
 
   enum {
-    SOURCE_SLOT = 0,
+    PRIVATE_SLOT = 0,  // Must be first slot for JSCLASS_SLOT0_IS_NSISUPPORTS.
+    SOURCE_SLOT,
     ELEMENT_PROPERTY_SLOT,
     INTRODUCTION_SCRIPT_SLOT,
-    PRIVATE_SLOT,
+    STENCILS_SLOT,
     RESERVED_SLOTS
   };
+
+  // Delazification stencils can be aggregated in
+  // InitialStencilAndDelazification, this structure might be used for
+  // different purposes.
+  //  - Collecting: The goal is to aggregate all delazified functions in order
+  //    to aggregate them for serialization.
+  //  - Sharing: The goal is to use the InitialStencilAndDelazification as a way
+  //    to share multiple threads efforts towards parsing a Script Source
+  //    content.
+  //
+  // See setCollectingDelazifications and setSharingDelazifications for details.
+  static constexpr uintptr_t STENCILS_COLLECTING_DELAZIFICATIONS_FLAG = 0x1;
+  static constexpr uintptr_t STENCILS_SHARING_DELAZIFICATIONS_FLAG = 0x2;
+  static constexpr uintptr_t STENCILS_MASK = 0x3;
+
+  void clearStencils();
+
+  template <uintptr_t flag>
+  void setStencilsFlag();
+
+  template <uintptr_t flag>
+  void unsetStencilsFlag();
+
+  template <uintptr_t flag>
+  bool isStencilsFlagSet() const;
+
+ public:
+  // Associate stencils to this ScriptSourceObject.
+  // The consumer should call setCollectingDelazifications or
+  // setSharingDelazifications after this.
+  void setStencils(
+      already_AddRefed<frontend::InitialStencilAndDelazifications> stencils);
+
+  // Start collecting delazifications.
+  // This is a temporary state until unsetCollectingDelazifications is called,
+  // and this expects a pair of set/unset call.
+  //
+  // The caller should check isCollectingDelazifications before calling this.
+  void setCollectingDelazifications();
+
+  // Clear the flag for collecting delazifications.
+  //
+  // If setSharingDelazifications wasn't called, this clears the association
+  // with the stencils.
+  void unsetCollectingDelazifications();
+
+  // Returns true if setCollectingDelazifications was called and
+  // unsetCollectingDelazifications is not yet called.
+  bool isCollectingDelazifications() const;
+
+  // Start sharing delazifications with others.
+  // This is a permanent state.
+  //
+  // The flag is orthogonal to setCollectingDelazifications.
+  void setSharingDelazifications();
+
+  // Returns true if setSharingDelazifications was called.
+  bool isSharingDelazifications() const;
+
+  // Return the associated stencils if any.
+  // Returns nullptr if stencils is not associated
+  frontend::InitialStencilAndDelazifications* maybeGetStencils();
 };
 
 // ScriptWarmUpData represents a pointer-sized field in BaseScript that stores
@@ -1301,7 +1322,8 @@ static_assert(sizeof(ScriptWarmUpData) == sizeof(uintptr_t),
 // This class doesn't use the GC barrier wrapper classes. BaseScript::swapData
 // performs a manual pre-write barrier when detaching PrivateScriptData from a
 // script.
-class alignas(uintptr_t) PrivateScriptData final : public TrailingArray {
+class alignas(uintptr_t) PrivateScriptData final
+    : public TrailingArray<PrivateScriptData> {
  private:
   uint32_t ngcthings = 0;
 
@@ -1540,12 +1562,12 @@ class BaseScript : public gc::TenuredCellWithNonGCPointer<uint8_t> {
   SourceExtent extent() const { return extent_; }
 
   [[nodiscard]] bool appendSourceDataForToString(JSContext* cx,
-                                                 js::StringBuffer& buf);
+                                                 js::StringBuilder& buf);
 
   // Line number (1-origin)
   uint32_t lineno() const { return extent_.lineno; }
   // Column number in UTF-16 code units
-  JS::LimitedColumnNumberZeroOrigin column() const { return extent_.column; }
+  JS::LimitedColumnNumberOneOrigin column() const { return extent_.column; }
 
   JS::DelazificationOption delazificationMode() const {
     return scriptSource()->delazificationMode();
@@ -1661,6 +1683,10 @@ class BaseScript : public gc::TenuredCellWithNonGCPointer<uint8_t> {
   static constexpr size_t offsetOfWarmUpData() {
     return offsetof(BaseScript, warmUpData_);
   }
+
+#if defined(DEBUG) || defined(JS_JITSPEW)
+  void dumpStringContent(js::GenericPrinter& out) const;
+#endif
 };
 
 extern void SweepScriptData(JSRuntime* rt);
@@ -1888,6 +1914,7 @@ class JSScript : public js::BaseScript {
   inline bool canIonCompile() const;
   inline void disableIon();
 
+  inline bool isBaselineCompilingOffThread() const;
   inline bool canBaselineCompile() const;
   inline void disableBaselineCompile();
 
@@ -1932,6 +1959,9 @@ class JSScript : public js::BaseScript {
   inline void incWarmUpCounter();
   inline void resetWarmUpCounterForGC();
 
+  inline void updateLastICStubCounter();
+  inline uint32_t warmUpCountAtLastICStub() const;
+
   void resetWarmUpCounterToDelayIonCompilation();
 
   unsigned getWarmUpResetCount() const {
@@ -1980,7 +2010,7 @@ class JSScript : public js::BaseScript {
 
   void addSizeOfJitScript(mozilla::MallocSizeOf mallocSizeOf,
                           size_t* sizeOfJitScript,
-                          size_t* sizeOfBaselineFallbackStubs) const;
+                          size_t* sizeOfAllocSites) const;
 
   mozilla::Span<const js::TryNote> trynotes() const {
     return immutableScriptData()->tryNotes();
@@ -2014,6 +2044,10 @@ class JSScript : public js::BaseScript {
     MOZ_ASSERT(sharedData_);
     return immutableScriptData()->notes();
   }
+  js::SrcNote* notesEnd() const {
+    MOZ_ASSERT(sharedData_);
+    return immutableScriptData()->notes() + numNotes();
+  }
 
   JSString* getString(js::GCThingIndex index) const {
     return &gcthings()[index].as<JSString>();
@@ -2025,6 +2059,23 @@ class JSScript : public js::BaseScript {
     return getString(GET_GCTHING_INDEX(pc));
   }
 
+  bool atomizeString(JSContext* cx, jsbytecode* pc) {
+    MOZ_ASSERT(containsPC<js::GCThingIndex>(pc));
+    MOZ_ASSERT(js::JOF_OPTYPE((JSOp)*pc) == JOF_STRING);
+    js::GCThingIndex index = GET_GCTHING_INDEX(pc);
+    JSString* str = getString(index);
+    if (str->isAtom()) {
+      return true;
+    }
+    JSAtom* atom = js::AtomizeString(cx, str);
+    if (!atom) {
+      return false;
+    }
+    js::gc::CellPtrPreWriteBarrier(data_->gcthings()[index]);
+    data_->gcthings()[index] = JS::GCCellPtr(atom);
+    return true;
+  }
+
   JSAtom* getAtom(js::GCThingIndex index) const {
     return &gcthings()[index].as<JSString>().asAtom();
   }
@@ -2126,10 +2177,6 @@ class JSScript : public js::BaseScript {
   // invariants of debuggee compartments, scripts, and frames.
   inline bool isDebuggee() const;
 
-  // Create an allocation site associated with this script/JitScript to track
-  // nursery allocations.
-  js::gc::AllocSite* createAllocSite();
-
   // A helper class to prevent relazification of the given function's script
   // while it's holding on to it.  This class automatically roots the script.
   class AutoDelazify;
@@ -2172,15 +2219,15 @@ class JSScript : public js::BaseScript {
   void dumpRecursive(JSContext* cx);
 
   static bool dump(JSContext* cx, JS::Handle<JSScript*> script,
-                   DumpOptions& options, js::Sprinter* sp);
+                   DumpOptions& options, js::StringPrinter* sp);
   static bool dumpSrcNotes(JSContext* cx, JS::Handle<JSScript*> script,
-                           js::Sprinter* sp);
+                           js::GenericPrinter* sp);
   static bool dumpTryNotes(JSContext* cx, JS::Handle<JSScript*> script,
-                           js::Sprinter* sp);
+                           js::GenericPrinter* sp);
   static bool dumpScopeNotes(JSContext* cx, JS::Handle<JSScript*> script,
-                             js::Sprinter* sp);
+                             js::GenericPrinter* sp);
   static bool dumpGCThings(JSContext* cx, JS::Handle<JSScript*> script,
-                           js::Sprinter* sp);
+                           js::GenericPrinter* sp);
 #endif
 };
 
@@ -2212,17 +2259,10 @@ extern JS::UniqueChars FormatIntroducedFilename(const char* filename,
                                                 uint32_t lineno,
                                                 const char* introducer);
 
-struct GSNCache;
-
-const js::SrcNote* GetSrcNote(GSNCache& cache, JSScript* script,
-                              jsbytecode* pc);
-
-extern const js::SrcNote* GetSrcNote(JSContext* cx, JSScript* script,
-                                     jsbytecode* pc);
-
 extern jsbytecode* LineNumberToPC(JSScript* script, unsigned lineno);
 
-extern JS_PUBLIC_API unsigned GetScriptLineExtent(JSScript* script);
+extern JS_PUBLIC_API unsigned GetScriptLineExtent(
+    JSScript* script, JS::LimitedColumnNumberOneOrigin* columnp = nullptr);
 
 #ifdef JS_CACHEIR_SPEW
 void maybeUpdateWarmUpCount(JSScript* script);
@@ -2235,12 +2275,12 @@ namespace js {
 
 extern unsigned PCToLineNumber(
     JSScript* script, jsbytecode* pc,
-    JS::LimitedColumnNumberZeroOrigin* columnp = nullptr);
+    JS::LimitedColumnNumberOneOrigin* columnp = nullptr);
 
 extern unsigned PCToLineNumber(
-    unsigned startLine, JS::LimitedColumnNumberZeroOrigin startCol,
-    SrcNote* notes, jsbytecode* code, jsbytecode* pc,
-    JS::LimitedColumnNumberZeroOrigin* columnp = nullptr);
+    unsigned startLine, JS::LimitedColumnNumberOneOrigin startCol,
+    SrcNote* notes, SrcNote* notesEnd, jsbytecode* code, jsbytecode* pc,
+    JS::LimitedColumnNumberOneOrigin* columnp = nullptr);
 
 /*
  * This function returns the file and line number of the script currently
