@@ -302,7 +302,24 @@ void WaylandSurface::RequestFrameCallbackLocked(
   MOZ_DIAGNOSTIC_ASSERT(&aProofOfLock == mSurfaceLock);
 
   // Frame callback will be added by Map.
-  if (!mIsMapped || !mFrameCallbackEnabled || !mFrameCallbackHandler.IsSet()) {
+  if (!mIsMapped) {
+    return;
+  }
+
+  if (mPendingOpaqueRegion && !mOpaqueRegionFrameCallback) {
+    LOGVERBOSE(
+        "WaylandSurface::RequestFrameCallbackLocked(): add opaque frame "
+        "callback handler");
+    static const struct wl_callback_listener listener{
+        [](void* aData, struct wl_callback* callback, uint32_t time) {
+          RefPtr waylandSurface = static_cast<WaylandSurface*>(aData);
+          waylandSurface->OpaqueCallbackHandler();
+        }};
+    mOpaqueRegionFrameCallback = wl_surface_frame(mSurface);
+    wl_callback_add_listener(mOpaqueRegionFrameCallback, &listener, this);
+  }
+
+  if (!mFrameCallbackEnabled || !mFrameCallbackHandler.IsSet()) {
     return;
   }
 
@@ -628,6 +645,9 @@ void WaylandSurface::UnmapLocked(WaylandSurfaceLock& aSurfaceLock) {
   mParentSurface = nullptr;
   mFormats = nullptr;
 
+  MozClearPointer(mPendingOpaqueRegion, wl_region_destroy);
+  MozClearPointer(mOpaqueRegionFrameCallback, wl_callback_destroy);
+
   // We can't release mSurface if it's used by Gdk for frame callback routing,
   // it will be released at GdkCleanUpLocked().
   if (!mIsPendingGdkCleanup) {
@@ -640,6 +660,17 @@ void WaylandSurface::UnmapLocked(WaylandSurfaceLock& aSurfaceLock) {
   // Remove references to WaylandBuffers attached to mSurface,
   // we don't want to get any buffer release callback when we're unmapped.
   ReleaseAllWaylandTransactionsLocked(aSurfaceLock);
+
+  // Add ref until all events are processed
+  AddRef();
+  static const struct wl_callback_listener listener{
+      [](void* aData, struct wl_callback* callback, uint32_t time) {
+        RefPtr surface = dont_AddRef(static_cast<WaylandSurface*>(aData));
+        LOGS_VERBOSE("WaylandSurface::UnmapLocked() finished callback [%p] ",
+                     aData);
+      }};
+  wl_callback_add_listener(wl_display_sync(WaylandDisplayGetWLDisplay()),
+                           &listener, this);
 }
 
 void WaylandSurface::Commit(WaylandSurfaceLock* aProofOfLock, bool aForceCommit,
@@ -703,6 +734,29 @@ bool WaylandSurface::DisableUserInputLocked(
   return true;
 }
 
+void WaylandSurface::OpaqueCallbackHandler() {
+  WaylandSurfaceLock lock(this);
+  if (mPendingOpaqueRegion) {
+    LOGVERBOSE("WaylandSurface::OpaqueCallbackHandler()");
+    wl_surface_set_opaque_region(mSurface, mPendingOpaqueRegion);
+    MozClearPointer(mPendingOpaqueRegion, wl_region_destroy);
+    mSurfaceNeedsCommit = true;
+  }
+}
+
+void WaylandSurface::SetOpaqueLocked(const WaylandSurfaceLock& aProofOfLock) {
+  MOZ_DIAGNOSTIC_ASSERT(&aProofOfLock == mSurfaceLock);
+  if (!mSurface || !IsOpaqueRegionEnabled()) {
+    return;
+  }
+  LOGVERBOSE("WaylandSurface::SetOpaqueLocked()");
+  MozClearPointer(mPendingOpaqueRegion, wl_region_destroy);
+  mPendingOpaqueRegion =
+      wl_compositor_create_region(WaylandDisplayGet()->GetCompositor());
+  wl_region_add(mPendingOpaqueRegion, 0, 0, INT32_MAX, INT32_MAX);
+  RequestFrameCallbackLocked(aProofOfLock);
+}
+
 void WaylandSurface::SetOpaqueRegionLocked(
     const WaylandSurfaceLock& aProofOfLock, const gfx::IntRegion& aRegion) {
   MOZ_DIAGNOSTIC_ASSERT(&aProofOfLock == mSurfaceLock);
@@ -714,18 +768,19 @@ void WaylandSurface::SetOpaqueRegionLocked(
 
   // Region should be in surface-logical coordinates, so we need to divide by
   // the buffer scale. We use round-in in order to be safe with subpixels.
-  UnknownScaleFactor scale(GetScaleSafe());
-  wl_region* region =
+  UnknownScaleFactor scale(GetScale());
+
+  MozClearPointer(mPendingOpaqueRegion, wl_region_destroy);
+  mPendingOpaqueRegion =
       wl_compositor_create_region(WaylandDisplayGet()->GetCompositor());
   for (auto iter = aRegion.RectIter(); !iter.Done(); iter.Next()) {
     const auto& rect = gfx::RoundedIn(iter.Get().ToUnknownRect() / scale);
-    wl_region_add(region, rect.x, rect.y, rect.Width(), rect.Height());
+    wl_region_add(mPendingOpaqueRegion, rect.x, rect.y, rect.Width(),
+                  rect.Height());
     LOGVERBOSE("  region [%d, %d] -> [%d x %d]", rect.x, rect.y, rect.Width(),
                rect.Height());
   }
-  wl_surface_set_opaque_region(mSurface, region);
-  wl_region_destroy(region);
-  mSurfaceNeedsCommit = true;
+  RequestFrameCallbackLocked(aProofOfLock);
 }
 
 void WaylandSurface::SetOpaqueRegion(const gfx::IntRegion& aRegion) {
@@ -733,20 +788,6 @@ void WaylandSurface::SetOpaqueRegion(const gfx::IntRegion& aRegion) {
   SetOpaqueRegionLocked(lock, aRegion);
 }
 
-void WaylandSurface::SetOpaqueLocked(const WaylandSurfaceLock& aProofOfLock) {
-  MOZ_DIAGNOSTIC_ASSERT(&aProofOfLock == mSurfaceLock);
-  if (!mSurface || !IsOpaqueRegionEnabled()) {
-    return;
-  }
-  LOGVERBOSE("WaylandSurface::SetOpaqueLocked()");
-  wl_region* region =
-      wl_compositor_create_region(WaylandDisplayGet()->GetCompositor());
-  wl_region_add(region, 0, 0, INT32_MAX, INT32_MAX);
-  wl_surface_set_opaque_region(mSurface, region);
-  wl_region_destroy(region);
-  mSurfaceNeedsCommit = true;
-}
-
 void WaylandSurface::ClearOpaqueRegionLocked(
     const WaylandSurfaceLock& aProofOfLock) {
   MOZ_DIAGNOSTIC_ASSERT(&aProofOfLock == mSurfaceLock);
@@ -754,11 +795,10 @@ void WaylandSurface::ClearOpaqueRegionLocked(
     return;
   }
   LOGVERBOSE("WaylandSurface::ClearOpaqueLocked()");
-  wl_region* region =
+  MozClearPointer(mPendingOpaqueRegion, wl_region_destroy);
+  mPendingOpaqueRegion =
       wl_compositor_create_region(WaylandDisplayGet()->GetCompositor());
-  wl_surface_set_opaque_region(mSurface, region);
-  wl_region_destroy(region);
-  mSurfaceNeedsCommit = true;
+  RequestFrameCallbackLocked(aProofOfLock);
 }
 
 void WaylandSurface::FractionalScaleHandler(void* data,
@@ -837,11 +877,7 @@ void WaylandSurface::ClearScaleLocked(const WaylandSurfaceLock& aProofOfLock) {
 void WaylandSurface::SetCeiledScaleLocked(
     const WaylandSurfaceLock& aProofOfLock, int aScreenCeiledScale) {
   MOZ_DIAGNOSTIC_ASSERT(&aProofOfLock == mSurfaceLock);
-
-  // Allow to set scale for unmapped surfaces unconditionally.
-  // It sets initial scale factor so we have something to work with.
-  if (!mIsMapped || IsCeiledScaleLocked(aProofOfLock)) {
-    // mScreenScale = (double)aScreenCeiledScale;
+  if (IsCeiledScaleLocked(aProofOfLock)) {
     mScreenScale = aScreenCeiledScale;
     LOGWAYLAND("WaylandSurface::SetCeiledScaleLocked() scale %f",
                (double)mScreenScale);
@@ -1023,7 +1059,7 @@ wl_egl_window* WaylandSurface::GetEGLWindow(nsIntSize aUnscaledSize) {
     return nullptr;
   }
 
-  auto scale = GetScaleSafe();
+  auto scale = GetScale();
   // TODO: Correct size rounding
   nsIntSize scaledSize((int)floor(aUnscaledSize.width * scale),
                        (int)floor(aUnscaledSize.height * scale));
@@ -1059,7 +1095,7 @@ bool WaylandSurface::SetEGLWindowSize(nsIntSize aScaledSize) {
     return true;
   }
 
-  auto scale = GetScaleSafe();
+  auto scale = GetScale();
   // TODO: Avoid precision lost here? Load coordinates from window?
   nsIntSize unscaledSize((int)round(aScaledSize.width / scale),
                          (int)round(aScaledSize.height / scale));
@@ -1105,7 +1141,7 @@ void WaylandSurface::RemoveTransactionLocked(
   if (mBufferTransactions.IsEmpty()) {
     return;
   }
-  LOGWAYLAND("WaylandSurface::RemoveTransactionLocked() [%p]",
+  LOGVERBOSE("WaylandSurface::RemoveTransactionLocked() [%p]",
              (void*)aTransaction);
   MOZ_DIAGNOSTIC_ASSERT(aTransaction->IsDeleted());
   [[maybe_unused]] bool removed =
@@ -1128,6 +1164,10 @@ BufferTransaction* WaylandSurface::GetNextTransactionLocked(
   // DeleteTransactionLocked() may delete BufferTransaction so
   // iterate with ref taken.
   for (auto t : transactions) {
+    LOGVERBOSE(
+        "WaylandSurface::GetNextTransactionLocked() transaction [%p] det %d "
+        "del %d",
+        t.get(), t->IsDetached(), t->IsDeleted());
     if (t == nextTransaction) {
       mBufferTransactions.AppendElement(nextTransaction);
       addedNext = true;
@@ -1157,7 +1197,7 @@ bool WaylandSurface::AttachLocked(const WaylandSurfaceLock& aSurfaceLock,
   MOZ_DIAGNOSTIC_ASSERT(&aSurfaceLock == mSurfaceLock);
   MOZ_DIAGNOSTIC_ASSERT(mSurface);
 
-  auto scale = GetScaleSafe();
+  auto scale = GetScale();
   LayoutDeviceIntSize bufferSize = aBuffer->GetSize();
   // TODO: rounding?
   SetSizeLocked(aSurfaceLock, gfx::IntSize(bufferSize.width, bufferSize.height),
@@ -1269,18 +1309,7 @@ double WaylandSurface::GetScale() {
     return scale;
   }
 
-  LOGVERBOSE("WaylandSurface::GetScale() no scale available");
-  return sNoScale;
-}
-
-double WaylandSurface::GetScaleSafe() {
-  double scale = GetScale();
-  if (scale != sNoScale) {
-    return scale;
-  }
-
-  // Return default scale for now - we'll repaint after show anyway.
-  return 1;
+  return ScreenHelperGTK::GetGTKMonitorFractionalScaleFactor();
 }
 
 void WaylandSurface::SetParentLocked(const WaylandSurfaceLock& aProofOfLock,
