@@ -23,10 +23,14 @@
 #endif
 
 #ifdef MOZ_WIDGET_ANDROID
-#  include "AndroidBridge.h"
+#  include "AndroidBuild.h"
 #  include "mozilla/java/GeckoAppShellWrappers.h"
 #endif
 
+#ifdef MOZ_BACKGROUNDTASKS
+#  include "mozilla/BackgroundTasks.h"
+#endif
+
 #include "mozilla/Services.h"
 
 #include "nsCRT.h"
@@ -66,14 +70,6 @@ static const ManifestDirective kParsingTable[] = {
     "manifest",         1, true, false,
     &nsComponentManagerImpl::ManifestManifest, nullptr,
   },
-  {
-    "component",        2, false, false,
-    &nsComponentManagerImpl::ManifestComponent, nullptr,
-  },
-  {
-    "contract",         2, false, false,
-    &nsComponentManagerImpl::ManifestContract, nullptr,
-  },
   {
     "category",         3, false, false,
     &nsComponentManagerImpl::ManifestCategory, nullptr,
@@ -91,8 +87,6 @@ static const ManifestDirective kParsingTable[] = {
     nullptr, &nsChromeRegistry::ManifestSkin,
   },
   {
-    // NB: note that while skin manifests can use this, they are only allowed
-    // to use it for chrome://../skin/ URLs
     "override",         2, true, false,
     nullptr, &nsChromeRegistry::ManifestOverride,
   },
@@ -122,7 +116,7 @@ void LogMessage(const char* aMsg, ...) {
   va_end(args);
 
   nsCOMPtr<nsIConsoleMessage> error =
-      new nsConsoleMessage(NS_ConvertUTF8toUTF16(formatted.get()).get());
+      new nsConsoleMessage(NS_ConvertUTF8toUTF16(formatted.get()));
   console->LogMessage(error);
 }
 
@@ -156,11 +150,10 @@ void LogMessageWithContext(FileLocation& aFile, uint32_t aLineNumber,
     return;
   }
 
-  nsresult rv = error->Init(NS_ConvertUTF8toUTF16(formatted.get()),
-                            NS_ConvertUTF8toUTF16(file), u""_ns, aLineNumber, 0,
-                            nsIScriptError::warningFlag, "chrome registration",
-                            false /* from private window */,
-                            true /* from chrome context */);
+  nsresult rv = error->Init(
+      NS_ConvertUTF8toUTF16(formatted.get()), file, aLineNumber, 0,
+      nsIScriptError::warningFlag, "chrome registration"_ns,
+      false /* from private window */, true /* from chrome context */);
   if (NS_FAILED(rv)) {
     return;
   }
@@ -402,6 +395,10 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
 #if defined(MOZ_WIDGET_ANDROID)
   constexpr auto kTablet = u"tablet"_ns;
 #endif
+  // You might expect this to be guarded by MOZ_BACKGROUNDTASKS, but it's not
+  // possible to have conditional manifest contents, so we need to recognize and
+  // discard these tokens even when MOZ_BACKGROUNDTASKS is not set.
+  constexpr auto kBackgroundTask = u"backgroundtask"_ns;
 
   constexpr auto kMain = u"main"_ns;
   constexpr auto kContent = u"content"_ns;
@@ -471,9 +468,9 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
                             gtk_minor_version);
 #elif defined(MOZ_WIDGET_ANDROID)
   bool isTablet = false;
-  if (mozilla::AndroidBridge::Bridge()) {
-    mozilla::AndroidBridge::Bridge()->GetStaticStringField(
-        "android/os/Build$VERSION", "RELEASE", osVersion);
+  if (jni::IsAvailable()) {
+    jni::String::LocalRef release = java::sdk::Build::VERSION::RELEASE();
+    osVersion.Assign(release->ToString());
     isTablet = java::GeckoAppShell::IsTablet();
   }
 #endif
@@ -484,10 +481,6 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
     process = kMain;
   }
 
-  // Because contracts must be registered after CIDs, we save and process them
-  // at the end.
-  nsTArray<CachedDirective> contracts;
-
   char* token;
   char* newline = aBuf;
   uint32_t line = 0;
@@ -525,7 +518,7 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
 
     const ManifestDirective* directive = nullptr;
     for (const ManifestDirective* d = kParsingTable;
-         d < ArrayEnd(kParsingTable); ++d) {
+         d < std::end(kParsingTable); ++d) {
       if (!strcmp(d->directive, token)) {
         directive = d;
         break;
@@ -539,13 +532,6 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
       continue;
     }
 
-    if (!directive->ischrome && NS_BOOTSTRAPPED_LOCATION == aType) {
-      LogMessageWithContext(
-          aFile, line,
-          "Bootstrapped manifest not allowed to use '%s' directive.", token);
-      continue;
-    }
-
     NS_ASSERTION(directive->argc < 4, "Need to reset argv array length");
     char* argv[4];
     for (int i = 0; i < directive->argc; ++i) {
@@ -570,6 +556,14 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
     TriState stProcess = eUnspecified;
 #if defined(MOZ_WIDGET_ANDROID)
     TriState stTablet = eUnspecified;
+#endif
+#ifdef MOZ_BACKGROUNDTASKS
+    // When in background task mode, default to not registering
+    // category directivies unless backgroundtask=1 is specified.
+    TriState stBackgroundTask = (BackgroundTasks::IsBackgroundTaskMode() &&
+                                 strcmp("category", directive->directive) == 0)
+                                    ? eBad
+                                    : eUnspecified;
 #endif
     int flags = 0;
 
@@ -597,6 +591,18 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
       }
 #endif
 
+      // You might expect this to be guarded by MOZ_BACKGROUNDTASKS, it's not
+      // possible to have conditional manifest contents.
+      bool flag;
+      if (CheckFlag(kBackgroundTask, wtoken, flag)) {
+#if defined(MOZ_BACKGROUNDTASKS)
+        // Background task mode is active: filter.
+        stBackgroundTask =
+            (flag == BackgroundTasks::IsBackgroundTaskMode()) ? eOK : eBad;
+#endif /* defined(MOZ_BACKGROUNDTASKS) */
+        continue;
+      }
+
       if (directive->contentflags) {
         bool flag;
         if (CheckFlag(kContentAccessible, wtoken, flag)) {
@@ -630,6 +636,9 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
         stGeckoVersion == eBad || stOs == eBad || stOsVersion == eBad ||
 #ifdef MOZ_WIDGET_ANDROID
         stTablet == eBad ||
+#endif
+#ifdef MOZ_BACKGROUNDTASKS
+        stBackgroundTask == eBad ||
 #endif
         stABI == eBad || stProcess == eBad) {
       continue;
@@ -656,10 +665,4 @@ void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
           mgrcx, line, argv);
     }
   }
-
-  for (uint32_t i = 0; i < contracts.Length(); ++i) {
-    CachedDirective& d = contracts[i];
-    nsComponentManagerImpl::gComponentManager->ManifestContract(mgrcx, d.lineno,
-                                                                d.argv);
-  }
 }
