@@ -3,24 +3,27 @@ import os
 import re
 import struct
 from collections import defaultdict
-
 from uuid import UUID
 
+import buildconfig
 from mozbuild.util import FileAvoidWrite
 from perfecthash import PerfectHash
-import buildconfig
-
 
 NO_CONTRACT_ID = 0xFFFFFFFF
 
 PHF_SIZE = 512
 
-ENDIAN = "<" if buildconfig.substs["TARGET_ENDIANNESS"] == "little" else ">"
+TINY_PHF_SIZE = 16
+
+# In tests, we might not have a (complete) buildconfig.
+ENDIAN = (
+    "<" if buildconfig.substs.get("TARGET_ENDIANNESS", "little") == "little" else ">"
+)
 
 
 # Represents a UUID in the format used internally by Gecko, and supports
 # serializing it in that format to both C++ source and raw byte arrays.
-class UUIDRepr(object):
+class UUIDRepr:
     def __init__(self, uuid):
         self.uuid = uuid
 
@@ -53,13 +56,15 @@ class UUIDRepr(object):
 # values don't matter, since the code generator emits symbolic constants for
 # these values, but we use the same values as the enum constants for clarity.
 class ProcessSelector:
-    ANY_PROCESS = 0x0
-    MAIN_PROCESS_ONLY = 0x1
-    CONTENT_PROCESS_ONLY = 0x2
-    ALLOW_IN_GPU_PROCESS = 0x4
-    ALLOW_IN_VR_PROCESS = 0x8
-    ALLOW_IN_SOCKET_PROCESS = 0x10
-    ALLOW_IN_RDD_PROCESS = 0x20
+    ANY_PROCESS = 0
+    MAIN_PROCESS_ONLY = 1 << 0
+    CONTENT_PROCESS_ONLY = 1 << 1
+    ALLOW_IN_GPU_PROCESS = 1 << 2
+    ALLOW_IN_VR_PROCESS = 1 << 3
+    ALLOW_IN_SOCKET_PROCESS = 1 << 4
+    ALLOW_IN_RDD_PROCESS = 1 << 5
+    ALLOW_IN_UTILITY_PROCESS = 1 << 6
+    ALLOW_IN_GMPLUGIN_PROCESS = 1 << 7
     ALLOW_IN_GPU_AND_MAIN_PROCESS = ALLOW_IN_GPU_PROCESS | MAIN_PROCESS_ONLY
     ALLOW_IN_GPU_AND_SOCKET_PROCESS = ALLOW_IN_GPU_PROCESS | ALLOW_IN_SOCKET_PROCESS
     ALLOW_IN_GPU_AND_VR_PROCESS = ALLOW_IN_GPU_PROCESS | ALLOW_IN_VR_PROCESS
@@ -70,12 +75,33 @@ class ProcessSelector:
     ALLOW_IN_GPU_RDD_AND_SOCKET_PROCESS = (
         ALLOW_IN_GPU_PROCESS | ALLOW_IN_RDD_PROCESS | ALLOW_IN_SOCKET_PROCESS
     )
+    ALLOW_IN_GPU_RDD_SOCKET_AND_UTILITY_PROCESS = (
+        ALLOW_IN_GPU_PROCESS
+        | ALLOW_IN_RDD_PROCESS
+        | ALLOW_IN_SOCKET_PROCESS
+        | ALLOW_IN_UTILITY_PROCESS
+    )
     ALLOW_IN_GPU_RDD_VR_AND_SOCKET_PROCESS = (
         ALLOW_IN_GPU_PROCESS
         | ALLOW_IN_RDD_PROCESS
         | ALLOW_IN_VR_PROCESS
         | ALLOW_IN_SOCKET_PROCESS
     )
+    ALLOW_IN_GPU_RDD_VR_SOCKET_AND_UTILITY_PROCESS = (
+        ALLOW_IN_GPU_PROCESS
+        | ALLOW_IN_RDD_PROCESS
+        | ALLOW_IN_VR_PROCESS
+        | ALLOW_IN_SOCKET_PROCESS
+        | ALLOW_IN_UTILITY_PROCESS
+    )
+    ALLOW_IN_GPU_RDD_VR_SOCKET_UTILITY_AND_GMPLUGIN_PROCESS = (
+        ALLOW_IN_GPU_PROCESS
+        | ALLOW_IN_RDD_PROCESS
+        | ALLOW_IN_VR_PROCESS
+        | ALLOW_IN_SOCKET_PROCESS
+        | ALLOW_IN_UTILITY_PROCESS
+        | ALLOW_IN_GMPLUGIN_PROCESS
+    )
 
 
 # Maps ProcessSelector constants to the name of the corresponding
@@ -94,7 +120,10 @@ PROCESSES = {
     ProcessSelector.ALLOW_IN_GPU_VR_AND_SOCKET_PROCESS: "ALLOW_IN_GPU_VR_AND_SOCKET_PROCESS",
     ProcessSelector.ALLOW_IN_RDD_AND_SOCKET_PROCESS: "ALLOW_IN_RDD_AND_SOCKET_PROCESS",
     ProcessSelector.ALLOW_IN_GPU_RDD_AND_SOCKET_PROCESS: "ALLOW_IN_GPU_RDD_AND_SOCKET_PROCESS",
+    ProcessSelector.ALLOW_IN_GPU_RDD_SOCKET_AND_UTILITY_PROCESS: "ALLOW_IN_GPU_RDD_SOCKET_AND_UTILITY_PROCESS",  # NOQA: E501
     ProcessSelector.ALLOW_IN_GPU_RDD_VR_AND_SOCKET_PROCESS: "ALLOW_IN_GPU_RDD_VR_AND_SOCKET_PROCESS",  # NOQA: E501
+    ProcessSelector.ALLOW_IN_GPU_RDD_VR_SOCKET_AND_UTILITY_PROCESS: "ALLOW_IN_GPU_RDD_VR_SOCKET_AND_UTILITY_PROCESS",  # NOQA: E501
+    ProcessSelector.ALLOW_IN_GPU_RDD_VR_SOCKET_UTILITY_AND_GMPLUGIN_PROCESS: "ALLOW_IN_GPU_RDD_VR_SOCKET_UTILITY_AND_GMPLUGIN_PROCESS",  # NOQA: E501
 }
 
 
@@ -108,10 +137,32 @@ def lower_module_id(module):
     return "ModuleID::%s" % module.name
 
 
+# Corresponds to the Module::BackgroundTasksSelector enum in Module.h. The
+# actual values don't matter, since the code generator emits symbolic constants
+# for these values, but we use the same values as the enum constants for
+# clarity.
+class BackgroundTasksSelector:
+    NO_TASKS = 0x0
+    ALL_TASKS = 0xFFFF
+
+
+# Maps BackgroundTasksSelector constants to the name of the corresponding
+# Module::BackgroundTasksSelector enum value.
+BACKGROUNDTASKS = {
+    BackgroundTasksSelector.ALL_TASKS: "ALL_TASKS",
+    BackgroundTasksSelector.NO_TASKS: "NO_TASKS",
+}
+
+
+# Emits the C++ symbolic constant corresponding to a BackgroundTasks constant.
+def lower_backgroundtasks(backgroundtasks):
+    return "Module::BackgroundTasksSelector::%s" % BACKGROUNDTASKS[backgroundtasks]
+
+
 # Represents a static string table, indexed by offset. This allows us to
 # reference strings from static data structures without requiring runtime
 # relocations.
-class StringTable(object):
+class StringTable:
     def __init__(self):
         self.entries = {}
         self.entry_list = []
@@ -171,7 +222,7 @@ interfaces = []
 # Represents a C++ namespace, containing a set of classes and potentially
 # sub-namespaces. This is used to generate pre-declarations for incomplete
 # types referenced in XPCOM manifests.
-class Namespace(object):
+class Namespace:
     def __init__(self, name=None):
         self.name = name
         self.classes = set()
@@ -205,7 +256,7 @@ class Namespace(object):
 
 
 # Represents a component defined in an XPCOM manifest's `Classes` array.
-class ModuleEntry(object):
+class ModuleEntry:
     next_anon_id = 0
 
     def __init__(self, data, init_idx):
@@ -240,7 +291,7 @@ class ModuleEntry(object):
         self.legacy_constructor = data.get("legacy_constructor", None)
         self.init_method = data.get("init_method", [])
 
-        self.jsm = data.get("jsm", None)
+        self.esModule = data.get("esModule", None)
 
         self.external = data.get(
             "external", not (self.headers or self.legacy_constructor)
@@ -248,6 +299,8 @@ class ModuleEntry(object):
         self.singleton = data.get("singleton", False)
         self.overridable = data.get("overridable", False)
 
+        self.protocol_config = data.get("protocol_config", None)
+
         if "name" in data:
             self.anonymous = False
             self.name = data["name"]
@@ -262,7 +315,7 @@ class ModuleEntry(object):
                 % (str(self.cid), ", ".join(map(repr, self.contract_ids)), str_)
             )
 
-        if self.jsm:
+        if self.esModule:
             if not self.constructor:
                 error("JavaScript components must specify a constructor")
 
@@ -311,34 +364,23 @@ class ModuleEntry(object):
             else "{ 0x%x }" % NO_CONTRACT_ID
         )
 
-        return """
-        /* {name} */ {{
-          /* {{{cid_string}}} */
-          {cid},
+        return f"""
+        /* {self.name} */ {{
+          /* {{{str(self.cid)}}} */
+          {self.cid.to_cxx()},
           {contract_id},
-          {processes},
-        }}""".format(
-            name=self.name,
-            cid=self.cid.to_cxx(),
-            cid_string=str(self.cid),
-            contract_id=contract_id,
-            processes=lower_processes(self.processes),
-        )
+          {lower_processes(self.processes)},
+        }}"""
 
-    # Generates the C++ code for a JSServiceEntry represengin this module.
+    # Generates the C++ code for a JSServiceEntry representing this module.
     def lower_js_service(self):
-        return """
+        return f"""
         {{
-          {js_name},
-          ModuleID::{name},
-          {{ {iface_offset} }},
-          {iface_count}
-        }}""".format(
-            js_name=strings.entry_to_cxx(self.js_name),
-            name=self.name,
-            iface_offset=self.interfaces_offset,
-            iface_count=len(self.interfaces),
-        )
+          {strings.entry_to_cxx(self.js_name)},
+          ModuleID::{self.name},
+          {{ {self.interfaces_offset} }},
+          {len(self.interfaces)}
+        }}"""
 
     # Generates the C++ code necessary to construct an instance of this
     # component.
@@ -359,18 +401,18 @@ class ModuleEntry(object):
 
         if self.legacy_constructor:
             res += (
-                "      return /* legacy */ %s(nullptr, aIID, aResult);\n"
+                "      return /* legacy */ %s(aIID, aResult);\n"
                 % self.legacy_constructor
             )
             return res
 
-        if self.jsm:
+        if self.esModule:
             res += (
                 "      nsCOMPtr<nsISupports> inst;\n"
-                "      MOZ_TRY(ConstructJSMComponent(nsLiteralCString(%s),\n"
-                "                                    %s,\n"
-                "                                    getter_AddRefs(inst)));"
-                "\n" % (json.dumps(self.jsm), json.dumps(self.constructor))
+                "      MOZ_TRY(ConstructESModuleComponent(nsLiteralCString(%s),\n"
+                "                                         %s,\n"
+                "                                         getter_AddRefs(inst)));"
+                "\n" % (json.dumps(self.esModule), json.dumps(self.constructor))
             )
         elif self.external:
             res += (
@@ -461,6 +503,57 @@ static inline ::mozilla::xpcom::CreateInstanceHelper Create(nsresult* aRv = null
 
         return res
 
+    # Generates the rust code for the `xpcom::components::<name>` entry
+    # corresponding to this component. This may not be called for modules
+    # without an explicit `name` (in which cases, `self.anonymous` will be
+    # true).
+    def lower_getters_rust(self):
+        assert not self.anonymous
+
+        substs = {
+            "name": self.name,
+            "id": "super::ModuleID::%s" % self.name,
+        }
+
+        res = (
+            """
+#[allow(non_snake_case)]
+pub mod %(name)s {
+    /// Get the singleton service instance for this component.
+    pub fn service<T: crate::XpCom>() -> Result<crate::RefPtr<T>, nserror::nsresult> {
+        let mut ga = crate::GetterAddrefs::<T>::new();
+        let rv = unsafe { super::Gecko_GetServiceByModuleID(%(id)s, &T::IID, ga.void_ptr()) };
+        if rv.failed() {
+            return Err(rv);
+        }
+        ga.refptr().ok_or(nserror::NS_ERROR_NO_INTERFACE)
+    }
+"""
+            % substs
+        )
+
+        if not self.singleton:
+            res += (
+                """
+    /// Create a new instance of this component.
+    pub fn create<T: crate::XpCom>() -> Result<crate::RefPtr<T>, nserror::nsresult> {
+        let mut ga = crate::GetterAddrefs::<T>::new();
+        let rv = unsafe { super::Gecko_CreateInstanceByModuleID(%(id)s, &T::IID, ga.void_ptr()) };
+        if rv.failed() {
+            return Err(rv);
+        }
+        ga.refptr().ok_or(nserror::NS_ERROR_NO_INTERFACE)
+    }
+"""
+                % substs
+            )
+
+        res += """\
+}
+"""
+
+        return res
+
 
 # Returns a quoted string literal representing the given raw string, with
 # certain special characters replaced so that it can be used in a C++-style
@@ -471,19 +564,54 @@ def pretty_string(string):
 
 # Represents a static contract ID entry, corresponding to a C++ ContractEntry
 # struct, mapping a contract ID to a static module entry.
-class ContractEntry(object):
+class ContractEntry:
     def __init__(self, contract, module):
         self.contract = contract
         self.module = module
 
+    def to_cxx(self):
+        return f"""
+        {{
+          {strings.entry_to_cxx(self.contract)},
+          {lower_module_id(self.module)},
+        }}"""
+
+
+# Represents a static ProtocolHandler entry, corresponding to a C++
+# ProtocolEntry struct, mapping a scheme to a static module entry and metadata.
+class ProtocolHandler:
+    def __init__(self, config, module):
+        def error(str_):
+            raise Exception(
+                "Error defining protocol handler %s (%s): %s"
+                % (str(module.cid), ", ".join(map(repr, module.contract_ids)), str_)
+            )
+
+        self.module = module
+        self.scheme = config.get("scheme", None)
+        if self.scheme is None:
+            error("No scheme defined for protocol component")
+        self.flags = config.get("flags", None)
+        if self.flags is None:
+            error("No flags defined for protocol component")
+        self.default_port = config.get("default_port", -1)
+        self.has_dynamic_flags = config.get("has_dynamic_flags", False)
+
     def to_cxx(self):
         return """
         {{
-          {contract},
-          {module_id},
-        }}""".format(
-            contract=strings.entry_to_cxx(self.contract),
+          .mScheme = {scheme},
+          .mProtocolFlags = {flags},
+          .mDefaultPort = {default_port},
+          .mModuleID = {module_id},
+          .mHasDynamicFlags = {has_dynamic_flags},
+        }}
+        """.format(
+            scheme=strings.entry_to_cxx(self.scheme),
             module_id=lower_module_id(self.module),
+            flags=" | ".join("nsIProtocolHandler::%s" % flag for flag in self.flags),
+            default_port=self.default_port,
+            has_dynamic_flags="true" if self.has_dynamic_flags else "false",
         )
 
 
@@ -495,7 +623,11 @@ def gen_categories(substs, categories):
 
     count = 0
     for category, entries in sorted(categories.items()):
-        entries.sort()
+
+        def k(entry):
+            return tuple(entry[0]["name"]) + entry[1:]
+
+        entries.sort(key=k)
 
         cats.append(
             "  { %s,\n"
@@ -505,13 +637,20 @@ def gen_categories(substs, categories):
 
         ents.append("  /* %s */\n" % pretty_string(category))
         for entry, value, processes in entries:
+            name = entry["name"]
+            backgroundtasks = entry.get(
+                "backgroundtasks", BackgroundTasksSelector.NO_TASKS
+            )
+
             ents.append(
                 "  { %s,\n"
                 "    %s,\n"
+                "    %s,\n"
                 "    %s },\n"
                 % (
-                    strings.entry_to_cxx(entry),
+                    strings.entry_to_cxx(name),
                     strings.entry_to_cxx(value),
+                    lower_backgroundtasks(backgroundtasks),
                     lower_processes(processes),
                 )
             )
@@ -586,13 +725,11 @@ def gen_constructors(entries):
     constructors = []
     for entry in entries:
         constructors.append(
-            """\
-    case {id}: {{
-{constructor}\
+            f"""\
+    case {lower_module_id(entry)}: {{
+{entry.lower_constructor()}\
     }}
-""".format(
-                id=lower_module_id(entry), constructor=entry.lower_constructor()
-            )
+"""
         )
 
     return "".join(constructors)
@@ -607,6 +744,17 @@ def gen_getters(entries):
     return "".join(entry.lower_getters() for entry in entries if not entry.anonymous)
 
 
+# Generates the rust getter code for each named component entry in the
+# `xpcom::components::` module.
+def gen_getters_rust(entries):
+    entries = list(entries)
+    entries.sort(key=lambda e: e.name)
+
+    return "".join(
+        entry.lower_getters_rust() for entry in entries if not entry.anonymous
+    )
+
+
 def gen_includes(substs, all_headers):
     headers = set()
     absolute_headers = set()
@@ -626,10 +774,34 @@ def gen_includes(substs, all_headers):
     substs["relative_includes"] = "\n".join(relative_includes) + "\n"
 
 
-def to_list(val):
+def to_category_list(val):
+    # Entries can be bare strings (like `"m-browser"`), lists of bare strings,
+    # or dictionaries (like `{"name": "m-browser", "backgroundtasks":
+    # BackgroundTasksSelector.ALL_TASKS}`), somewhat recursively.
+
+    def ensure_dict(v):
+        # Turn `v` into `{"name": v}` if it's not already a dict.
+        if isinstance(v, dict):
+            return v
+        return {"name": v}
+
     if isinstance(val, (list, tuple)):
-        return val
-    return (val,)
+        return tuple(ensure_dict(v) for v in val)
+
+    if isinstance(val, dict):
+        # Explode `{"name": ["x", "y"], "backgroundtasks": ...}` into
+        # `[{"name": "x", "backgroundtasks": ...}, {"name": "y", "backgroundtasks": ...}]`.
+        names = val.pop("name")
+
+        vals = []
+        for entry in to_category_list(names):
+            d = dict(val)
+            d["name"] = entry["name"]
+            vals.append(d)
+
+        return tuple(vals)
+
+    return (ensure_dict(val),)
 
 
 def gen_substs(manifests):
@@ -659,14 +831,15 @@ def gen_substs(manifests):
                     value, process = entry
                 else:
                     value, process = entry, 0
-                categories[category].append((key, value, process))
+                categories[category].append(({"name": key}, value, process))
 
     cids = set()
     contracts = []
     contract_map = {}
     js_services = {}
+    protocol_handlers = {}
 
-    jsms = set()
+    esModules = set()
 
     types = set()
 
@@ -682,30 +855,44 @@ def gen_substs(manifests):
             contract_map[contract_id] = entry
 
         for category, entries in mod.categories.items():
-            for entry in to_list(entries):
+            for entry in to_category_list(entries):
                 categories[category].append((entry, mod.contract_id, mod.processes))
 
         if mod.type and not mod.headers:
             types.add(mod.type)
 
-        if mod.jsm:
-            jsms.add(mod.jsm)
+        if mod.esModule:
+            esModules.add(mod.esModule)
 
         if mod.js_name:
             if mod.js_name in js_services:
                 raise Exception("Duplicate JS service name: %s" % mod.js_name)
             js_services[mod.js_name] = mod
 
+        if mod.protocol_config:
+            handler = ProtocolHandler(mod.protocol_config, mod)
+            if handler.scheme in protocol_handlers:
+                raise Exception("Duplicate protocol handler: %s" % handler.scheme)
+            protocol_handlers[handler.scheme] = handler
+
         if str(mod.cid) in cids:
             raise Exception("Duplicate cid: %s" % str(mod.cid))
         cids.add(str(mod.cid))
 
     cid_phf = PerfectHash(modules, PHF_SIZE, key=lambda module: module.cid.bytes)
 
-    contract_phf = PerfectHash(contracts, PHF_SIZE, key=lambda entry: entry.contract)
+    contract_phf = PerfectHash(
+        contracts, PHF_SIZE, key=lambda entry: entry.contract.encode()
+    )
 
     js_services_phf = PerfectHash(
-        list(js_services.values()), PHF_SIZE, key=lambda entry: entry.js_name
+        list(js_services.values()), PHF_SIZE, key=lambda entry: entry.js_name.encode()
+    )
+
+    protocol_handlers_phf = PerfectHash(
+        list(protocol_handlers.values()),
+        TINY_PHF_SIZE,
+        key=lambda entry: entry.scheme.encode(),
     )
 
     js_services_json = {}
@@ -721,13 +908,19 @@ def gen_substs(manifests):
 
     substs["module_count"] = len(modules)
     substs["contract_count"] = len(contracts)
+    substs["protocol_handler_count"] = len(protocol_handlers)
+
+    substs["default_protocol_handler_idx"] = protocol_handlers_phf.get_index(b"default")
 
     gen_module_funcs(substs, module_funcs)
 
     gen_includes(substs, headers)
 
-    substs["component_jsms"] = (
-        "\n".join(" %s," % strings.entry_to_cxx(jsm) for jsm in sorted(jsms)) + "\n"
+    substs["component_esmodules"] = (
+        "\n".join(
+            " %s," % strings.entry_to_cxx(esModule) for esModule in sorted(esModules)
+        )
+        + "\n"
     )
 
     substs["interfaces"] = gen_interfaces(interfaces)
@@ -738,6 +931,8 @@ def gen_substs(manifests):
 
     substs["component_getters"] = gen_getters(cid_phf.entries)
 
+    substs["component_getters_rust"] = gen_getters_rust(cid_phf.entries)
+
     substs["module_cid_table"] = cid_phf.cxx_codegen(
         name="ModuleByCID",
         entry_type="StaticModule",
@@ -776,7 +971,21 @@ def gen_substs(manifests):
         key_length="aKey.Length()",
     )
 
-    substs["js_services_json"] = json.dumps(js_services_json, sort_keys=True, indent=4)
+    substs["protocol_handlers_table"] = protocol_handlers_phf.cxx_codegen(
+        name="LookupProtocolHandler",
+        entry_type="StaticProtocolHandler",
+        entries_name="gStaticProtocolHandlers",
+        lower_entry=lambda entry: entry.to_cxx(),
+        return_type="const StaticProtocolHandler*",
+        return_entry="return entry.Scheme() == aKey ? &entry : nullptr;",
+        key_type="const nsACString&",
+        key_bytes="aKey.BeginReading()",
+        key_length="aKey.Length()",
+    )
+
+    substs["js_services_json"] = (
+        json.dumps(js_services_json, sort_keys=True, indent=2) + "\n"
+    )
 
     # Do this only after everything else has been emitted so we're sure the
     # string table is complete.
@@ -794,8 +1003,10 @@ def read_manifest(filename):
         "buildconfig": buildconfig,
         "defined": defined,
         "ProcessSelector": ProcessSelector,
+        "BackgroundTasksSelector": BackgroundTasksSelector,
     }
-    exec(open(filename).read(), glbl)
+    code = compile(open(filename).read(), filename, "exec")
+    exec(code, glbl)
     return glbl
 
 
@@ -803,7 +1014,7 @@ def main(fd, conf_file, template_file):
     def open_output(filename):
         return FileAvoidWrite(os.path.join(os.path.dirname(fd.name), filename))
 
-    conf = json.load(open(conf_file, "r"))
+    conf = json.load(open(conf_file))
 
     deps = set()
 
@@ -823,7 +1034,7 @@ def main(fd, conf_file, template_file):
         return substs[match.group(1)]
 
     with open_output("StaticComponents.cpp") as fh:
-        with open(template_file, "r") as tfh:
+        with open(template_file) as tfh:
             template = tfh.read()
 
         fh.write(re.sub(r"//# @([a-zA-Z_]+)@\n", replacer, template))
@@ -840,6 +1051,8 @@ def main(fd, conf_file, template_file):
 #ifndef StaticComponentData_h
 #define StaticComponentData_h
 
+#include <stddef.h>
+
 namespace mozilla {
 namespace xpcom {
 
@@ -851,10 +1064,28 @@ static constexpr size_t kStaticCategoryCount = %(category_count)d;
 
 static constexpr size_t kModuleInitCount = %(init_count)d;
 
+static constexpr size_t kStaticProtocolHandlerCount = %(protocol_handler_count)d;
+
+static constexpr size_t kDefaultProtocolHandlerIndex = %(default_protocol_handler_idx)d;
+
 }  // namespace xpcom
 }  // namespace mozilla
 
 #endif
+"""
+            % substs
+        )
+
+    with open_output("components.rs") as fh:
+        fh.write(
+            """\
+/// Unique IDs for each statically-registered module.
+#[repr(u16)]
+pub enum ModuleID {
+%(module_ids)s
+}
+
+%(component_getters_rust)s
 """
             % substs
         )
@@ -888,6 +1119,10 @@ enum class ModuleID : uint16_t {
 %(module_ids)s
 };
 
+// May be added as a friend function to allow constructing services via
+// private constructors and init methods.
+nsresult CreateInstanceImpl(ModuleID aID, const nsIID& aIID, void** aResult);
+
 class MOZ_STACK_CLASS StaticModuleHelper : public nsCOMPtr_helper {
  public:
   StaticModuleHelper(ModuleID aId, nsresult* aErrorPtr)
