@@ -5,14 +5,19 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "gtest/gtest.h"
+#include "mozilla/gtest/MozAssertions.h"
+#include "mozilla/ipc/DataPipe.h"
 #include "mozilla/SpinEventLoopUntil.h"
 #include "nsIAsyncInputStream.h"
 #include "nsComponentManagerUtils.h"
+#include "nsIAsyncOutputStream.h"
 #include "nsIInputStream.h"
 #include "nsIMultiplexInputStream.h"
+#include "nsIPipe.h"
 #include "nsISeekableStream.h"
 #include "nsStreamUtils.h"
 #include "nsThreadUtils.h"
+#include "nsIThread.h"
 #include "Helpers.h"
 
 using mozilla::GetCurrentSerialEventTarget;
@@ -31,11 +36,11 @@ TEST(MultiplexInputStream, Seek_SET)
   nsCOMPtr<nsIInputStream> inputStream2;
   nsCOMPtr<nsIInputStream> inputStream3;
   nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputStream1), buf1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = NS_NewCStringInputStream(getter_AddRefs(inputStream2), buf2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = NS_NewCStringInputStream(getter_AddRefs(inputStream3), buf3);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCOMPtr<nsIMultiplexInputStream> multiplexStream =
       do_CreateInstance("@mozilla.org/io/multiplex-input-stream;1");
@@ -44,11 +49,11 @@ TEST(MultiplexInputStream, Seek_SET)
   ASSERT_TRUE(stream);
 
   rv = multiplexStream->AppendStream(inputStream1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = multiplexStream->AppendStream(inputStream2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = multiplexStream->AppendStream(inputStream3);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   int64_t tell;
   uint64_t length;
@@ -59,72 +64,72 @@ TEST(MultiplexInputStream, Seek_SET)
 
   // Seek forward in first input stream
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_SET, 1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf1.Length() + buf2.Length() + buf3.Length() - 1,
             length);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 1);
 
   // Check read is correct
   rv = stream->Read(readBuf, 3, &count);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)3, count);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf1.Length() + buf2.Length() + buf3.Length() - 4,
             length);
   ASSERT_EQ(0, strncmp(readBuf, "ell", count));
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 4);
 
   // Seek to start of third input stream
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_SET,
                         buf1.Length() + buf2.Length());
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf3.Length(), length);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(tell, buf1.Length() + buf2.Length());
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(tell, int64_t(buf1.Length() + buf2.Length()));
 
   // Check read is correct
   rv = stream->Read(readBuf, 5, &count);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)5, count);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf3.Length() - 5, length);
   ASSERT_EQ(0, strncmp(readBuf, "Foo b", count));
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(tell, buf1.Length() + buf2.Length() + 5);
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(tell, int64_t(buf1.Length() + buf2.Length() + 5));
 
   // Seek back to start of second stream (covers bug 1272371)
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_SET, buf1.Length());
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf2.Length() + buf3.Length(), length);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(tell, buf1.Length());
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(tell, int64_t(buf1.Length()));
 
   // Check read is correct
   rv = stream->Read(readBuf, 6, &count);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)6, count);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf2.Length() - 6 + buf3.Length(), length);
   ASSERT_EQ(0, strncmp(readBuf, "The qu", count));
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(tell, buf1.Length() + 6);
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(tell, int64_t(buf1.Length() + 6));
 }
 
 TEST(MultiplexInputStream, Seek_CUR)
@@ -140,11 +145,11 @@ TEST(MultiplexInputStream, Seek_CUR)
   nsCOMPtr<nsIInputStream> inputStream2;
   nsCOMPtr<nsIInputStream> inputStream3;
   nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputStream1), buf1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = NS_NewCStringInputStream(getter_AddRefs(inputStream2), buf2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = NS_NewCStringInputStream(getter_AddRefs(inputStream3), buf3);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCOMPtr<nsIMultiplexInputStream> multiplexStream =
       do_CreateInstance("@mozilla.org/io/multiplex-input-stream;1");
@@ -153,11 +158,11 @@ TEST(MultiplexInputStream, Seek_CUR)
   ASSERT_TRUE(stream);
 
   rv = multiplexStream->AppendStream(inputStream1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = multiplexStream->AppendStream(inputStream2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = multiplexStream->AppendStream(inputStream3);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   int64_t tell;
   uint64_t length;
@@ -168,54 +173,54 @@ TEST(MultiplexInputStream, Seek_CUR)
 
   // Seek forward in first input stream
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_CUR, 1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf1.Length() + buf2.Length() + buf3.Length() - 1,
             length);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 1);
 
   // Check read is correct
   rv = stream->Read(readBuf, 3, &count);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)3, count);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)buf1.Length() + buf2.Length() + buf3.Length() - 4,
             length);
   ASSERT_EQ(0, strncmp(readBuf, "ell", count));
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 4);
 
   // Let's go to the second stream
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_CUR, 11);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 15);
   rv = stream->Read(readBuf, 3, &count);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)3, count);
   ASSERT_EQ(0, strncmp(readBuf, "qui", count));
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 18);
 
   // Let's go back to the first stream
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_CUR, -9);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 9);
   rv = stream->Read(readBuf, 3, &count);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)3, count);
   ASSERT_EQ(0, strncmp(readBuf, "ldT", count));
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 12);
 }
 
@@ -232,11 +237,11 @@ TEST(MultiplexInputStream, Seek_END)
   nsCOMPtr<nsIInputStream> inputStream2;
   nsCOMPtr<nsIInputStream> inputStream3;
   nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputStream1), buf1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = NS_NewCStringInputStream(getter_AddRefs(inputStream2), buf2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = NS_NewCStringInputStream(getter_AddRefs(inputStream3), buf3);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCOMPtr<nsIMultiplexInputStream> multiplexStream =
       do_CreateInstance("@mozilla.org/io/multiplex-input-stream;1");
@@ -245,11 +250,11 @@ TEST(MultiplexInputStream, Seek_END)
   ASSERT_TRUE(stream);
 
   rv = multiplexStream->AppendStream(inputStream1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = multiplexStream->AppendStream(inputStream2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = multiplexStream->AppendStream(inputStream3);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   int64_t tell;
   uint64_t length;
@@ -258,27 +263,27 @@ TEST(MultiplexInputStream, Seek_END)
 
   // SEEK_END wants <=0 values
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_END, 1);
-  ASSERT_TRUE(NS_FAILED(rv));
+  ASSERT_NS_FAILED(rv);
 
   // Let's go to the end.
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_END, 0);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)0, length);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(tell, buf1.Length() + buf2.Length() + buf3.Length());
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(tell, int64_t(buf1.Length() + buf2.Length() + buf3.Length()));
 
   // -1
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_END, -1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ((uint64_t)1, length);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(tell, buf1.Length() + buf2.Length() + buf3.Length() - 1);
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(tell, int64_t(buf1.Length() + buf2.Length() + buf3.Length() - 1));
 
   // Almost at the beginning
   tell = 1;
@@ -286,12 +291,12 @@ TEST(MultiplexInputStream, Seek_END)
   tell -= buf2.Length();
   tell -= buf3.Length();
   rv = seekStream->Seek(nsISeekableStream::NS_SEEK_END, tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   rv = stream->Available(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(length, buf1.Length() + buf2.Length() + buf3.Length() - 1);
   rv = seekStream->Tell(&tell);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(tell, 1);
 }
 
@@ -351,7 +356,9 @@ TEST(MultiplexInputStream, AsyncWait_withEventTarget)
   ASSERT_FALSE(cb->Called());
 
   // Eventually it is called.
-  MOZ_ALWAYS_TRUE(mozilla::SpinEventLoopUntil([&]() { return cb->Called(); }));
+  MOZ_ALWAYS_TRUE(mozilla::SpinEventLoopUntil(
+      "xpcom:TEST(MultiplexInputStream, AsyncWait_withEventTarget)"_ns,
+      [&]() { return cb->Called(); }));
   ASSERT_TRUE(cb->Called());
 }
 
@@ -392,7 +399,9 @@ TEST(MultiplexInputStream, AsyncWait_withEventTarget_closureOnly)
   ASSERT_FALSE(cb->Called());
 
   // Eventually it is called.
-  MOZ_ALWAYS_TRUE(mozilla::SpinEventLoopUntil([&]() { return cb->Called(); }));
+  MOZ_ALWAYS_TRUE(mozilla::SpinEventLoopUntil(
+      "xpcom:TEST(MultiplexInputStream, AsyncWait_withEventTarget_closureOnly)"_ns,
+      [&]() { return cb->Called(); }));
   ASSERT_TRUE(cb->Called());
 }
 
@@ -405,6 +414,9 @@ class ClosedStream final : public nsIInputStream {
   NS_IMETHOD
   Available(uint64_t* aLength) override { return NS_BASE_STREAM_CLOSED; }
 
+  NS_IMETHOD
+  StreamStatus() override { return NS_BASE_STREAM_CLOSED; }
+
   NS_IMETHOD
   Read(char* aBuffer, uint32_t aCount, uint32_t* aReadCount) override {
     MOZ_CRASH("This should not be called!");
@@ -446,6 +458,11 @@ class AsyncStream final : public nsIAsyncInputStream {
     return mState == eClosed ? NS_BASE_STREAM_CLOSED : NS_OK;
   }
 
+  NS_IMETHOD
+  StreamStatus() override {
+    return mState == eClosed ? NS_BASE_STREAM_CLOSED : NS_OK;
+  }
+
   NS_IMETHOD
   Read(char* aBuffer, uint32_t aCount, uint32_t* aReadCount) override {
     MOZ_CRASH("This should not be called!");
@@ -499,6 +516,9 @@ class BlockingStream final : public nsIInputStream {
   NS_IMETHOD
   Available(uint64_t* aLength) override { return NS_BASE_STREAM_CLOSED; }
 
+  NS_IMETHOD
+  StreamStatus() override { return NS_BASE_STREAM_CLOSED; }
+
   NS_IMETHOD
   Read(char* aBuffer, uint32_t aCount, uint32_t* aReadCount) override {
     // We are actually empty.
@@ -600,6 +620,9 @@ class NonBufferableStringStream final : public nsIInputStream {
   NS_IMETHOD
   Available(uint64_t* aLength) override { return mStream->Available(aLength); }
 
+  NS_IMETHOD
+  StreamStatus() override { return mStream->StreamStatus(); }
+
   NS_IMETHOD
   Read(char* aBuffer, uint32_t aCount, uint32_t* aReadCount) override {
     return mStream->Read(aBuffer, aCount, aReadCount);
@@ -637,17 +660,17 @@ TEST(MultiplexInputStream, Bufferable)
   buf1.AssignLiteral("Hello ");
   nsCOMPtr<nsIInputStream> inputStream1;
   nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputStream1), buf1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCString buf2;
   buf2.AssignLiteral("world");
   nsCOMPtr<nsIInputStream> inputStream2 = new NonBufferableStringStream(buf2);
 
   rv = multiplexStream->AppendStream(inputStream1);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   rv = multiplexStream->AppendStream(inputStream2);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCOMPtr<nsIInputStream> stream(do_QueryInterface(multiplexStream));
   ASSERT_TRUE(!!stream);
@@ -655,7 +678,7 @@ TEST(MultiplexInputStream, Bufferable)
   char buf3[1024];
   uint32_t size = 0;
   rv = stream->ReadSegments(NS_CopySegmentToBuffer, buf3, sizeof(buf3), &size);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   ASSERT_EQ(size, buf1.Length() + buf2.Length());
   ASSERT_TRUE(!strncmp(buf3, "Hello world", size));
@@ -674,10 +697,10 @@ TEST(MultiplexInputStream, QILengthInputStream)
   {
     nsCOMPtr<nsIInputStream> inputStream;
     nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputStream), buf);
-    ASSERT_TRUE(NS_SUCCEEDED(rv));
+    ASSERT_NS_SUCCEEDED(rv);
 
     rv = multiplexStream->AppendStream(inputStream);
-    ASSERT_TRUE(NS_SUCCEEDED(rv));
+    ASSERT_NS_SUCCEEDED(rv);
 
     nsCOMPtr<nsIInputStreamLength> fsis = do_QueryInterface(multiplexStream);
     ASSERT_TRUE(!fsis);
@@ -694,7 +717,7 @@ TEST(MultiplexInputStream, QILengthInputStream)
         new testing::LengthInputStream(buf, true, false);
 
     nsresult rv = multiplexStream->AppendStream(inputStream);
-    ASSERT_TRUE(NS_SUCCEEDED(rv));
+    ASSERT_NS_SUCCEEDED(rv);
 
     nsCOMPtr<nsIInputStreamLength> fsis = do_QueryInterface(multiplexStream);
     ASSERT_TRUE(!!fsis);
@@ -711,7 +734,7 @@ TEST(MultiplexInputStream, QILengthInputStream)
         new testing::LengthInputStream(buf, true, true);
 
     nsresult rv = multiplexStream->AppendStream(inputStream);
-    ASSERT_TRUE(NS_SUCCEEDED(rv));
+    ASSERT_NS_SUCCEEDED(rv);
 
     nsCOMPtr<nsIInputStreamLength> fsis = do_QueryInterface(multiplexStream);
     ASSERT_TRUE(!!fsis);
@@ -733,17 +756,17 @@ TEST(MultiplexInputStream, LengthInputStream)
 
   nsCOMPtr<nsIInputStream> inputStream;
   nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputStream), buf);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   rv = multiplexStream->AppendStream(inputStream);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   // A LengthInputStream, non-async.
   RefPtr<testing::LengthInputStream> lengthStream =
       new testing::LengthInputStream(buf, true, false);
 
   rv = multiplexStream->AppendStream(lengthStream);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCOMPtr<nsIInputStreamLength> fsis = do_QueryInterface(multiplexStream);
   ASSERT_TRUE(!!fsis);
@@ -751,8 +774,8 @@ TEST(MultiplexInputStream, LengthInputStream)
   // Size is the sum of the 2 streams.
   int64_t length;
   rv = fsis->Length(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-  ASSERT_EQ(buf.Length() * 2, length);
+  ASSERT_NS_SUCCEEDED(rv);
+  ASSERT_EQ(int64_t(buf.Length() * 2), length);
 
   // An async LengthInputStream.
   RefPtr<testing::LengthInputStream> asyncLengthStream =
@@ -760,7 +783,7 @@ TEST(MultiplexInputStream, LengthInputStream)
                                      NS_BASE_STREAM_WOULD_BLOCK);
 
   rv = multiplexStream->AppendStream(asyncLengthStream);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   nsCOMPtr<nsIAsyncInputStreamLength> afsis =
       do_QueryInterface(multiplexStream);
@@ -775,17 +798,19 @@ TEST(MultiplexInputStream, LengthInputStream)
   rv = afsis->AsyncLengthWait(callback, GetCurrentSerialEventTarget());
   ASSERT_EQ(NS_OK, rv);
 
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() { return callback->Called(); }));
-  ASSERT_EQ(buf.Length() * 3, callback->Size());
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "xpcom:TEST(MultiplexInputStream, LengthInputStream) 1"_ns,
+      [&]() { return callback->Called(); }));
+  ASSERT_EQ(int64_t(buf.Length() * 3), callback->Size());
 
   // Now a negative stream
   lengthStream = new testing::LengthInputStream(buf, true, false, NS_OK, true);
 
   rv = multiplexStream->AppendStream(lengthStream);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   rv = fsis->Length(&length);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
   ASSERT_EQ(-1, length);
 
   // Another async LengthInputStream.
@@ -793,7 +818,7 @@ TEST(MultiplexInputStream, LengthInputStream)
       buf, true, true, NS_BASE_STREAM_WOULD_BLOCK);
 
   rv = multiplexStream->AppendStream(asyncLengthStream);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   afsis = do_QueryInterface(multiplexStream);
   ASSERT_TRUE(!!afsis);
@@ -807,7 +832,127 @@ TEST(MultiplexInputStream, LengthInputStream)
   rv = afsis->AsyncLengthWait(callback2, GetCurrentSerialEventTarget());
   ASSERT_EQ(NS_OK, rv);
 
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() { return callback2->Called(); }));
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "xpcom:TEST(MultiplexInputStream, LengthInputStream) 2"_ns,
+      [&]() { return callback2->Called(); }));
   ASSERT_FALSE(callback1->Called());
   ASSERT_TRUE(callback2->Called());
 }
+
+void TestMultiplexStreamReadWhileWaiting(nsIAsyncInputStream* pipeIn,
+                                         nsIAsyncOutputStream* pipeOut) {
+  // We had an issue where a stream which was read while a message was in-flight
+  // to report the stream was ready, meaning that the stream reported 0 bytes
+  // available when checked in the MultiplexInputStream's callback, and was
+  // skipped over.
+
+  nsCOMPtr<nsIThread> mainThread = NS_GetCurrentThread();
+
+  nsCOMPtr<nsIMultiplexInputStream> multiplexStream =
+      do_CreateInstance("@mozilla.org/io/multiplex-input-stream;1");
+  ASSERT_NS_SUCCEEDED(multiplexStream->AppendStream(pipeIn));
+
+  nsCOMPtr<nsIInputStream> stringStream;
+  ASSERT_TRUE(NS_SUCCEEDED(
+      NS_NewCStringInputStream(getter_AddRefs(stringStream), "xxxx\0"_ns)));
+  ASSERT_NS_SUCCEEDED(multiplexStream->AppendStream(stringStream));
+
+  nsCOMPtr<nsIAsyncInputStream> asyncMultiplex =
+      do_QueryInterface(multiplexStream);
+  ASSERT_TRUE(asyncMultiplex);
+
+  RefPtr<testing::InputStreamCallback> cb = new testing::InputStreamCallback();
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->AsyncWait(cb, 0, 0, mainThread));
+  EXPECT_FALSE(cb->Called());
+
+  NS_ProcessPendingEvents(mainThread);
+  EXPECT_FALSE(cb->Called());
+
+  uint64_t available;
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, 0u);
+
+  // Write some data to the pipe, which should wake up the async wait message to
+  // be delivered.
+  char toWrite[] = "1234";
+  uint32_t written;
+  ASSERT_NS_SUCCEEDED(pipeOut->Write(toWrite, sizeof(toWrite), &written));
+  EXPECT_EQ(written, sizeof(toWrite));
+  EXPECT_FALSE(cb->Called());
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, sizeof(toWrite));
+
+  // Read that data from the stream
+  char toRead[sizeof(toWrite)];
+  uint32_t read;
+  ASSERT_TRUE(
+      NS_SUCCEEDED(asyncMultiplex->Read(toRead, sizeof(toRead), &read)));
+  EXPECT_EQ(read, sizeof(toRead));
+  EXPECT_STREQ(toRead, toWrite);
+  EXPECT_FALSE(cb->Called());
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, 0u);
+
+  // The multiplex stream will have detected the read and prevented the callback
+  // from having been called yet.
+  NS_ProcessPendingEvents(mainThread);
+  EXPECT_FALSE(cb->Called());
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, 0u);
+
+  // Write more data and close, then make sure we can read everything else in
+  // the stream.
+  char toWrite2[] = "56789";
+  ASSERT_TRUE(
+      NS_SUCCEEDED(pipeOut->Write(toWrite2, sizeof(toWrite2), &written)));
+  EXPECT_EQ(written, sizeof(toWrite2));
+  EXPECT_FALSE(cb->Called());
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, sizeof(toWrite2));
+
+  ASSERT_NS_SUCCEEDED(pipeOut->Close());
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  // XXX: Theoretically if the multiplex stream could detect it, we could report
+  // `sizeof(toWrite2) + 4` because the stream is complete, but there's no way
+  // for the multiplex stream to know.
+  EXPECT_EQ(available, sizeof(toWrite2));
+
+  NS_ProcessPendingEvents(mainThread);
+  EXPECT_TRUE(cb->Called());
+
+  // Read that final bit of data and make sure we read it.
+  char toRead2[sizeof(toWrite2)];
+  ASSERT_TRUE(
+      NS_SUCCEEDED(asyncMultiplex->Read(toRead2, sizeof(toRead2), &read)));
+  EXPECT_EQ(read, sizeof(toRead2));
+  EXPECT_STREQ(toRead2, toWrite2);
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, 5u);
+
+  // Read the extra data as well.
+  char extraRead[5];
+  ASSERT_TRUE(
+      NS_SUCCEEDED(asyncMultiplex->Read(extraRead, sizeof(extraRead), &read)));
+  EXPECT_EQ(read, sizeof(extraRead));
+  EXPECT_STREQ(extraRead, "xxxx");
+  ASSERT_NS_SUCCEEDED(asyncMultiplex->Available(&available));
+  EXPECT_EQ(available, 0u);
+}
+
+TEST(MultiplexInputStream, ReadWhileWaiting_nsPipe)
+{
+  nsCOMPtr<nsIAsyncInputStream> pipeIn;
+  nsCOMPtr<nsIAsyncOutputStream> pipeOut;
+  NS_NewPipe2(getter_AddRefs(pipeIn), getter_AddRefs(pipeOut), true, true);
+  TestMultiplexStreamReadWhileWaiting(pipeIn, pipeOut);
+}
+
+TEST(MultiplexInputStream, ReadWhileWaiting_DataPipe)
+{
+  RefPtr<mozilla::ipc::DataPipeReceiver> pipeIn;
+  RefPtr<mozilla::ipc::DataPipeSender> pipeOut;
+  ASSERT_TRUE(NS_SUCCEEDED(mozilla::ipc::NewDataPipe(
+      mozilla::ipc::kDefaultDataPipeCapacity, getter_AddRefs(pipeOut),
+      getter_AddRefs(pipeIn))));
+  TestMultiplexStreamReadWhileWaiting(pipeIn, pipeOut);
+}
